/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include "message_hist.hh"

cMessageHistory::cMessageHistory(void)
: list<cMsg>()
{
  auto_ptr<cMsg> msg(cMsgFactory::createStartMsg());

  add(*msg);
}

cMessageHistory::~cMessageHistory(void)
{ }

void cMessageHistory::initialize(void)
{
  clear();

  auto_ptr<cMsg> msg(cMsgFactory::createStartMsg());

  add(*msg);
}

cMessageHistory::iterator cMessageHistory::add(const cMsg &n)
{
  list<cMsg>::iterator t= end();
  find(t, n);

  // Handles double receptions (the ACK-retransmission scheme allows this)
  if (t != end() && n == *t)
    return end();

  while (t != end() && *t < n)
    t++;                        // predefined order for simultaneous events
  return insert(t, n);
}

void cMessageHistory::discard(double horizon, list<cMsg>::iterator &before)
{
  while (begin() != end() &&  (*begin()).timestamp() < horizon
      && begin() != before)
    pop_front();
}

void cMessageHistory::find(list<cMsg>::iterator &t, const cMsg &m)
{
  if (t != begin())
    for (t--; ; t--)
      {
      if ((*t) < m)
        {
        t++;
        break;
        }
    
      if (t == begin())
        break;
      }

  while (t != end() && (*t) < m)
    t++;
}

