/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/

#ifndef __MESSAGE_HIST_H
#define __MESSAGE_HIST_H

#include <map>

#include "gen.hh"
#include "msg.hh"

namespace Condel {

// forward references

/// An event history.
/// This is a simple list of events that may shape the development of a DVE.

class cMessageHistory : public list<cMsg>
{
public:
  // define iterator types
  typedef list<cMsg>::iterator iterator;


  cMessageHistory(void);
  virtual ~cMessageHistory(void);

  virtual void initialize(void);

  /// Add a new event into the queue.
  /// @param new_message A message to be added.  It will be added at the
  /// correct position.  If that event is in the queue already, it will be
  /// ignored and end() is returned.
  /// @return An iterator pointing to the newly inserted message.  If the
  /// event was in the queue already, end() is returned.
  iterator add(const cMsg& new_message);

  /// Prune the past of this history.
  /// Starting from the oldest event, this will release and discard all
  /// event while @em both the conditions are true.
  /// @param new_horizon All events older than new_horizon are discarded.
  /// @param before All events before this iterator are discarded (the
  /// event pointed to by before is kept).
  void discard(double new_horizon, list<cMsg>::iterator &before);

  /// Find the event in the queue.
  /// @param it This iterator is set to the event in the queue with the same
  /// id-data as the given one.  If that event does not exist (the usual case),
  /// the iterator returns the next event after (so if you insert with the
  /// returned it, you'll be in order).
  /// @param m A message only used for its ID data, namely cMsg::getHostID(),
  /// cMsg::timestamp(), and cMsg::getOrder().
  void find(list<cMsg>::iterator &it, const cMsg &m);

  /// The end of the list.
  /// Indicates an end-of-list.  This iterator does @em not point to a legal
  /// event.
  list<cMsg>::iterator end(void)
    { return list<cMsg>::end(); }
};
  

};

#endif // __MESSAGE_HIST_H
