/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include "world_opt.hh"
#include "host.hh"
#include "scenario.hh"
#include "message_hist.hh"
#include "instance.hh"
#include "instance_factory.hh"
#include "load.hh"



cOptWorld::cOptWorld(cHost &host, const string& name,
        cMessageHistory &hist)
: cWorld(host, name), m_history(hist), m_current(m_history.begin())
{ }

const cOptWorld &cOptWorld::operator=(const cOptWorld &o)
{
  cWorld::operator=(o);
  m_current= o.m_current;
  return *this;
}

auto_ptr<cWorld> cOptWorld::clone(void)
{
  auto_ptr<cWorld> ret(new cOptWorld(host(), getName(), m_history));
  *(cOptWorld *) ret.get() = *this;

  return ret;
}

void cOptWorld::initialize(void)
{
  clear();
  m_current= m_history.begin();
}

void cOptWorld::simulate(double newtime, cLoadMeter &load)
{
  // set current to the right spot.

  while (simTime() < newtime)
    {                                   // This loops performs tick-interrupts
    double time= cHost::roundTime(simTime()+ cHost::TICK_TIME);

    if (time> newtime)
      time= newtime;

    // Currently, the events corresponding to m_simtime are NOT yet performed.
    // Advantage:  less rerolls  Disadvantage:  No up-to-date version?
    while (simTime() < time)
      {                                 // loop over events until time reached
      cMessageHistory::iterator next= m_current;
      next++;
      if (next != m_history.end() && simTime() >= (*next).timestamp())
        execute_event(load);

      else
        {
        double nt= time;
        if ( next != m_history.end() && (*next).timestamp() < nt )
          nt= (*next).timestamp();

        host().getScenario().simulate(*this, nt, load);
        }
      load.addCPU(2);
      }
    }
}

void cOptWorld::execute_event(cLoadMeter &load)
{
  cMessageHistory::iterator next= m_current;
  next++;
  if (next == m_history.end()) return;

  m_current++;
  cMsg *msg= &(*m_current);
  if (msg->getType() != cMsg::DATA)
    return;

  if (!isReplicated(msg->getObjectID()))
    {                                           // Create new instance
    // Sometimes an update-message arrives before an AddInstance message.
    // Such messages can only be ignored for the time being (will be useful
    // lateron after backtracking).
    if (!strcmp(msg->name(),"AddInstance"))
      {
      scoped_ptr<cInstance>
        ni(cInstanceFactory::createInstance(msg->getObjectID(),
                                            msg->getObjectName()));

      ni->setAttributes(msg->getAttributes());
      addInstance(ni.get());
      load.addCPU(10);
      }
    }
  else
    {                                           // instance exists, copy state
    cInstance &ei(getInstance(msg->getObjectID()));
    cAttributeSet eas(ei.getAttributes());
    cAttributeSet &nas= msg->getAttributes();

    cAttributeSet::const_iterator it;
    for(it= nas.begin(); it!= nas.end(); it++)
      if (eas.hasAttribute((*it).first))
        eas.get((*it).first).setValue((*it).second.getValue());

    ei.setAttributes(eas);
    load.addCPU(5);
    }

  load.addCPU(2);
}





