/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
// a collection of instances, each with a unique object ID
// For use in an optimistic consistency scheme, allows updating to any
// point in the future. 

#ifndef __WORLD_OPT_H
#define __WORLD_OPT_H

#include "world.hh"
#include "message_hist.hh"


namespace Condel {

class cScenario;
class cLoadMeter;


/// A world in an optimistic setting.

class cOptWorld : public cWorld
{
protected:
  cMessageHistory &m_history;
  cMessageHistory::iterator m_current;      // The last event processed

public:
  /// The constructor.
  /// @param host The host this world is on.
  /// @param name The name of this world.
  /// @param msgHist A reference to the single message history being held
  /// in cConsistencyOpt.  This reference is kept for use in this class.
  cOptWorld(cHost &host, const string &name, cMessageHistory &msgHist);
  virtual ~cOptWorld(void) {}

  const cOptWorld &operator=(const cOptWorld &o);

  auto_ptr<cWorld> clone(void);

  void initialize(void);

  /// The last event.
  /// This is the last event that was simulated on this world.
  cMessageHistory::iterator &current(void) { return m_current; }

  /// Execute a single event.
  /// This finds the next event after current(), executes it, then sets
  /// current to it.
  void execute_event(cLoadMeter &load);

  virtual void simulate(double newtime, cLoadMeter &load);
};

};

#endif // __WORLD_OPT_H
