#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# clean the temporary storage directories on all slaves

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}

# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"
source "${CONDEL_ROOT}/experiments/common_files/akaroa.sh"

# get host list
ak_setup_vars

# check for running simulations
log "checking for active simulations..."
[ -f "${HOME}/.akmaster" ] && {
    log "found a .akmaster file, a simulation might still be running"
    log "want me to cleanup anyways? (y/n)"
    read ss_inp
    [ "${ss_inp}" != "y" ] && exit 0
    rm -f "${HOME}/.akmaster"
    log "removed ${HOME}/.akmaster"
}

# for each host, check the temporary storage directory
# delete all writable directories
host_counter=0
for host in `get_potential_hosts`; do
    log "cleaning up host ${host}"
    ssh "${host}" "for f in $AK_TMP_BASE/*; do echo found file \$f; [ -w \$f ] && { echo removing \$f; rm -Rf \$f; } || echo cannot remove \$f; done"
    host_counter=`expr "${host_counter}" + 1`
done

# print summary
log "${host_counter} hosts have been checked"

