#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Sven Grottke, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# run an experiment with a given parameter set using Akaroa
# usage: distrib_search.sh measure_option base_dir sim_time parm_set output_file
# measure_option can be "net" or "div"

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}

# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"
source "${CONDEL_ROOT}/experiments/common_files/akaroa.sh"


unset TMP_RESULTS
# cleanup, will be called automagically on exit (signal 0)
ds_cleanup() {
    # do not remove the temporary files if an error occured
    [ "${sim_err_code}" ] && return
    
    # remove config files
    [ -f "${INIFILE_STUB}" ] && rm "${INIFILE_STUB}"
    [ -f "${NEDFILE}" ] && rm "${NEDFILE}"
    [ -f "${AKENV_FILE}" ] && rm "${AKENV_FILE}"

    # remove temporary results
    [ "${TMP_RESULTS}" -a -d "${TMP_RESULTS}" -a -w "${TMP_RESULTS}" ] && {
        rm -Rf "${TMP_RESULTS}" || log "could not remove ${TMP_RESULTS}"
    }
}
add_exit_handler ds_cleanup



# sanity check
[ $# = 5 ] || {
    error "usage: distrib_search.sh measure_option base_dir sim_time parm_set output_file(measure_option can be \"net\" or \"div\")"
}

    
# set some paths and variables
EXP_BASE="$2"
COMMON_DIR="${CONDEL_ROOT}/experiments/common_files";
BINARY="${EXP_BASE}/bin/simulation.sh"
DISTRIB_RUN="${COMMON_DIR}/distrib_run.sh"
CONF_EXP="${EXP_BASE}/scripts/conf_exp.sh"
CONF_EXP_NET="${EXP_BASE}/scripts/conf_exp_net.sh"
CONF_EXP_DIV="${EXP_BASE}/scripts/conf_exp_div.sh"
SINGLE_RUN="${COMMON_DIR}/single_run.sh"
SINGLE_SEARCH="${COMMON_DIR}/single_search.pl"
RESULTS="${EXP_BASE}/results"
RESULT_FILE="$5"
INIFILE_STUB=`mktemp "${EXP_BASE}/omnetpp_dyna.ini.XXXXXX"` || error "could not create temporary .ini file"
NEDFILE=`mktemp "${EXP_BASE}/omnetpp_dyna.ned.XXXXXX"` || error "could not create temporary .ned file"

# remove old results
[ -f "${RESULT_FILE}" ] && rm "${RESULT_FILE}"

# setup akaroa
log "starting Akaroa..."
ak_init
log "done."

# get some basic parameters from the experiment
case "$1" in
"-net" ) conf_exp_measure=${CONF_EXP_NET};;
"-div" ) conf_exp_measure=${CONF_EXP_DIV};;
    * )  echo -e "mesure-option invalid!";
         error "usage: distrib_search.sh measure_option base_dir sim_time parm_set output_file(measure_option can be \"net\" or \"div\")";;
esac

#echo -e "${conf_exp_measure} --write-parms \"$4\"\n"
parm_set=`${conf_exp_measure} --write-parms $4`
expected_value=`${conf_exp_measure} --get-expected-value $4`
defaults=`${conf_exp_measure} --get-defaults`
range=`${conf_exp_measure} --get-range`
precision=`${conf_exp_measure} --get-precision`
maxsteps=`${conf_exp_measure} --get-maxsteps`
measure_name=`${conf_exp_measure} --get-measure-name`
measure_name_nospace=`echo ${measure_name} | sed -e s/[^[:alnum:]]//g`
default1=`get_nth 1 $defaults`
default2=`get_nth 2 $defaults`
start=`get_nth 1 $range`
end=`get_nth 2 $range`
sim_time="$3"

echo -e "${SINGLE_SEARCH} \"${EXP_BASE}\" \"${measure_name}\" ${expected_value} ${sim_time} ${parm_set} $default1 $default2 $start $end $precision $maxsteps ${COMMON_DIR} ${RESULT_FILE}\n"
eval ${SINGLE_SEARCH} ${EXP_BASE} '"'${measure_name}'"' ${expected_value} ${sim_time} ${parm_set} $default1 $default2 $start $end $precision $maxsteps ${COMMON_DIR} ${RESULT_FILE}



