#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Sven Grottke, Jiehua Chen, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# run an experiment with a given parameter set
# usage: single_run.sh base_dir sim_time parm_set output_file

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}

# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# cleanup, will be called automagically on exit (signal 0)
sr_cleanup() {
    # do not remove temporary files if an error occured
    [ "${sim_err_code}" ] && return

    # remove config files
    [ -f "${INIFILE}" ] && rm "${INIFILE}"
    [ -f "${NEDFILE}" ] && rm "${NEDFILE}"
}
add_exit_handler sr_cleanup



# sanity check
[ $# = 4 ] || {
    error "usage: single_run.sh base_dir sim_time parm_set output_file"
}
    
# set some paths and variables
EXP_BASE="$1"
BINARY="${EXP_BASE}/bin/simulation.sh"
CONF_EXP="${EXP_BASE}/scripts/conf_exp.sh"
RESULTS="${EXP_BASE}/results"
RESULT_FILE="$4"
INIFILE=`mktemp "${EXP_BASE}/omnetpp_dyna.ini.XXXXXX"` || error "could not create temporary .ini file"
NEDFILE=`mktemp "${EXP_BASE}/omnetpp_dyna.ned.XXXXXX"` || error "could not create temporary .ned file"

# create NED file
"${CONF_EXP}" --create-ned $3 > "${NEDFILE}" || error "could not create NED file ${NEDFILE}"

# create .ini file
cat >"${INIFILE}" <<EOF
# omnetpp ini file

include ${CONDEL_ROOT}/omnetpp.ini

[General]
preload-ned-files=${NEDFILE}
output-vector-file = ${RESULTS}/omnetpp.vec
output-scalar-file = ${RESULT_FILE}
sim-time-limit = $2

EOF
${CONF_EXP} --create-ini $3 >>"${INIFILE}" || error "could not create INI file ${INIFILE}"

# execute
${BINARY} -u Cmdenv -f ${INIFILE} 1>/dev/null || {
    error "simulation failed"
}

# write pseudo module containing the run parameters
${CONF_EXP} --write-parms $3 >> "${RESULT_FILE}" || error "could not write run parameters to ${RESULT_FILE}"

# write pseudo module containing variance and disco weights
VAR_WEIGHTS=`${CONF_EXP} --get-variance-weights` || error "could not get variance weights"
echo -e "scalar \"variance weights\"\t\"${VAR_WEIGHTS}\"\t0" >> "${RESULT_FILE}"
DISCO_WEIGHTS=`${CONF_EXP} --get-disco-weights` || error "could not get disco weights"
echo -e "scalar \"disco weights\"\t\"${DISCO_WEIGHTS}\"\t0" >> "${RESULT_FILE}"

# write pseudo value signalling that the experiment finished successfully
echo -e "scalar \"status\"\t\"success\"\t0" >> "${RESULT_FILE}"
