#!/usr/bin/perl
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Sven Grottke, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# usage: single_search.pl base_dir sim_time parm_set default1 default2 {x_min} {x_max} precision maxstep common_dir output_scafile
# precision is percentage
use strict;

# constant for the precision calculation
my $EPS = 0.00001;
# for file handle
my ${base_dir}, my ${quoted_measurement}, my ${COMMON_DIR}, my ${single_run}, my ${distrib_run}, my ${network},
    my ${measure_name}, my ${expected_value}, my ${output_file}, my ${OUTPUT_SCAFILE}, my ${TMP_SCAFILE}, my ${tmp_dir};
# for x calculation
my ${sim_time}, my ${parm_set}, my $default1, my $default2, my ${x_min}, my ${x_max}, my ${precision_distrib},
    my ${precision_single}, my ${maxsteps}, my $y, my ${parm_set_nospace}, my $x1, my $x2, my $dif1, my $dif2,
    my $x_new, my ${y_mean};
sub usage{
     print STDERR "usage: single_search.pl base_dir measurement expected_value sim_time parm_set default1 default2 {x_min} {x_max} precision(percentage) maxsteps common_dir output_scafile\n";
}

sub failure{
    my $failedparm = shift;
    
    if ( defined $failedparm ){
        print STDERR "$failedparm format is wrong\n";
    }
    else{
        print STDERR "parameters failed!\n";
      }
    &usage;
    print STDERR "Please see the usage for more details.\n";
    die "aborting.";
}

sub gaussian_rand {
    my ($u1, $u2);  # uniformly distributed random numbers
    my $s;          # variance, then a weight
    my ($g1, $g2);  # gaussian-distributed numbers
    
    do{
        do {
            $u1 = 2 * rand() - 1;
            $u2 = 2 * rand() - 1;
            $s = $u1*$u1 + $u2*$u2;
        } while ( $s >= 1 );
        
        $s = sqrt( (-2 * log($s))  / $s );
        $g2 = $u1 * $s;
        $g1 = $u2 * $s;
    } while( $g1 > 1 || $g1 < -1 || $g2 > 1 || $g2 < -1 );

    print "u1: $u1, u2: $u2\n";
    
    # return both if wanted, else just one
    return wantarray ? ($g1, $g2) : $g1;
}

sub mean_value {
    my $filename = shift;
    my ${measure_name} = shift;
    my $sum = 0.0;
    my $count = 0;
    open(SCAFILE, $filename);
    print DATAFILE "open file: $filename, measurename: ${measure_name}\n";
    my @scalines = <SCAFILE>;
    foreach( @scalines ){
        if( /\"${network}.average\"\s+\"${measure_name}\"\s+([\.\d\-e]+)/ ){
            $sum += $1;
            $count++;
        }
    }
    if( $count == 0 ){
        return -1;
    }else{
        $sum /= $count;
        print DATAFILE "y: $sum\n";
        return $sum;
    }
}

sub find_x {
    my $found = 0;
    my $count = 0; 
    $dif1 = 0.0;
    $dif2 = 0.0;
    my $dif = 0.0;
    # use either single_run.sh or distrib_run.sh 
    my $executable = shift;
    my $precision = shift;
    # Preperation1  for the interpolation
    my ${parm_set1} = ${parm_set};
    my ${parm_set2} = ${parm_set};
    my ${y_count} = 0;
    my ${RESULT_FILE_NEW};
    # x-values out of range?
    if( ($x1 < ${x_min} && $x2 < ${x_min}) || ($x1 > ${x_max} && $x2 > ${x_max}) ){
        $x1 = ${x_min};
        $x2 = ${x_max};
    }

    ${parm_set1} =~ s/X/${x1}/g;
    # print DATAFILE "parm_set2: ${parm_set2}\n";
    my ${RESULT_FILE1}="${tmp_dir}/${measure_name}_${expected_value}_default1_${x1}_parm_set_${parm_set_nospace}.sca";
    if( -f ${RESULT_FILE1} ){
        print DATAFILE "rm ${RESULT_FILE1}\n";
        system("rm ${RESULT_FILE1}");
    }

    # produce the accordance y-values with default-x1
    print DATAFILE "${executable} ${base_dir} ${sim_time} \"${parm_set1}\" ${RESULT_FILE1}\n";
    system("${executable} ${base_dir} ${sim_time} \"${parm_set1}\" ${RESULT_FILE1}");
    my $y1 = mean_value(${RESULT_FILE1}, ${quoted_measurement});

    # produce the accordance y-values with default-x2
    ${parm_set2} =~ s/X/${x2}/g;
    my ${RESULT_FILE2}="${tmp_dir}/${measure_name}_${expected_value}_default2_${x2}_parm_set_${parm_set_nospace}.sca";
    if( -f ${RESULT_FILE2} ){
        system("rm ${RESULT_FILE2}");
    }
    print DATAFILE "${executable} ${base_dir} ${sim_time} \"${parm_set2}\" ${RESULT_FILE2}\n";
    system("${executable} ${base_dir} ${sim_time} \"${parm_set2}\" ${RESULT_FILE2}");

    # calculate the meanvalue y2 in SCAFILE for x2 
    my $y2 = mean_value(${RESULT_FILE2}, ${quoted_measurement});

    #my $ABS=abs($y-${expected_value})/${expected_value};
    #print DATAFILE("ABS: $ABS, count: $count\n");
    while( $count <= $maxsteps ){
        $count++;
        $dif1 = abs(${expected_value}-$y1);
        $dif2 = abs(${expected_value}-$y2);
        print DATAFILE "$count) expected_value: ${expected_value}\n";
        print DATAFILE "x1: $x1, y1: $y1, dif1: $dif1\n";
        print DATAFILE "x2: $x2, y2: $y2, dif2: $dif2\n";
        print DATAFILE "x_new: ${x_new}, y: $y, dif: $dif\n";
        my ${parm_set_new} = ${parm_set};
        $y = ${expected_value};
        # calculate the new x-value with gradient methode
        if( $dif1/${expected_value} <= ${precision} && $dif1 <= $dif2){
            print DATAFILE "x1: $x1, y1: $y1, dif1: $dif1\n";
            print DATAFILE "$x1\n";
            ${TMP_SCAFILE} = ${RESULT_FILE1};
            $found = 1;
            last;
        }elsif( $dif2/${expected_value} <= ${precision} ){
            print DATAFILE "x2: $x2, y2: $y2, dif2: $dif2\n";
            print DATAFILE "$x2\n";
            ${TMP_SCAFILE} = ${RESULT_FILE2};
            $found = 1;
            last;
        }elsif( abs( $y1 - $y2 ) <= $EPS ){
            my $tm = $y1 - $y2;
            $found = 1;
            ${TMP_SCAFILE} = ${RESULT_FILE1};
            print DATAFILE "x2: $x2, y2: $y2, dif2: $dif2\n";
            print DATAFILE "$x2\n";
            print DATAFILE "abs(y1-y2): $tm\n";
            # print DATAFILE "-1\n";
            last;
        }else{
            ${x_new} = ( -$x1*$y2 + $y1*$x2 - $y*$x2 + $x1*$y) / ($y1 - $y2);
            print DATAFILE "calculated x: ${x_new}\n";
            if( ${x_new} < ${x_min} ){
                ${x_new} = ${x_min};
                # next;
            }
            if( ${x_new} > ${x_max} ){
                ${x_new} = ${x_max};
                # next;
            }
            # calculate a new x value with difference to the former one according
            # to the standard-normal-distribution
            my $length;
            my $probability;
            do{
                $probability = gaussian_rand();
            }while( $probability < -1 || $probability > 1 );
            print DATAFILE "probability: $probability\n";
            if( $probability <= 0 ){
                $length = ${x_new} - ${x_min};
            }else{
                $length = ${x_max} - ${x_new};
            }
            ${x_new} = $probability * $length * 0.1 + ${x_new};
            print DATAFILE "x_new: ${x_new}\n";
            
            # execute with the new x value
            ${parm_set_new} =~ s/X/${x_new}/;
            ${RESULT_FILE_NEW}="${tmp_dir}/${measure_name}_${expected_value}_x_new_${x_new}_parm_set_${parm_set_nospace}.sca";
            if( -f ${RESULT_FILE_NEW} ){
                print DATAFILE "rm ${RESULT_FILE_NEW}\n";
                system("rm ${RESULT_FILE_NEW}");
            }
            print DATAFILE "$count : ${executable} ${base_dir} ${sim_time} \"${parm_set_new}\" ${RESULT_FILE_NEW}\n";
            system("${executable} ${base_dir} ${sim_time} \"${parm_set_new}\" ${RESULT_FILE_NEW}");
            $y = mean_value(${RESULT_FILE_NEW}, ${quoted_measurement});
#         $dif1 = abs($expected_value-$y1);
#         $dif2 = abs($expected_value-$y2);
#         
#         print DATAFILE "x1: $x1, y1: $y1, dif1: $dif1\n";
#         print DATAFILE "x2: $x2, y2: $y2, dif2: $dif2\n";
            $dif = abs(${expected_value}-$y);
            if( $dif1 <= $dif2 ){
                $x2 = ${x_new};
                $y2 = $y;
                $dif2 = $dif;
                ${RESULT_FILE2} = ${RESULT_FILE_NEW};
            }else{
                $x1 = ${x_new};
                $y1 = $y;
                $dif1 = $dif;
                ${RESULT_FILE1} = ${RESULT_FILE_NEW};
            }
        }
    }
    if( $found == 0 ){
        if( $dif1 <= $dif2  ){
            ${TMP_SCAFILE} = ${RESULT_FILE1};
        }else{
            ${TMP_SCAFILE} = ${RESULT_FILE2};
        }
    }
}

# quote the parameters
my @QUOTED_ARGV = @ARGV;
map { s/(\W)/\\$1/g; } @QUOTED_ARGV;

my $argc = 0;
# start with mandatory paramaters: filename start stop parm
my $length = @ARGV - 0;
if ( @ARGV != 13 ) { &failure(); }
${base_dir} = $ARGV[$argc++];
${network} = ${base_dir};
$network =~ s:^.*/::;
my $measurement = $ARGV[$argc];
${quoted_measurement} = @QUOTED_ARGV[$argc++];
${measure_name} = $measurement;
${measure_name} =~ s/(\W)//g;
#print DATAFILE "${quoted_measurement}\n";
if ( $ARGV[$argc] !~ /^[\.\d]+/ || $ARGV[$argc] <= $EPS ) {
    &failure("$ARGV[$argc]\nexpected_value");
}
${expected_value} = $ARGV[$argc++];   # expected value must be non negative
if ( $ARGV[$argc] !~ /^[\.\d]+/ ) { &failure("simulation time"); }
${sim_time} = $ARGV[$argc++];     # simulation time

# parameter set for single_run.sh or distrib_run.sh
${parm_set} = $ARGV[$argc++];
#print DATAFILE "${parm_set}\n";

# information about the measure_name
if ( $ARGV[$argc] !~ /^[\.\d]+/ ) { &failure("$ARGV[$argc] first default value for ${measure_name}"); }
$default1 = $ARGV[$argc++];
if ( $ARGV[$argc] !~ /^[\.\d]+/ ) { &failure("second default value for ${measure_name}"); }
$default2 = $ARGV[$argc++];
if ( $ARGV[$argc] !~ /^[\.\d]+/ ) { &failure("start range"); }
${x_min} = $ARGV[$argc++];
if ( $ARGV[$argc] !~ /^[\.\d]+/ ) { &failure("end range"); }
${x_max} = $ARGV[$argc++];
if ( $ARGV[$argc] !~ /^[\.\d]+/ ) { &failure("${measure_name} precision"); }
${precision_distrib} = $ARGV[$argc++];
${precision_single} = ${precision_distrib} * 2;
if ( $ARGV[$argc] !~ /^[\.\d]+/ ) { &failure("max. steps for calculation"); }
$maxsteps = $ARGV[$argc++];
${COMMON_DIR} = $ARGV[$argc++];
${single_run} = "${COMMON_DIR}/single_run.sh";
${distrib_run} = "${COMMON_DIR}/distrib_run.sh";
${OUTPUT_SCAFILE} = $ARGV[$argc++];

# check all the parameters
# print DATAFILE "max. steps: $maxsteps, range: ${x_min} -> ${x_max}, precision_distrib: ${precision_distrib}, default1: $default1, default2: $default2, parm_set ${parm_set}\n";

# y is the measure_name calculated
${tmp_dir} = `mktemp -d ${base_dir}/results/search_tmp.XXXXX`;
chomp ${tmp_dir};
$y = 0;
${parm_set_nospace} = ${parm_set};
${parm_set_nospace} =~ s/(\b)(\s+)(\b)/\1_\3/g;
${parm_set_nospace} =~ s/(\")//g;
#print DATAFILE "parm_set_nospace: ${parm_set_nospace}\n";
$x1 = ${default1};
$x2 = ${default2};
$output_file = "${tmp_dir}/${measure_name}.data";
$x_new = 0.0;
open(DATAFILE,">>${output_file}");

#find_x(${single_run}, ${precision_single});
find_x(${distrib_run}, ${precision_distrib});
my ${parm_set_new} = ${parm_set};
${parm_set_new} =~ s/X/${x_new}/;
# check, if the sca file exists
-r ${TMP_SCAFILE} or die;
system("cp ${TMP_SCAFILE} ${OUTPUT_SCAFILE} ");
${y_mean} = mean_value(${TMP_SCAFILE}, ${quoted_measurement});
open(OUTPUTSCAFILE,">>${OUTPUT_SCAFILE}");
print DATAFILE ${y_mean};
#print OUTPUTSCAFILE "${y_mean}\n";
close(OUTPUTSCAFILE);
system("rm -Rf ${tmp_dir}");

close(DATAFILE);
