#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Sven Grottke, Jiehua Chen, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# this is a wrapper script for the applications run by the Akaroa slaves. It
# redirects their stdout and stderr into files so that they are available for
# debugging
# usage: slave_wrapper.sh output_dir tmp_base_dir ini_stub application [arguments]
#
# known bugs:
#
#   - more than one simulation engine on the same slave is not supported (it
#     is now, but it relies on the process ID, which might be duplicated)
#     TODO: use mktemp instead (important: also for .vec and .sca)

# set output directory and file names
[ $# -ge 1 ] || exit 1
OUTPUT_DIR="$1"
shift
[ -d "${OUTPUT_DIR}" -a -w "${OUTPUT_DIR}" ] || exit 1
STDERR_FILE="${OUTPUT_DIR}/${HOST}.$$.stderr"
[ -a "${STDERR_FILE}" ] && exit 1
# from here on, we can produce debug output
log() {
    echo "$@" >> "${STDERR_FILE}"
}
error() {
    log "ERROR: $@"
    log "$0 aborting."
    exit 1
}

STDOUT_FILE="${OUTPUT_DIR}/${HOST}.$$.stdout"
[ -a "${STDOUT_FILE}" ] && error "${STDOUT_FILE} already exists."

# create a temporary directory to store the results
TMP_BASE_DIR="$1"
shift
TMP_DIR=`TMPDIR="${TMP_BASE_DIR}" mktemp -tdq akaroa.slave.XXXXXXXX` || error "could not create temporary directory"
log "results will be stored in temporary directory ${TMP_DIR}"

# tell the master instance that a simulation engine was run on this host
SLAVE_LOCK="${OUTPUT_DIR}/slave.${HOST}.$$.lock"
[ -e "${SLAVE_LOCK}" ] && error "${SLAVE_LOCK} already exists"
echo "${TMP_DIR}" > "${SLAVE_LOCK}" || error "could not register slave lock file ${SLAVE_LOCK}"

# create the actual .ini file from the stub by appending the path to .vec and .sca
INIFILE_STUB="$1"
shift
INIFILE="${TMP_DIR}/omnetpp.ini"
cat "${INIFILE_STUB}" > "${INIFILE}"
echo " " >> "${INIFILE}"
echo "[General]" >> "${INIFILE}"
echo "output-vector-file = ${TMP_DIR}/omnetpp.vec" >> "${INIFILE}"
echo "output-scalar-file = ${TMP_DIR}/omnetpp.sca" >> "${INIFILE}"

# now, run the application
[ $# -ge 2 ] || error "application name missing."
APP_NAME="$1"
shift

# install trap handler for SIGINT
#handle_sigterm() {
#    log "received SIGTERM, terminating application with PID ${APP_PID}"
#    kill -SIGTERM "${APP_PID}"
#}
#trap handle_sigterm SIGTERM

#log "trap list:"
#trap -p >> "${STDERR_FILE}"

log "starting application ${APP_NAME} with arguments $@"
CWD=`pwd`
log "current working directory: $CWD"
[ -r ../../omnetpp.ini ] && {
    log "../../omnetpp.ini readable"
} || {
    log "../../omnetpp.ini not readable!!!!!!"
}
exec "${APP_NAME}" -f "${INIFILE}" "$@" 1>>"${STDOUT_FILE}" 2>>"${STDERR_FILE}"
#"${APP_NAME}" "$@" 1>>"${STDOUT_FILE}" 2>>"${STDERR_FILE}" &
#APP_PID=$!
#log "application started with PID ${APP_PID}"
#wait "${APP_PID}"
#APP_ERR="$?"
#log "application ${APP_PID} finished with exit code ${APP_ERR}"

# signalise that we're done
#log "slave_wrapper finished."

#exit "${APP_ERR}"
