#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# write the NED file for pong_loose to stdout, gain information about
# run-specific parameters, write the application-specific part of an
# INI file, or write the run parameters into an OMNet++ .sca file
# usage: 1) write a .ned file to stdout
#         conf_exp.sh --create-ned
#        2) write variance weights to stdout
#         conf_exp.sh --get-variance-weights
#        3) write disco weights to stdout
#         conf_exp.sh --get-disco-weights
#        4) write the application-specific portion of an INI file to stdout
#         conf_exp.sh --create-ini
#        5) write the run parameters as a pseudo-module in OMNet++ .sca file
#           syntax
#         conf_exp.sh --write-parms
# will be called by both, single_run.sh and distrib_run.sh

usage() {
    echo "possible usages: conf_exp.sh --create-ned" >&2
    echo "                 conf_exp.sh --get-variance-weights" >&2
    echo "                 conf_exp.sh --get-disco-weights" >&2
    echo "                 conf_exp.sh --create-ini" >&2
    echo "                 conf_exp.sh --write-parms" >&2
}

VAR_WEIGHTS="cBall::positionX 1 cBall::positionY 1 cBall::velocityX 0.5 cBall::velocityY 0.5 cRacket::positionX 1 cRacket::positionY 1 cRacket::velocityX 1 cRacket::velocityY 0 cScore::score 1"
DISCO_WEIGHTS="cBall::positionX 1 cBall::positionY 1 cBall::velocityX 0.5 cBall::velocityY 0.5 cRacket::positionX 1 cRacket::positionY 1 cRacket::velocityX 1 cRacket::velocityY 0 cScore::score 1"

# check command
[ $# -ge 1 ] || {
    usage
    exit 1
}

[ "$1" = "--get-variance-weights" ] && {
    echo "${VAR_WEIGHTS}"
    exit 0
}
[ "$1" = "--get-disco-weights" ] && {
    echo "${DISCO_WEIGHTS}"
    exit 0
}

[ "$1" = "--create-ned" ] && {

    cat <<EOF
// NED file for the Pong experiment using ideal consistency

simple cPongLooseHost
    parameters:
        regular_updates: const,
        ball_distance: const,
        racket_distance: const,
        loss_rate: const,
	disco_weights: string,
        render: bool;
    gates:
        in: in;
        out: out;
endsimple

simple cEye
    parameters:
        variance_weights: string;
endsimple


module pongIdeal
    submodules:
        player_left:  cPongLooseHost;
		parameters: disco_weights=
			"$DISCO_WEIGHTS";
                display: "i=device/pc,cyan";

        player_right: cPongLooseHost;
		parameters: disco_weights=
			"$DISCO_WEIGHTS";
                display: "i=device/pc,gold";

        eye: cEye;
		parameters: variance_weights=
			"$VAR_WEIGHTS";
                display: "i=block/eyes";

    connections:
        player_left.out --> player_right.in;
        player_left.in <-- player_right.out;
endmodule

network pong_ideal : pongIdeal
endnetwork
EOF

    exit 0
}

[ "$1" = "--create-ini" ] && {

    cat <<EOF
[General]
network=pong_ideal

[Parameters]
pong_ideal.*.regular_updates = 0.01;
pong_ideal.*.ball_distance = 0;
pong_ideal.*.racket_distance = 0;
pong_ideal.*.loss_rate = 0;
pong_ideal.*.render = false;

EOF

    exit 0
}

[ "$1" = "--write-parms" ] && {
    exit 0
}

# no legal parameters
usage
exit 1
