#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Jan Sablatnig, Sven Grottke.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# masterscript for the ball update distance measurement script(distrib_ball.sh) and the analysis script(plot2d.pl)
# usage: ball_master.sh min_ball max_ball loss_rate delay update_frequency racket_distance out_dir

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}
# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 7 ] || {
    error "usage: ball_master.sh min_ball max_ball loss_rate delay update_frequency racket_distance output_directory"
}

# set some paths
PONG_LOOSE_BASE="${CONDEL_ROOT}/experiments/pong_loose"
PONG_IDEAL_BASE="${CONDEL_ROOT}/experiments/pong_ideal"
PONG_LOOSE_RESULTS="${PONG_LOOSE_BASE}/results"
PONG_IDEAL_RESULTS="${PONG_IDEAL_BASE}/results"
PLOT_BASE="${CONDEL_ROOT}/util/plot"
PLOT_OUT_BASE=$7
PLOT_OUT_PACKETS="${PLOT_OUT_BASE}/packets"
PLOT_OUT_DIV="${PLOT_OUT_BASE}/div"
PLOT_OUT_SCORE="${PLOT_OUT_BASE}/score"
PLOT_OUT_DISCO="${PLOT_OUT_BASE}/disco"

# check if the output directories exist and are writable
[ -d "${PLOT_OUT_PACKETS}" -a -w "${PLOT_OUT_PACKETS}" ] || {
    error "${PLOT_OUT_PACKETS} does not exist or is not writable"
}

# check if the output directories exist and are writable
[ -d "${PLOT_OUT_DIV}" -a -w "${PLOT_OUT_DIV}" ] || {
    error "${PLOT_OUT_DIV} does not exist or is not writable"
}

# check if the output directories exist and are writable
[ -d "${PLOT_OUT_SCORE}" -a -w "${PLOT_OUT_SCORE}" ] || {
    error "${PLOT_OUT_SCORE} does not exist or is not writable"
}

# check if the output directories exist and are writable
[ -d "${PLOT_OUT_DISCO}" -a -w "${PLOT_OUT_DISCO}" ] || {
    error "${PLOT_OUT_DISCO} does not exist or is not writable"
}

# set the executable measurement- and plotscript
DISTRIB_BALL="${PONG_LOOSE_BASE}/scripts/distrib_ball.sh"
SCA_FILE="${PONG_LOOSE_RESULTS}/pong_loose.var_ball.sca"
IDEAL_FILE="${PONG_IDEAL_RESULTS}/pong_ideal.sca" 
PLOT2D="${PLOT_BASE}/plot2d.pl"
FAILED_FILE="${CONDEL_ROOT}/experiments/common_files/measurement_failed.ps"

# set some variables
MIN_BALL=$1
MAX_BALL=$2
LOSS=$3
DELAY=$4
UPDATE=$5
RACKET_DIST=$6
ball="ball update distance"
host_left="pong_loose.average"
host_eye="pong_loose.eye"
packets="avg. packets sent"
div="D"
score="score achieved /s"
disco="avg. G"

# set the filename from results and the ps_file for output
GRAPHIC_PACKETS="${PLOT_OUT_PACKETS}/ball_packets_${MIN_BALL}_${MAX_BALL}_${LOSS}_${DELAY}_${UPDATE}_${RACKET_DIST}.ps"
GRAPHIC_DIV="${PLOT_OUT_DIV}/ball_div_${MIN_BALL}_${MAX_BALL}_${LOSS}_${DELAY}_${UPDATE}_${RACKET_DIST}.ps"
GRAPHIC_SCORE="${PLOT_OUT_SCORE}/ball_score_${MIN_BALL}_${MAX_BALL}_${LOSS}_${DELAY}_${UPDATE}_${RACKET_DIST}.ps"
GRAPHIC_DISCO="${PLOT_OUT_DISCO}/ball_disco_${MIN_BALL}_${MAX_BALL}_${LOSS}_${DELAY}_${UPDATE}_${RACKET_DIST}.ps"


# #execute
log "starting the measurement script..."
${DISTRIB_BALL} "${MIN_BALL}" "${MAX_BALL}" "${LOSS}" "${DELAY}" "${UPDATE}" "${RACKET_DIST}" || {
    # copy an error message postscript file
    for dest in "${GRAPHIC_PACKETS}" "${GRAPHIC_DIV}" "${GRAPHIC_SCORE}" "${GRAPHIC_DISCO}"; do
        cp "${FAILED_FILE}" "${dest}"
    done

    log "measurement failed, aborting"
    exit 0
}

# # copy the sca to /loss/sca/.
SCA_FILE="${PLOT_OUT_BASE}/sca/pong_loose.var_ball_${MIN_BALL}_${MAX_BALL}_${LOSS}_${DELAY}_${UPDATE}_${RACKET_DIST}.sca"
cp "${PONG_LOOSE_RESULTS}/pong_loose.var_ball.sca" "${SCA_FILE}"

# plots
# packets
log "starting the plot for \"${packets}\" at \"${host_left}\""
${PLOT2D} "${SCA_FILE}" "${MIN_BALL}" "${MAX_BALL}" "$ball" "-lx" "$host_left" "$packets" > "${GRAPHIC_PACKETS}" || {
    cp "${FAILED_FILE}" "${GRAPHIC_PACKETS}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_BALL} ${MAX_BALL} $ball -lx $host_left $packets > $GRAPHIC_PACKETS"
}

# div
log "starting the plot for \"${div}\" at \"${host_eye}\""
${PLOT2D} "${SCA_FILE}" "${MIN_BALL}" "${MAX_BALL}" "$ball" "-lx" "$host_eye" "$div" > "${GRAPHIC_DIV}" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DIV}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_BALL} ${MAX_BALL} $ball -lx $host_eye $div > $GRAPHIC_DIV"
}

# score
log "starting the plot for \"${score}\" at \"${host_left}\""
${PLOT2D} "${SCA_FILE}" "${MIN_BALL}" "${MAX_BALL}" "$ball"   "-i" "${IDEAL_FILE}" "-lx" "$host_left" "$score" > "${GRAPHIC_SCORE}" || {
    cp "${FAILED_FILE}" "${GRAPHIC_SCORE}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_BALL} ${MAX_BALL} $ball  -i ${IDEAL_FILE} -lx $host_left $score > ${GRAPHIC_SCORE}"
}

# disco
log "starting the plot for \"${disco}\" at \"${host_left}\""
${PLOT2D} "${SCA_FILE}" "${MIN_BALL}" "${MAX_BALL}" "$ball" "-lx" "$host_left" "$disco" > "${GRAPHIC_DISCO}" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DISCO}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_BALL} ${MAX_BALL} $ball -lx $host_left $disco > ${GRAPHIC_DISCO}"
}
