#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Sven Grottke, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

#masterscript for the loss rate measurement script(distrib_loss.sh) and the analysis script(plot2d.pl)
# usage: loss_master.sh min_loss max_loss stepsize delay regular_update ball_distance racket_distance out_dir

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}
# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 8 ] || {
    error "usage: loss_master.sh min_loss max_loss stepsize delay regular_update ball_distance racket_distance output_directory"
}

# set some paths
PONG_LOOSE_BASE="${CONDEL_ROOT}/experiments/pong_loose"
PONG_IDEAL_BASE="${CONDEL_ROOT}/experiments/pong_ideal"
PONG_LOOSE_RESULTS="${PONG_LOOSE_BASE}/results"
PONG_IDEAL_RESULTS="${PONG_IDEAL_BASE}/results"
PLOT_BASE="${CONDEL_ROOT}/util/plot"
PLOT_OUT_BASE="$8"
PLOT_OUT_PACKETS="${PLOT_OUT_BASE}/packets"
PLOT_OUT_DIV="${PLOT_OUT_BASE}/div"
PLOT_OUT_SCORE="${PLOT_OUT_BASE}/score"
PLOT_OUT_DISCO="${PLOT_OUT_BASE}/disco"

# check if the output directories exist and are writable
[ -d "${PLOT_OUT_PACKETS}" -a -w "${PLOT_OUT_PACKETS}" ] || {
    error "${PLOT_OUT_PACKETS} does not exist or is not writable"
}
[ -d "${PLOT_OUT_DIV}" -a -w "${PLOT_OUT_DIV}" ] || {
    error "${PLOT_OUT_DIV} does not exist or is not writable"
}
[ -d "${PLOT_OUT_SCORE}" -a -w "${PLOT_OUT_SCORE}" ] || {
    error "${PLOT_OUT_SCORE} does not exist or is not writable"
}
[ -d "${PLOT_OUT_DISCO}" -a -w "${PLOT_OUT_DISCO}" ] || {
    error "${PLOT_OUT_DISCO} does not exist or is not writable"
}

# set the executable measurement- and plotscript
DISTRIB_LOSS="${PONG_LOOSE_BASE}/scripts/distrib_loss.sh"
SCA_FILE="${PONG_LOOSE_RESULTS}/pong_loose.var_loss.sca"
IDEAL_FILE="${PONG_IDEAL_RESULTS}/pong_ideal.sca" 
PLOT2D="${PLOT_BASE}/plot2d.pl"
FAILED_FILE="${CONDEL_ROOT}/experiments/common_files/measurement_failed.ps"

# set some variables
# player_left is now player@3
MIN_LOSS=$1
MAX_LOSS=$2
STEPSIZE=$3
DELAY=$4
REGULAR_UPDATE=$5
BALL_DIST=$6
RACKET_DIST=$7
loss="loss rate"
host_left="pong_loose.average"
host_eye="pong_loose.eye"
packets="avg. packets sent"
div="D"
score="score achieved /s"
disco="avg. G"

# set the filenames from results and the ps_file for output
GRAPHIC_PACKETS="${PLOT_OUT_PACKETS}/loss_packets_${MIN_LOSS}_${MAX_LOSS}_${DELAY}_${REGULAR_UPDATE}_${BALL_DIST}_${RACKET_DIST}.ps"
GRAPHIC_DIV="${PLOT_OUT_DIV}/loss_div_${MIN_LOSS}_${MAX_LOSS}_${DELAY}_${REGULAR_UPDATE}_${BALL_DIST}_${RACKET_DIST}.ps"
GRAPHIC_SCORE="${PLOT_OUT_SCORE}/loss_score_${MIN_LOSS}_${MAX_LOSS}_${DELAY}_${REGULAR_UPDATE}_${BALL_DIST}_${RACKET_DIST}.ps"
GRAPHIC_DISCO="${PLOT_OUT_DISCO}/loss_disco_${MIN_LOSS}_${MAX_LOSS}_${DELAY}_${REGULAR_UPDATE}_${BALL_DIST}_${RACKET_DIST}.ps"


# execute 
log "starting the measurement script..."
${DISTRIB_LOSS} "${MIN_LOSS}" "${MAX_LOSS}" "${STEPSIZE}" "${DELAY}" "${REGULAR_UPDATE}" "${BALL_DIST}" "${RACKET_DIST}" || {
    # copy an error message postscript file
    for dest in "${GRAPHIC_PACKETS}" "${GRAPHIC_DIV}" "${GRAPHIC_SCORE}" "${GRAPHIC_DISCO}"; do
        cp "${FAILED_FILE}" "${dest}"
    done

    log "measurement failed, aborting"
    exit 0
}

# copy the sca under /loss/sca/.
SCA_FILE="${PLOT_OUT_BASE}/sca/pong_loose.var_loss_${MIN_LOSS}_${MAX_LOSS}_stepsize_${STEPSIZE}_${DELAY}_${REGULAR_UPDATE}_${BALL_DIST}_${RACKET_DIST}.sca"
cp "${PONG_LOOSE_RESULTS}/pong_loose.var_loss.sca" ${SCA_FILE}

# plots
# packets 
log "starting the plot for \"${packets}\" at \"${host_left}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOSS}" "${MAX_LOSS}" "$loss" "$host_left" "$packets" > "$GRAPHIC_PACKETS" || {
    cp "${FAILED_FILE}" "${GRAPHIC_PACKETS}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOSS} ${MAX_LOSS} $loss $host_left $packets > $GRAPHIC_PACKETS"
}

# div
log "starting the plot for \"${div}\" at \"${host_eye}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOSS}" "${MAX_LOSS}" "$loss" "$host_eye" "$div" > "$GRAPHIC_DIV" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DIV}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOSS} ${MAX_LOSS} $loss $host_eye $div > $GRAPHIC_DIV"
}

# score
log "starting the plot for \"${score}\" at \"${host_left}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOSS}" "${MAX_LOSS}" "$loss" "-i" "${IDEAL_FILE}" "$host_left" "$score" > "$GRAPHIC_SCORE" || {
    cp "${FAILED_FILE}" "${GRAPHIC_SCORE}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOSS} ${MAX_LOSS} $loss -i ${IDEAL_FILE} $host_left $score > $GRAPHIC_SCORE"
}

# disco
log "starting the plot for \"${disco}\" at \"${host_left}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOSS}" "${MAX_LOSS}" "$loss" "$host_left" "$disco" > "$GRAPHIC_DISCO" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DISCO}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOSS} ${MAX_LOSS} $loss $host_left $disco > $GRAPHIC_DISCO"
}
