#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Jan Sablatnig, Sven Grottke.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# masterscript for the whole experiments.
# usage: masterscript.sh out_dir

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}
# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 1 ] || {
    error "usage: masterscript.sh output_directory"
}

# set some paths
PONG_COMMON="${CONDEL_ROOT}/experiments/common_files"
PONG_LOOSE_BASE="${CONDEL_ROOT}/experiments/pong_loose"
PONG_IDEAL_BASE="${CONDEL_ROOT}/experiments/pong_ideal"
PONG_LOOSE_RESULTS="${PONG_LOOSE_BASE}/results"
PONG_IDEAL_RESULTS="${PONG_IDEAL_BASE}/results"
PONG_LOOSE_SCRIPT_BASE="${PONG_LOOSE_BASE}/scripts"
PLOT_BASE="${CONDEL_ROOT}/util/plot"
PLOT_OUT_BASE=$1
PLOT_OUT_LOSS_BASE="${PLOT_OUT_BASE}/loss"
PLOT_OUT_DELAY_BASE="${PLOT_OUT_BASE}/delay"
PLOT_OUT_UPDATE_BASE="${PLOT_OUT_BASE}/update"
PLOT_OUT_BALL_BASE="${PLOT_OUT_BASE}/ball"
PLOT_OUT_RACKET_BASE="${PLOT_OUT_BASE}/racket"

# create directories
for parm in loss delay update ball racket; do 
    for measurement in packets div score disco sca; do
        mkdir -p "${PLOT_OUT_BASE}/${parm}/${measurement}" || {
            error "can not create ${PLOT_OUT_BASE}/${parm}/${measurement}"
        }
    done
done

# set variables for the executables
DISTRIB_RUN_SCRIPT="${PONG_COMMON}/distrib_run.sh"
LOSS_MASTER="${PONG_LOOSE_SCRIPT_BASE}/loss_master.sh"
DELAY_MASTER="${PONG_LOOSE_SCRIPT_BASE}/delay_master.sh"
UPDATE_MASTER="${PONG_LOOSE_SCRIPT_BASE}/update_master.sh"
BALL_MASTER="${PONG_LOOSE_SCRIPT_BASE}/ball_master.sh"
RACKET_MASTER="${PONG_LOOSE_SCRIPT_BASE}/racket_master.sh"

# set ranges
# LOSS_RANGE="0 0.001 0.02"
# DELAY_RANGE="0 0.1 0.5"
# UPDATE_RANGE="0.05 1 6.4"
# BALL_RANGE="0 5 1024"
# RACKET_RANGE="0 25 512"

# defaults only
LOSS_RANGE="0.001"
DELAY_RANGE="0.1"
UPDATE_RANGE="1"
BALL_RANGE="5"
RACKET_RANGE="25"

# for pong_ideal.sca
$DISTRIB_RUN_SCRIPT "${PONG_IDEAL_BASE}" "50000s" "" "${PONG_IDEAL_RESULTS}/pong_ideal.sca"

# for loss_master.sh
$LOSS_MASTER 0 0.02 0.001 0.1 0 5 25 ${PLOT_OUT_LOSS_BASE}

# for update_master.sh
$UPDATE_MASTER 0.15 10 0.001 0.1 10240 10240 ${PLOT_OUT_UPDATE_BASE}

# for loss_master.sh
for delay in ${DELAY_RANGE}; do
    for update in ${UPDATE_RANGE}; do
        for ball in ${BALL_RANGE}; do
            for racket in ${RACKET_RANGE}; do
                $LOSS_MASTER 0 0.02 0.001 $delay $update $ball $racket ${PLOT_OUT_LOSS_BASE}
            done
        done
    done
done

# for delay_master.sh
for loss in ${LOSS_RANGE}; do
    for update in ${UPDATE_RANGE}; do
        for ball in ${BALL_RANGE}; do
            for racket in ${RACKET_RANGE}; do
                $DELAY_MASTER 0  0.5 0.005 $loss $update $ball $racket ${PLOT_OUT_DELAY_BASE}
            done
        done
    done
done

# for update_master.sh
for loss in ${LOSS_RANGE}; do
    for delay in ${DELAY_RANGE}; do
        for ball in ${BALL_RANGE}; do
            for racket in ${RACKET_RANGE}; do
                $UPDATE_MASTER 0.15 10 $loss $delay $ball $racket ${PLOT_OUT_UPDATE_BASE}
            done
        done
    done
done

# for ball_master.sh
for loss in ${LOSS_RANGE}; do
    for delay in ${DELAY_RANGE}; do
        for update in ${UPDATE_RANGE}; do
            for racket in ${RACKET_RANGE}; do
                $BALL_MASTER 1 1024 $loss $delay $update $racket ${PLOT_OUT_BALL_BASE}
            done
        done
    done
done

# for racket_master.sh
for loss in ${LOSS_RANGE}; do
    for delay in ${DELAY_RANGE}; do
        for update in ${UPDATE_RANGE}; do
            for ball in ${BALL_RANGE}; do
                $RACKET_MASTER 1 512 $loss $delay $update $ball ${PLOT_OUT_RACKET_BASE}
            done
        done
    done
done



