#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# varies the network loss rate, leaving all other parameters constant
# usage: single_loss.sh min_loss max_loss stepsize delay regular_update ball_distance racket_distance

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}

# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 7 ] || {
    error "usage: single_loss.sh min_loss max_loss stepsize delay regular_update ball_distance racket_distance"
}
    
# set some paths and variables
PONG_LOOSE_BASE="${CONDEL_ROOT}/experiments/pong_loose"
SINGLE_RUN="${CONDEL_ROOT}/experiments/common_files/single_run.sh"
RESULTS="${PONG_LOOSE_BASE}/results"
VEC_FILE="${RESULTS}/omnetpp.vec"
SCA_FILE="${RESULTS}/pong_loose.var_loss..sca"

# remove old results
[ -f "${VEC_FILE}" ] && rm "${VEC_FILE}"
[ -f "${SCA_FILE}" ] && rm "${SCA_FILE}"

# step through it
loss=$1
while [ `echo "[1]sa $2 $loss !>a z p" | dc` = 1 ]; do
    log "measuring with a loss rate of ${loss} ..."
    RESULT_FILE="${RESULTS}/pong_loose_${loss}_$4_$5_$6_$7.sca" 
    ${SINGLE_RUN} "${PONG_LOOSE_BASE}" "50000s" "${loss} $4 $5 $6 $7" "${RESULT_FILE}" && {
        cat "${RESULT_FILE}" >> "${SCA_FILE}"
    } || {
        log "single_run.sh failed with the following parameters: ${loss} $4 $5 $6 $7 ${RESULT_FILE}"
    }
    rm "${RESULT_FILE}"
    loss=`echo "8 k $loss $3 + p q" | dc`
done

# done
log "single_loss.sh finished with exit status ok."
