/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include <omnetpp.h>

#include "pong_loose_scenario.hh"
#include "message_router.hh"
#include "instance_factory.hh"
#include "ball.hh"
#include "racket.hh"
#include "score.hh"
#include "consistency_loose.hh"


cPongLooseScenario::cPongLooseScenario(cHost& host)
: cPongScenario(host)
{ }

void cPongLooseScenario::initialize()
{
  cPongScenario::initialize();
  cLooseConsistency &con= *((cLooseConsistency *) &host().getConsistency());

  double ball_distance= 0.0;
  if (host().hasPar("ball_distance"))
    ball_distance= host().par("ball_distance");
  con.setMaxdist(s_ballID, ball_distance);


  double racket_distance= 0.0;
  if (host().hasPar("racket_distance"))
  racket_distance= host().par("racket_distance");
  if (m_isPlayerLeft)
    con.setMaxdist(s_racketLeftID, racket_distance);
  else
    con.setMaxdist(s_racketRightID, racket_distance);
}

void cPongLooseScenario::simulate(cWorld& world, double newtime,
        cLoadMeter &load) const
{
  bool balldir= true;
  
  if (world.isReplicated(s_ballID))
    {
    const cBall& ball = (cBall&) world.getInstance(s_ballID);
    balldir= ball.getVelocity()[0] < 0.0;
    }

  cPongScenario::simulate(world, newtime, load);

  if (!world.isReplicated(s_ballID)) return;

  const cBall& ball = (cBall&) world.getInstance(s_ballID);
  const bool movingAway = m_isPlayerLeft && ball.getVelocity()[0] >= 0
        || !m_isPlayerLeft && ball.getVelocity()[0] <0;
  const bool newballdir= ball.getVelocity()[0] < 0.0;

  if (balldir != newballdir)
    {
    if (movingAway)
        {
        // assume ownership
#       if VERBOSITY >= 1
        ev << "ball crossed my base line, " << host().name()
           << " taking ownership\n";
#       endif

        ((cLooseConsistency *) &host().getConsistency())->setOwner(s_ballID);
        }
    else
        {
        // yield ownership
#       if VERBOSITY >= 1
        ev << "ball crossed opponents base line, " << host().name()
           << " yielding ownership\n";
#       endif

        ((cLooseConsistency *) &host().getConsistency())->setOwner(s_ballID,
                                                                   false);
        }
      }

  load.addCPU(5);
}
