/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/

#ifndef __PONG_LOOSE_SCENARIO_H
#define __PONG_LOOSE_SCENARIO_H

#include "pong_scenario.hh"
#include "consistency_loose.hh"

namespace Condel {

// forward reference
class cMessageRouter;
class cRacket;
class cBall;

/// The Pong scenario with loose consistency.
/// This does things that are specific for loose consistency.  That's really
/// just the whole ownership and maximum player/ghost distance setting.
 
class cPongLooseScenario : public cPongScenario {
public:
  cPongLooseScenario(cHost& host);
  virtual ~cPongLooseScenario(void) {}

  /// Reset everything.
  /// Call cPongScenario::initialize(), then set the player/ghost distances
  /// for racket and ball.
  virtual void initialize(void);

  /// Perform deterministic actions.
  /// Call cPongScenario::simulate(), then take/yield ownership of the ball
  /// according to its new direction.  The player that the ball moves away
  /// from owns it.
  /// @param world The world that is advanced.
  /// @param newtime Time to advance the system to.  This should be in the
  /// future of the world's cWorld::simTime().  Given in seconds.
  /// @param load A tool to measure CPU and memory usage during the simulation.
  virtual void simulate(cWorld& world, double newtime, cLoadMeter &load) const;
};
  
};

#endif // __PONG_LOOSE_SCENARIO_H
