#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Sven Grottke, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# masterscript for the batching measurement script(distrib_batching.sh) and the analysis script(plot2d.pl)
# usage: batching_master.sh min_batching max_batching stepsize loss_rate avg_delay reception_delay local_lag timeout out_dir 

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}
# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 9 ] || {
    error "usage: batching_master.sh min_batching max_batching stepsize loss_rate avg_delay reception_delay local_lag timeout output_directory"
}

# set some paths
PONG_OPT_BASE="${CONDEL_ROOT}/experiments/pong_opt"
PONG_IDEAL_BASE="${CONDEL_ROOT}/experiments/pong_ideal"
PONG_OPT_RESULTS="${PONG_OPT_BASE}/results"
PONG_IDEAL_RESULTS="${PONG_IDEAL_BASE}/results"
PLOT_BASE="${CONDEL_ROOT}/util/plot"
PLOT_OUT_BASE="$9"
PLOT_OUT_PACKETS="${PLOT_OUT_BASE}/packets"
#PLOT_OUT_CPU="${PLOT_OUT_BASE}/cpu"
PLOT_OUT_DIV="${PLOT_OUT_BASE}/div"
PLOT_OUT_SCORE="${PLOT_OUT_BASE}/score"
PLOT_OUT_DISCO="${PLOT_OUT_BASE}/disco"
OUT_SCA="${PLOT_OUT_BASE}/sca"

# check if the output directories exist and are writable
[ -d "${PLOT_OUT_PACKETS}" -a -w "${PLOT_OUT_PACKETS}" ] || {
    error "$10 ${PLOT_OUT_PACKETS} does not exist or is not writable"
}
# [ -d "${PLOT_OUT_CPU}" -a -w "${PLOT_OUT_CPU}" ] || {
#     error "${PLOT_OUT_CPU} does not exist or is not writable"
# }
[ -d "${PLOT_OUT_DIV}" -a -w "${PLOT_OUT_DIV}" ] || {
    error "${PLOT_OUT_DIV} does not exist or is not writable"
}
[ -d "${PLOT_OUT_SCORE}" -a -w "${PLOT_OUT_SCORE}" ] || {
    error "${PLOT_OUT_SCORE} does not exist or is not writable"
}
[ -d "${PLOT_OUT_DISCO}" -a -w "${PLOT_OUT_DISCO}" ] || {
    error "${PLOT_OUT_DISCO} does not exist or is not writable"
}
[ -d "${OUT_SCA}" -a -w "${OUT_SCA}" ] || {
    error "${OUT_SCA} does not exist or is not writable"
}

# set the executable measurement- and plotscript
DISTRIB_BATCHING="${PONG_OPT_BASE}/scripts/distrib_batching.sh"
SCA_FILE="${PONG_OPT_RESULTS}/pong_opt.var_batching.sca"
IDEAL_FILE="${PONG_IDEAL_RESULTS}/pong_ideal.sca"
PLOT2D="${PLOT_BASE}/plot2d.pl"
FAILED_FILE="${CONDEL_ROOT}/experiments/common_files/measurement_failed.ps"

# set some variables
# we use the default player:
# player@2[0]
MIN_BATCHING=$1
MAX_BATCHING=$2
STEPSIZE=$3
LOSS_RATE=$4
DELAY=$5
RECEPTION=$6
LOCAL=$7
TIMEOUT=$8
batching="batching"
host_left="pong_opt.average"
host_eye="pong_opt.eye"
packets="avg. packets sent"
cpu="avg. cpu ops /tick"
div="D"
score="score achieved /s"
disco="avg. G"

# set the filename from results and the ps_file for output
GRAPHIC_PACKETS="${PLOT_OUT_PACKETS}/batching_packets_${MIN_BATCHING}_${MAX_BATCHING}_${LOSS_RATE}_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}.ps"
GRAPHIC_CPU="${PLOT_OUT_CPU}/batching_cpu_${MIN_BATCHING}_${MAX_BATCHING}_${LOSS_RATE}_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}.ps"
GRAPHIC_DIV="${PLOT_OUT_DIV}/batching_div_${MIN_BATCHING}_${MAX_BATCHING}_${LOSS_RATE}_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}.ps"
GRAPHIC_SCORE="${PLOT_OUT_SCORE}/batching_score_${MIN_BATCHING}_${MAX_BATCHING}_${LOSS_RATE}_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}.ps"
GRAPHIC_DISCO="${PLOT_OUT_DISCO}/batching_disco_${MIN_BATCHING}_${MAX_BATCHING}_${LOSS_RATE}_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}.ps"


# execute
log "starting the measurement script..."
${DISTRIB_BATCHING} "${MIN_BATCHING}" "${MAX_BATCHING}" "${STEPSIZE}" "${LOSS_RATE}" "${DELAY}" "${RECEPTION}" "${LOCAL}" "${TIMEOUT}" || {
    # copy an error message postscript file
    for dest in "${GRAPHIC_PACKETS}" "${GRAPHIC_DIV}" "${GRAPHIC_SCORE}" "${GRAPHIC_DISCO}"; do
        cp "${FAILED_FILE}" "${dest}"
    done

    log "measurement failed, aborting"
    exit 0
}

# copy the sca file under /batching/sca/.
SCA_FILE="${OUT_SCA}/pong_opt.var_batching_${MIN_BATCHING}_${MAX_BATCHING}_stepsize_${STEPSIZE}_${LOSS_RATE}_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}.sca"
cp "${PONG_OPT_RESULTS}/pong_opt.var_batching.sca" "${SCA_FILE}"

# plots
# packets              
log "starting the plot for \"${packets}\" at \"${host_left}\""
${PLOT2D} "${SCA_FILE}" "${MIN_BATCHING}" "${MAX_BATCHING}" "$batching" "${host_left}" "$packets" > "$GRAPHIC_PACKETS" || {
    cp "${FAILED_FILE}" "${GRAPHIC_PACKETS}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_BATCHING} ${MAX_BATCHING} $batching ${host_left} $packets > $GRAPHIC_PACKETS"
}

# # cpu              
# log "starting the plot for \"${cpu}\" at \"${host}\""
# ${PLOT2D} "${SCA_FILE}" "${MIN_BATCHING}" "${MAX_BATCHING}" "$batching" "${host_left}" "$cpu" > "${GRAPHIC_CPU}" || {
#     cp "${FAILED_FILE}" "${GRAPHIC_CPU}"
#     log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_BATCHING} ${MAX_BATCHING} $batching ${host_left} $cpu > $GRAPHIC_CPU"
# }

# div
log "starting the plot for \"${div}\" at \"${host_eye}\""
${PLOT2D} "${SCA_FILE}" "${MIN_BATCHING}" "${MAX_BATCHING}" "$batching" "$host_eye" "$div" > "$GRAPHIC_DIV" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DIV}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_BATCHING} ${MAX_BATCHING} $batching $host_eye $div > $GRAPHIC_DIV"
}

# score
log "starting the plot for \"${score}\" at \"${host}\""
${PLOT2D} "${SCA_FILE}" "${MIN_BATCHING}" "${MAX_BATCHING}" "$batching" "-i" "${IDEAL_FILE}" "${host_left}" "$score" > "$GRAPHIC_SCORE" || {
    cp "${FAILED_FILE}" "${GRAPHIC_SCORE}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_BATCHING} ${MAX_BATCHING} $batching -i ${IDEAL_FILE} ${host_left} $score > $GRAPHIC_SCORE"
}

# disco
log "starting the plot for \"${disco}\" at \"${host}\""
${PLOT2D} "${SCA_FILE}" "${MIN_BATCHING}" "${MAX_BATCHING}" "$batching" "${host_left}" "$disco" > "$GRAPHIC_DISCO" || {
    cp "${FAILED_FILE}" "${GRAPHIC_SCORE}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_BATCHING} ${MAX_BATCHING} $batching ${host_left} $disco > $GRAPHIC_DISCO"
}
