#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Sven Grottke, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# gain information about run-specific parameters, parse the parameters for
# further use in dist_search.sh 
# usage: 1) write the run parameters for the input from conf_exp_net.sh file
#           conf_exp_net.sh --write-parms loss delay reception_delay local_lag timeout netload
#        2) get the expected value of netload 
#           conf_exp_net.sh --get-expected-value loss delay reception_delay local_lag timeout netload
#        3) get default parameters for the batching
#           conf_exp_net.sh --get-defaults
#        4) get the range of batching 
#           conf_exp_net.sh --get-range
#        5) get the precision for batching
#           conf_exp_net.sh --get-defaults
#        6) get nr. of maxsteps for batching
#           conf_exp_net.sh --get-maxsteps
#        7) get the full name of netload in SCAFILE
#           conf_exp_net.sh --get-measure-name
# will be called by dist_search.sh

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}

usage() {
    echo "possible usages: conf_exp_net.sh --write-parms loss delay reception_delay local_lag timeout netload" >&2
    echo "                 conf_exp_net.sh --get-expected-value loss delay reception_delay local_lag timeout netload" >&2
    echo "                 conf_exp_net.sh --get-defaults" >&2
    echo "                 conf_exp_net.sh --get-range" >&2
    echo "                 conf_exp_net.sh --get-precision" >&2
    echo "                 conf_exp_net.sh --get-maxsteps" >&2
    echo "                 conf_exp_net.sh --get-measure-name" >&2
}

# set the information about netload
DEFAULTFILE="${CONDEL_ROOT}/experiments/pong_opt/.net_batching_defaults"
default1=`tail -1 ${DEFAULTFILE}`
default2=`cat ${DEFAULTFILE} | tail -2 | head -1`
range="0.0 0.5"
precision="0.05"
maxsteps="15"
MEASURE_NAME="avg. packets sent"

# check for calls with one parameter
[ $# -ge 1 ] || {
    usage
    exit 1
}
# get the range of update rate
[ "$1" == "--get-range" ] && {
    echo -e "$range"
    exit 0
}
# get the precision of update rate
[ "$1" == "--get-precision" ] && {
    echo -e "$precision"
    exit 0
}
# get the max. steps for the calculation of update rate
[ "$1" == "--get-maxsteps" ] && {
    echo -e "${maxsteps}"
    exit 0
}
# get the full name of netload in SCAFILE
[ "$1" == "--get-measure-name" ] && {
    echo -e "${MEASURE_NAME}"
    exit 0
}
# get the defaults to interpolate
[ "$1" == "--get-defaults" ] && {
    echo -e "${default1} ${default2}"
    exit 0
}

# calls with full parameters
[ $# = 7 ] || {
    usage
    exit 1
}
# write parameters for the input from conf_exp.sh file
[ "$1" = "--write-parms" ] && {
    shift
    echo -e "\"$1 $2 $3 $4 $5 X\""
}

# get the expected value of netload
[ "$1" == "--get-expected-value" ] && {
    shift
    echo -e "$6"
    exit 0
}

