#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Sven Grottke, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# varies according to the option(netload or div), leaving all other parameters constant
# usage: distrib_compare.sh option min_value max_value stepsize loss_opt delay_opt reception_opt local_opt timeout_opt loss_loose delay_loose ball_dist_loose racket_dist_loose

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}

# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"
source "${CONDEL_ROOT}/experiments/common_files/akaroa.sh"

# sanity check
[ $# = 13 ] || {
    error "usage: distrib_compare.sh option min_value max_value stepsize loss_opt delay_opt reception_opt local_opt timeout_opt loss_loose delay_loose ball_dist_loose racket_dist_loose"
}
    
# set some paths and variables
PONG_OPT_BASE="${CONDEL_ROOT}/experiments/pong_opt"
PONG_LOOSE_BASE="${CONDEL_ROOT}/experiments/pong_loose"
COMMON_BASE="${CONDEL_ROOT}/experiments/common_files"
DISTRIB_RUN="${COMMON_BASE}/distrib_run.sh"
DISTRIB_SEARCH="${CONDEL_ROOT}/experiments/common_files/distrib_search.sh"
OPT_RESULTS="${PONG_OPT_BASE}/results"
LOOSE_RESULTS="${PONG_LOOSE_BASE}/results"
LOOSE_SCA_FILE="${LOOSE_RESULTS}/pong_loose.var_compare.sca"
OPT_SCA_FILE="${OPT_RESULTS}/pong_opt.var_compare.sca"


# remove old results
[ -f "${LOOSE_SCA_FILE}" ] && rm "${LOOSE_SCA_FILE}"
[ -f "${OPT_SCA_FILE}" ] && rm "${OPT_SCA_FILE}"

# setup akaroa
log "starting Akaroa..."
ak_init
log "done."

# result file names will be stored here
FILES_CREATED=""

# run experiment for all parameter settings
option=$1
shift
min=$1
shift
max=$1
shift
stepsize=$1
shift
parm_set_loose="$6 $7 X $8 $9"
value=$min
# result file names will be stored here
FILES_CREATED=""
# we start the compare calculation in loose algorithm first 
while [ `echo "[1]sa $max $value !>a z p" | dc` = 1 ]; do
    log "measuring with (loose) $option value:${value} ..."
    RESULT_FILE="${LOOSE_RESULTS}/pong_loose_compare_${value}_parmset_$6_$7_$8_$9.sca"
    FILES_CREATED="${FILES_CREATED} ${RESULT_FILE}"
    parm_set=`echo ${parm_set_loose} | sed -e s/X/${value}/g`
    ${DISTRIB_SEARCH} $option ${PONG_LOOSE_BASE} 50000s "${parm_set}" ${RESULT_FILE} &
    value=`echo "8 k $value $stepsize + p q" | dc`
done

log "waiting for all simulations"
wait

# scratch all the values in sca for further calculation in opt
case $option in
"-net" ) measure_name="avg. packets sent";
         host="pong_loose.average";;
"-div" ) measure_name="D";
         host="pong_loose.eye";;
esac

log "all simulations finished, collecting results from ${FILES_CREATED}"
# collect results
for RESULT_FILE in ${FILES_CREATED}; do
    # does the result file exist?
    [ -f "${RESULT_FILE}" -a -r "${RESULT_FILE}" ] || {
        log "result file missing or unreadable: ${RESULT_FILE}"
        continue
    }
    grep "^scalar \"status\"[[:space:]]\"success\"[[:space:]]0$" "${RESULT_FILE}" 1>/dev/null 2>&1 && {
    # check if the value already exists
        values=`fgrep "\"$host\"" ${LOOSE_SCA_FILE} | grep "\"${measure_name}\"" | cut -sf 3`
        test_value=`fgrep "\"$host\"" ${RESULT_FILE} | grep "\"${measure_name}\"" | cut -sf 3-`
        value_exist="exist"
        for value in $values; do
            log "comparing $value with ${test_value}"
            value_exist=`perl -e "if( abs(${value} - ${test_value}) <= $stepsize ) {print \"\";} else {print \"exist\"}"`
        done
        [ "$value" -eq "${test_value}" ] &&  RESULT_FILE=""

        [ "${value_exist}" != "" ] &&  cat "${RESULT_FILE}" >> "${LOOSE_SCA_FILE}"
    } || {
        log "distrib_search.sh failed for ${RESULT_FILE}"
    }
#    [ "${RESULT_FILE}" != "" ] && rm "${RESULT_FILE}"
done

values=`fgrep "\"$host\"" ${LOOSE_SCA_FILE} | grep "\"${measure_name}\"" | cut -sf 3`

parm_set_opt="$1 $2 $3 $4 $5 X"
# result file names will be stored here
FILES_CREATED=""

# scratch all the values in sca for further calculation in opt
case $option in
"-net" ) measure_name="avg. packets sent";
         host="pong_opt.average";;
"-div" ) measure_name="D";
         host="pong_opt.eye";;
esac
# we start the compare calculation in loose algorithm first

for value in $values; do
    log "measuring with (opt) $option value:${value} ..."
    RESULT_FILE="${OPT_RESULTS}/pong_opt_compare_${value}_parmset_$1_$2_$3_$4_$5.sca"
    FILES_CREATED="${FILES_CREATED} ${RESULT_FILE}"
    parm_set=`echo ${parm_set_opt} | sed -e s/X/${value}/g`
    ${DISTRIB_SEARCH} $option ${PONG_OPT_BASE} 50000s "${parm_set}" ${RESULT_FILE} &
done

log "waiting for all simulations"
wait

log "all simulations finished, collecting results from ${FILES_CREATED}"
# collect results
for RESULT_FILE in ${FILES_CREATED}; do
    # does the result file exist?
    [ -f "${RESULT_FILE}" -a -r "${RESULT_FILE}" ] || {
        log "result file missing or unreadable: ${RESULT_FILE}"
        continue
    }
    if "^scalar \"status\"[[:space:]]\"success\"[[:space:]]0$" "${RESULT_FILE}" 1>/dev/null 2>&1 && 
    then
        #does the value exists in sca already?
        values=`fgrep "\"$host\"" ${OPT_SCA_FILE} | grep "\"${measure_name}\"" | cut -sf 3`
        test_value=`fgrep "\"$host\"" ${RESULT_FILE} | grep "\"${measure_name}\"" | cut -sf 3`
        value_exist="exist"
        for value in $values; do
            value_exist=`perl -e "if( abs(${value} - ${test_value}) <= $stepsize ) {print \"\";} else {print \"exist\"}"`   
        done
        [ "${value_exist}" != "" ] &&  cat "${RESULT_FILE}" >> "${OPT_SCA_FILE}"
    else
        log "distrib_search.sh failed for ${RESULT_FILE}"
    fi
    # rm "${RESULT_FILE}"
done

log "distrib_compare.sh finished with exit status ok."
