#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Sven Grottke, Jiehua Chen, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# varies the local lag, leaving all other parameters constant
# usage: distrib_local.sh min_local max_local stepsize loss_rate delay reception_delay timeout batching

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}

# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"
source "${CONDEL_ROOT}/experiments/common_files/akaroa.sh"

# sanity check
[ $# = 8 ] || {
    error "usage: distrib_local.sh min_local max_local stepsize loss_rate delay reception_delay timeout batching"
}
    
# set some paths and variables
PONG_OPT_BASE="${CONDEL_ROOT}/experiments/pong_opt"
DISTRIB_RUN="${CONDEL_ROOT}/experiments/common_files/distrib_run.sh"
RESULTS="${PONG_OPT_BASE}/results"
SCA_FILE="${RESULTS}/pong_opt.var_local.sca"

# remove old results
[ -f "${SCA_FILE}" ] && rm "${SCA_FILE}"

# setup akaroa
log "starting Akaroa..."
ak_init
log "done."

# result file names will be stored here
FILES_CREATED=""

# run experiment for all parameter settings
local=$1
while [ `echo "[1]sa $2 $local !>a z p" | dc` = 1 ]; do
    log "measuring with a local of ${local} ..."
    RESULT_FILE="${RESULTS}/pong_opt_$4_$5_$6_${local}_$7_$8.sca"
    FILES_CREATED="${FILES_CREATED} ${RESULT_FILE}"
    ${DISTRIB_RUN} "${PONG_OPT_BASE}" "50000s" "$4 $5 $6 $local $7 $8" "${RESULT_FILE}" &
    local=`echo "8 k $local $3 + p q" | dc`
done

log "waiting for all simulations"
wait

log "all simulations finished, collecting results from ${FILES_CREATED}"
# collect results
for RESULT_FILE in ${FILES_CREATED}; do
    # does the result file exist?
    [ -f "${RESULT_FILE}" -a -r "${RESULT_FILE}" ] || {
        log "result file missing or unreadable: ${RESULT_FILE}"
        continue
    }
    grep "^scalar \"status\"[[:space:]]\"success\"[[:space:]]0$" "${RESULT_FILE}" 1>/dev/null 2>&1 && {
        cat "${RESULT_FILE}" >> "${SCA_FILE}"
    } || {
        log "distrib_run.sh failed for ${RESULT_FILE}"
    }
    rm "${RESULT_FILE}"
done

# done
log "distrib_local.sh finished with exit status ok."
