#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Sven Grottke, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# masterscript for the whole experiments.
# usage: masterscript.sh out_dir

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}
# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 1 ] || {
    error "usage: masterscript.sh output_directory"
}

# set some paths
PONG_COMMON="${CONDEL_ROOT}/experiments/common_files"
PONG_OPT_BASE="${CONDEL_ROOT}/experiments/pong_opt"
PONG_IDEAL_BASE="${CONDEL_ROOT}/experiments/pong_ideal"
PONG_OPT_RESULTS="${PONG_OPT_BASE}/results"
PONG_IDEAL_RESULTS="${PONG_IDEAL_BASE}/results"
PONG_OPT_SCRIPT_BASE="${PONG_OPT_BASE}/scripts"
PLOT_BASE="${CONDEL_ROOT}/util/plot"
PLOT_OUT_BASE=$1
PLOT_OUT_LOSS_BASE="${PLOT_OUT_BASE}/loss"
PLOT_OUT_DELAY_BASE="${PLOT_OUT_BASE}/delay"
PLOT_OUT_RECEPTION_BASE="${PLOT_OUT_BASE}/reception"
PLOT_OUT_LOCAL_BASE="${PLOT_OUT_BASE}/local"
PLOT_OUT_TIMEOUT_BASE="${PLOT_OUT_BASE}/timeout"
PLOT_OUT_BATCHING_BASE="${PLOT_OUT_BASE}/batching"

# create directories
for parm in loss delay reception local timeout batching; do 
    for measurement in packets div score disco sca; do
        mkdir -p "${PLOT_OUT_BASE}/${parm}/${measurement}" || {
            error "can not create ${PLOT_OUT_BASE}/${parm}/${measurement}"
        }
    done
done

# set variables for the executables
DISTRIB_RUN_SCRIPT="${PONG_COMMON}/distrib_run.sh"
LOSS_MASTER="${PONG_OPT_SCRIPT_BASE}/loss_master.sh"
DELAY_MASTER="${PONG_OPT_SCRIPT_BASE}/delay_master.sh"
RECEPTION_MASTER="${PONG_OPT_SCRIPT_BASE}/reception_master.sh"
LOCAL_MASTER="${PONG_OPT_SCRIPT_BASE}/local_master.sh"
TIMEOUT_MASTER="${PONG_OPT_SCRIPT_BASE}/timeout_master.sh"
BATCHING_MASTER="${PONG_OPT_SCRIPT_BASE}/batching_master.sh"

# set ranges
# LOSS_RANGE="0 0.001 0.02"
# DELAY_RANGE="0 0.1 0.5"
# RECEPTION_RANGE="0 1"
# LOCAL_RANGE="0  1"
# TIMEOUT_RANGE="0 0.4 5"

# defaults only
LOSS_RANGE="0.001"
DELAY_RANGE="0.1"
RECEPTION_RANGE="0.0"
LOCAL_RANGE="0.0"
TIMEOUT_RANGE="0.25"
TIMEOUT_FACTOR_RANGE="2.5"
BATCHING_RANGE="0.0"

# for pong_ideal.sca
#$DISTRIB_RUN_SCRIPT "${PONG_IDEAL_BASE}" "50000s" "" "${PONG_IDEAL_RESULTS}/pong_ideal.sca"

# for loss_master.sh
for delay in ${DELAY_RANGE}; do
    for reception in ${RECEPTION_RANGE}; do
        for local in ${LOCAL_RANGE}; do
            for timeout in ${TIMEOUT_RANGE}; do
                for batching in ${BATCHING_RANGE}; do
                    $LOSS_MASTER 0 0.02 0.001 $delay $reception $local $timeout $batching ${PLOT_OUT_LOSS_BASE}
                done
            done
        done
    done
done
   
# for delay_master.sh
for loss in ${LOSS_RANGE}; do
    for reception in ${RECEPTION_RANGE}; do
        for local in ${LOCAL_RANGE}; do
            for timeout_factor in ${TIMEOUT_FACTOR_RANGE}; do
                for batching in ${BATCHING_RANGE}; do
                    ${DELAY_MASTER} 0 0.5 0.005 $loss $reception $local ${timeout_factor} $batching ${PLOT_OUT_DELAY_BASE}
                done
            done
        done
    done
done

# for reception_master.sh
for loss in ${LOSS_RANGE}; do
    for delay in ${DELAY_RANGE}; do
	max_reception=`echo "8 k ${delay} 3 * p q" | dc`
        for local in ${LOCAL_RANGE}; do
            for timeout in ${TIMEOUT_RANGE}; do
                for batching in ${BATCHING_RANGE}; do
                    $RECEPTION_MASTER 0 ${max_reception} 0.02 $loss $delay $local $timeout $batching ${PLOT_OUT_RECEPTION_BASE}
                done
            done
        done
    done
done

# for local_master.sh
for loss in ${LOSS_RANGE}; do
    for delay in ${DELAY_RANGE}; do
        for reception in ${RECEPTION_RANGE}; do
            for timeout in ${TIMEOUT_RANGE}; do
                for batching in ${BATCHING_RANGE}; do
                    $LOCAL_MASTER 0 0.2 0.02 $loss $delay $reception $timeout $batching ${PLOT_OUT_LOCAL_BASE}
                done
            done
        done
    done
done

# for timeout_master.sh
for loss in ${LOSS_RANGE}; do
    for delay in ${DELAY_RANGE}; do
        for reception in ${RECEPTION_RANGE}; do
            for local in ${LOCAL_RANGE}; do
                for batching in ${BATCHING_RANGE}; do
                    $TIMEOUT_MASTER 0.1 1 0.05 $loss $delay $reception $local $batching ${PLOT_OUT_TIMEOUT_BASE}
                done
            done
        done
    done
done

# for batching_master.sh
for loss in ${LOSS_RANGE}; do
    for delay in ${DELAY_RANGE}; do
        for reception in ${RECEPTION_RANGE}; do
            for local in ${LOCAL_RANGE}; do
                for timeout in ${TIMEOUT_RANGE}; do
                    ${BATCHING_MASTER} 0 0.5 0.005 $loss $delay $reception $local $timeout ${PLOT_OUT_BATCHING_BASE}
                done
            done
        done
    done
done


