#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Jan Sablatnig, Sven Grottke.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# masterscript for nr. bees measurement script(distrib_nr_bees.sh)
# usage: nr_bees_master.sh min_nr_bees max_nr_bees

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}
# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"
source "${CONDEL_ROOT}/experiments/common_files/akaroa.sh"

# sanity check
[ $# = 2 ] || {
    error "usage: nr_bees_master.sh min_nr_bees max_nr_bees"
}

# set some paths and variables
BASE="${CONDEL_ROOT}/experiments/swarm_ideal"
RESULTS="${BASE}/results"
SCA_FILE="$RESULTS/swarm_ideal.nrBees_min_$1_max_$2.sca"

# remove old results
[ -f "${SCA_FILE}" ] && rm "${SCA_FILE}"

# set the executable measurement- and plotscript
DISTRIB_RUN="${CONDEL_ROOT}/experiments/common_files/distrib_run.sh"
FAILED_FILE="${CONDEL_ROOT}/experiments/common_files/measurement_failed.ps"
# setup akaroa
log "starting Akaroa..."
ak_init
log "done."

# result file names will be stored here
FILES_CREATED=""

# run experiment for all parameter settings
nr_bees=$1
while [ `echo "[1]sa $2 ${nr_bees} !>a z p" | dc` = 1 ]; do
    log "measuring with ${nr_bees} bees..."
    RESULT_FILE="${RESULTS}/swarm_ideal.nrBees_${nr_bees}.sca" 
    FILES_CREATED="${FILES_CREATED} ${RESULT_FILE}"
    ${DISTRIB_RUN} "${BASE}" "5000s" "${nr_bees}" "${RESULT_FILE}" &
    nr_bees=`echo "8 k ${nr_bees} 2 * p q" | dc`
done

log "waiting for all simulations"
wait

log "all simulations finished, collecting results from ${FILES_CREATED}"
# collect results
for RESULT_FILE in ${FILES_CREATED}; do
    log "${RESULT_FILE}"
    # does the result file exist?
    [ -f "${RESULT_FILE}" -a -r "${RESULT_FILE}" ] || {
        log "result file missing or unreadable: ${RESULT_FILE}"
        continue
    }
    grep "^scalar \"status\"[[:space:]]\"success\"[[:space:]]0$" "${RESULT_FILE}" 1>/dev/null 2>&1 && {
        cat "${RESULT_FILE}" >> "${SCA_FILE}"
    } || {
        log "distrib_run.sh failed for ${RESULT_FILE}"
    }
done

# done
log "distrib_nr_bees.sh finished with exit status ok."
