#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jan Sablatnig, Jiehua Chen, Sven Grottke.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# masterscript for the loss rate measurement script(distrib_loss.sh) and the analysis script(plot2d.pl)
# usage: loss_master.sh min_loss max_loss stepsize delay regular_update queen_distance bee_distance nr_bees out_dir

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}
# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 9 ] || {
    error "usage: loss_master.sh min_loss max_loss stepsize delay regular_update queen_distance bee_distance nr_bees output_directory"
}

# set some paths
SWARM_LOOSE_BASE="${CONDEL_ROOT}/experiments/swarm_loose"
SWARM_IDEAL_BASE="${CONDEL_ROOT}/experiments/swarm_ideal"
SWARM_LOOSE_RESULTS="${SWARM_LOOSE_BASE}/results"
SWARM_IDEAL_RESULTS="${SWARM_IDEAL_BASE}/results"
PLOT_BASE="${CONDEL_ROOT}/util/plot"
PLOT_OUT_BASE="$9"
PLOT_OUT_CPU="${PLOT_OUT_BASE}/cpu"
PLOT_OUT_PACKETS="${PLOT_OUT_BASE}/packets"
PLOT_OUT_DIV="${PLOT_OUT_BASE}/div"
PLOT_OUT_SCORE="${PLOT_OUT_BASE}/score"
PLOT_OUT_DISCO="${PLOT_OUT_BASE}/disco"
OUT_SCA="${PLOT_OUT_BASE}/sca"

# check if the output directories exist and are writable
[ -d "${PLOT_OUT_CPU}" -a -w "${PLOT_OUT_CPU}" ] || {
    error "${PLOT_OUT_CPU} does not exist or is not writable"
}
[ -d "${PLOT_OUT_PACKETS}" -a -w "${PLOT_OUT_PACKETS}" ] || {
    error "${PLOT_OUT_PACKETS} does not exist or is not writable"
}
[ -d "${PLOT_OUT_DIV}" -a -w "${PLOT_OUT_DIV}" ] || {
    error "${PLOT_OUT_DIV} does not exist or is not writable"
}
[ -d "${PLOT_OUT_SCORE}" -a -w "${PLOT_OUT_SCORE}" ] || {
    error "${PLOT_OUT_SCORE} does not exist or is not writable"
}
[ -d "${PLOT_OUT_DISCO}" -a -w "${PLOT_OUT_DISCO}" ] || {
    error "${PLOT_OUT_DISCO} does not exist or is not writable"
}
[ -d "${OUT_SCA}" -a -w "${OUT_SCA}" ] || {
    error "${OUT_SCA} does not exist or is not writable"
}

# set the executable measurement- and plotscript
DISTRIB_LOSS="${SWARM_LOOSE_BASE}/scripts/distrib_loss.sh"
SCA_FILE="${SWARM_LOOSE_RESULTS}/swarm_loose.var_loss.sca"
PLOT2D="${PLOT_BASE}/plot2d.pl"
FAILED_FILE="${CONDEL_ROOT}/experiments/common_files/measurement_failed.ps"


# set some variables we use swarm_loose.average the default
# player, which represents the average of whole hosts:
MIN_LOSS=$1
MAX_LOSS=$2
STEPSIZE=$3
DELAY=$4
REGULAR_UPDATE=$5
QUEEN_DIST=$6
BEE_DIST=$7
NR_BEES=$8
loss="loss rate"
host="swarm_loose.average"
host_eye="swarm_loose.eye"
cpu="avg. cpu ops /tick"
packets="avg. packets sent"
div="D"
score="score achieved /s"
disco="avg. G"

# scafile for ideal
IDEAL_FILE="${SWARM_IDEAL_RESULTS}/swarm_ideal.nrBees_${NR_BEES}.sca" 

# set the filenames from results and the ps_file for output
GRAPHIC_PACKETS="${PLOT_OUT_PACKETS}/loss_packets_${MIN_LOSS}_${MAX_LOSS}_${DELAY}_${REGULAR_UPDATE}_${QUEEN_DIST}_${BEE_DIST}_nrBees_${NR_BEES}.ps"
GRAPHIC_CPU="${PLOT_OUT_CPU}/loss_cpu_${MIN_LOSS}_${MAX_LOSS}_${DELAY}_${REGULAR_UPDATE}_${QUEEN_DIST}_${BEE_DIST}_nrBees_${NR_BEES}.ps"
GRAPHIC_DIV="${PLOT_OUT_DIV}/loss_div_${MIN_LOSS}_${MAX_LOSS}_${DELAY}_${REGULAR_UPDATE}_${QUEEN_DIST}_${BEE_DIST}_nrBees_${NR_BEES}.ps"
GRAPHIC_SCORE="${PLOT_OUT_SCORE}/loss_score_${MIN_LOSS}_${MAX_LOSS}_${DELAY}_${REGULAR_UPDATE}_${QUEEN_DIST}_${BEE_DIST}_nrBees_${NR_BEES}.ps"
GRAPHIC_DISCO="${PLOT_OUT_DISCO}/loss_disco_${MIN_LOSS}_${MAX_LOSS}_${DELAY}_${REGULAR_UPDATE}_${QUEEN_DIST}_${BEE_DIST}_nrBees_${NR_BEES}.ps"

# execute 
log "starting the measurement script..."
${DISTRIB_LOSS} "${MIN_LOSS}" "${MAX_LOSS}" "${STEPSIZE}" "${DELAY}" "${REGULAR_UPDATE}" "${QUEEN_DIST}" "${BEE_DIST}" "${NR_BEES}" || {
    # copy an error message postscript file
    for dest in "${GRAPHIC_PACKETS}" "${GRAPHIC_CPU}" "${GRAPHIC_DIV}" "${GRAPHIC_SCORE}" "${GRAPHIC_DISCO}"; do
        cp "${FAILED_FILE}" "${dest}"
    done

    log "measurement failed, aborting"
    exit 0
}

# copy the sca under /loss/sca/.
SCA_FILE="${OUT_SCA}/swarm_loose.var_loss_${MIN_LOSS}_${MAX_LOSS}_stepsize_${STEPSIZE}_${DELAY}_${REGULAR_UPDATE}_${QUEEN_DIST}_${BEE_DIST}_nrBees_${NR_BEES}.sca"
cp "${SWARM_LOOSE_RESULTS}/swarm_loose.var_loss.sca" "${SCA_FILE}"

# plots
# packets 
log "starting the plot for \"${packets}\" at \"${host}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOSS}" "${MAX_LOSS}" "$loss" "$host" "$packets" > "$GRAPHIC_PACKETS" || {
    cp "${FAILED_FILE}" "${GRAPHIC_PACKETS}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOSS} ${MAX_LOSS} $loss $host $packets > $GRAPHIC_PACKETS"
}

# cpu 
log "starting the plot for \"${cpu}\" at \"${host}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOSS}" "${MAX_LOSS}" "$loss" "$host" "$cpu" > "${GRAPHIC_CPU}" || {
    cp "${FAILED_FILE}" "${GRAPHIC_CPU}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOSS} ${MAX_LOSS} $loss $host $cpu > $GRAPHIC_CPU"
}

# div
log "starting the plot for \"${div}\" at \"${host_eye}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOSS}" "${MAX_LOSS}" "$loss" "$host_eye" "$div" > "$GRAPHIC_DIV" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DIV}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOSS} ${MAX_LOSS} $loss $host_eye $div > $GRAPHIC_DIV"
}

# score
log "starting the plot for \"${score}\" at \"${host}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOSS}" "${MAX_LOSS}" "$loss" "-i" "${IDEAL_FILE}" "$host" "$score" > "$GRAPHIC_SCORE" || {
    cp "${FAILED_FILE}" "${GRAPHIC_SCORE}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOSS} ${MAX_LOSS} $loss -i ${IDEAL_FILE} $host $score > $GRAPHIC_SCORE"
}

# disco
log "starting the plot for \"${disco}\" at \"${host}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOSS}" "${MAX_LOSS}" "$loss" "$host" "$disco" > "$GRAPHIC_DISCO" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DISCO}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOSS} ${MAX_LOSS} $loss $host $disco > $GRAPHIC_DISCO"
}
