#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jan Sablatnig, Jiehua Chen, Sven Grottke.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# masterscript for the whole experiments.
# usage: masterscript.sh nr_bees out_dir

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}
# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 1 ] || {
    error "usage: masterscript.sh output_directory"
}

# set some paths
SWARM_COMMON="${CONDEL_ROOT}/experiments/common_files"
SWARM_LOOSE_BASE="${CONDEL_ROOT}/experiments/swarm_loose"
SWARM_IDEAL_BASE="${CONDEL_ROOT}/experiments/swarm_ideal"
SWARM_LOOSE_RESULTS="${SWARM_LOOSE_BASE}/results"
SWARM_IDEAL_RESULTS="${SWARM_IDEAL_BASE}/results"
SWARM_LOOSE_SCRIPT_BASE="${SWARM_LOOSE_BASE}/scripts"
PLOT_BASE="${CONDEL_ROOT}/util/plot"
PLOT_OUT_BASE=$1
PLOT_OUT_LOSS_BASE="${PLOT_OUT_BASE}/loss"
PLOT_OUT_DELAY_BASE="${PLOT_OUT_BASE}/delay"
PLOT_OUT_UPDATE_BASE="${PLOT_OUT_BASE}/update"
PLOT_OUT_QUEEN_BASE="${PLOT_OUT_BASE}/queen"
PLOT_OUT_BEE_BASE="${PLOT_OUT_BASE}/bee"
PLOT_OUT_NR_BEES_BASE="${PLOT_OUT_BASE}/nr_bees"

# create directories
for parm in loss delay update queen bee nr_bees; do 
    for measurement in packets cpu div score disco sca; do
        mkdir -p "${PLOT_OUT_BASE}/${parm}/${measurement}" || {
            error "can not create ${PLOT_OUT_BASE}/${parm}/${measurement}"
        }
    done
done

# set variables for the executables
IDEAL_SCRIPT="${SWARM_IDEAL_BASE}/scripts/nr_bees_master.sh"
LOSS_MASTER="${SWARM_LOOSE_SCRIPT_BASE}/loss_master.sh"
DELAY_MASTER="${SWARM_LOOSE_SCRIPT_BASE}/delay_master.sh"
UPDATE_MASTER="${SWARM_LOOSE_SCRIPT_BASE}/update_master.sh"
QUEEN_MASTER="${SWARM_LOOSE_SCRIPT_BASE}/queen_master.sh"
BEE_MASTER="${SWARM_LOOSE_SCRIPT_BASE}/bee_master.sh"
NR_BEES_MASTER="${SWARM_LOOSE_SCRIPT_BASE}/nr_bees_master.sh"

# set ranges
# LOSS_RANGE="0 0.001 0.02"
# DELAY_RANGE="0 0.1 0.5"
# UPDATE_RANGE="0.05 1 6.4"
# QUEEN_RANGE="0 5 1024"
# BEE_RANGE="0 25 512"

# defaults only
LOSS_RANGE="0.001"
DELAY_RANGE="0.1"
UPDATE_RANGE="0.35"
QUEEN_RANGE="5"
BEE_RANGE="5"
NR_BEES="8"

# for swarm_ideal.sca
${IDEAL_SCRIPT} 5000 1 16

# for loss_master.sh
$LOSS_MASTER 0 0.02 0.001 0.1 0 5 5 ${NR_BEES} ${PLOT_OUT_LOSS_BASE} 

# vary bee distance with no regular update distance!
${BEE_MASTER} 1 64 0.001 0.1 0 5 ${NR_BEES} ${PLOT_OUT_BEE_BASE} 

# for update_master.sh
$UPDATE_MASTER 0.15 10 0.001 0.1 10240 10240 ${NR_BEES} ${PLOT_OUT_UPDATE_BASE}

# for loss_master.sh
for delay in ${DELAY_RANGE}; do
    for update in ${UPDATE_RANGE}; do
        for queen in ${QUEEN_RANGE}; do
            for bee in ${BEE_RANGE}; do :
               $LOSS_MASTER 0  0.02 0.001 $delay $update $queen $bee ${NR_BEES} ${PLOT_OUT_LOSS_BASE}
            done
        done
    done
done

# for delay_master.sh
for loss in ${LOSS_RANGE}; do
    for update in ${UPDATE_RANGE}; do
        for queen in ${QUEEN_RANGE}; do
            for bee in ${BEE_RANGE}; do :
                $DELAY_MASTER 0  0.5 0.005 $loss $update $queen $bee ${NR_BEES} ${PLOT_OUT_DELAY_BASE}
            done
        done
    done
done

# for update_master.sh
for loss in ${LOSS_RANGE}; do
    for delay in ${DELAY_RANGE}; do
        for queen in ${QUEEN_RANGE}; do
            for bee in ${BEE_RANGE}; do :
                $UPDATE_MASTER 0.15 10 $loss $delay $queen $bee ${NR_BEES} ${PLOT_OUT_UPDATE_BASE}
            done
        done
    done
done

# we don't use queen update distance any more
# # for queen_master.sh
# for loss in ${LOSS_RANGE}; do
#     for delay in ${DELAY_RANGE}; do
#         for update in ${UPDATE_RANGE}; do
#             for bee in ${BEE_RANGE}; do :
#                 $QUEEN_MASTER 1 1024 $loss $delay $update $bee ${NR_BEES} ${PLOT_OUT_QUEEN_BASE}
#             done
#         done
#     done
# done

# for bee_master.sh
for loss in ${LOSS_RANGE}; do
    for delay in ${DELAY_RANGE}; do
        for update in ${UPDATE_RANGE}; do
            for queen in ${QUEEN_RANGE}; do :
                $BEE_MASTER 1 64 $loss $delay $update $queen ${NR_BEES} ${PLOT_OUT_BEE_BASE}
            done
        done
    done
done

# for nr_bees_master.sh
for loss in ${LOSS_RANGE}; do
    for delay in ${DELAY_RANGE}; do
        for update in ${UPDATE_RANGE}; do
            for queen in ${QUEEN_RANGE}; do
                for bee in ${BEE_RANGE}; do :
                    $NR_BEES_MASTER 1 16 $loss $delay $update $queen $bee ${PLOT_OUT_NR_BEES_BASE}
                done
            done
        done
    done
done






