#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# run the pong_loose experiment with a given parameter set
# usage: single_run.sh loss_rate delay regular_update ball_distance racket_distance
# delay is given as the mean delay, at the moment an exponential distribution
# will be used

# error function
error() {
    echo "error: $@"
    echo "exiting"
    exit 1
}

# sanity check
[ $# = 5 ] || {
    error "usage: single_run.sh loss_rate delay regular_update ball_distance racket_distance"
}
    
# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}

# set some paths and variables
PONG_LOOSE_BASE="${CONDEL_ROOT}/experiments/pong_loose"
BINARY="${PONG_LOOSE_BASE}/bin/pong_loose"
CONF_EXP="${PONG_LOOSE_BASE}/scripts/conf_exp.sh"
RESULTS="${PONG_LOOSE_BASE}/results"
RESULT_FILE="${RESULTS}/omnetpp.sca"
INIFILE="${PONG_LOOSE_BASE}/pong_loose_dyna.ini"
NEDFILE="${PONG_LOOSE_BASE}/pong_loose_dyna.ned"

# create NED file
${CONF_EXP} --create-ned $1 $2 $3 $4 $5 > "${NEDFILE}" || error "could not create NED file ${NEDFILE}"

# create .ini file
cat >"${INIFILE}" <<EOF
# pong_loose-specific init

include ${CONDEL_ROOT}/omnetpp.ini

[General]
preload-ned-files=${NEDFILE}
network=pong_loose
output-vector-file = ${RESULTS}/omnetpp.vec
output-scalar-file = ${RESULTS}/omnetpp.sca
sim-time-limit = 50000s

EOF
${CONF_EXP} --create-ini $1 $2 $3 $4 $5 >>"${INIFILE}" || error "could not create INI file ${INIFILE}"

# execute
${BINARY} -u Cmdenv -f ${INIFILE} || {
    echo "simulation failed"
    exit 1
}

# write pseudo module containing the run parameters
${CONF_EXP} --write-parms $1 $2 $3 $4 $5 >> "${RESULT_FILE}" || error "could not write run parameters to ${RESULT_FILE}"

# write pseudo module containing variance and disco weights
VAR_WEIGHTS=`${CONF_EXP} --get-variance-weights` || error "could not get variance weights"
echo -e "scalar \"parm\"\t\"variance weights\"\t\"${VAR_WEIGHTS}\"" >> "${RESULT_FILE}"
DISCO_WEIGHTS=`${CONF_EXP} --get-disco-weights` || error "could not get disco weights"
echo -e "scalar \"parm\"\t\"disco weights\"\t\"${DISCO_WEIGHTS}\"" >> "${RESULT_FILE}"

# cleanup
rm "${INIFILE}" "${NEDFILE}"

