#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Jan Sablatnig, Sven Grottke.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# masterscript for the compare measurement script(distrib_compare.sh) and the analysis script for plotting
# usage: compare_master.sh min_compare max_compare stepsize loss_rate_opt avg_delay_opt reception_delay_opt local_lag_opt timeout_opt nr_bees_opt loss_loose delay_loose queen_dist_loose bee_dist_loose nr_bees_loose out_dir

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}
# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 15 ] || {
    error "usage: compare_master.sh min_compare max_compare stepsize loss_rate_opt avg_delay_opt reception_delay_opt local_lag_opt timeout_opt nr_bees_opt loss_loose delay_loose queen_dist_loose bee_dist_loose nr_bees_loose output_directory"
}

# set some paths
COMMON_BASE="${CONDEL_ROOT}/experiments/common_files"
SWARM_OPT_BASE="${CONDEL_ROOT}/experiments/swarm_opt"
SWARM_LOOSE_BASE="${CONDEL_ROOT}/experiments/swarm_loose"
SWARM_IDEAL_BASE="${CONDEL_ROOT}/experiments/swarm_ideal"
SWARM_OPT_RESULTS="${SWARM_OPT_BASE}/results"
SWARM_LOOSE_RESULTS="${SWARM_LOOSE_BASE}/results"
SWARM_IDEAL_RESULTS="${SWARM_IDEAL_BASE}/results"
PLOT_BASE="${CONDEL_ROOT}/util/plot"
PLOT_OUT_BASE="${15}"
PLOT_OUT_PACKETS="${PLOT_OUT_BASE}/packets"
PLOT_OUT_DIVERGENCE="${PLOT_OUT_BASE}/div"
OUT_SCA="${PLOT_OUT_BASE}/sca"
LOOSE_NET_SCA="${OUT_SCA}/swarm_loose.var_net.sca"
LOOSE_DIV_SCA="${OUT_SCA}/swarm_loose.var_div.sca"
OPT_NET_SCA="${OUT_SCA}/swarm_opt.var_net.sca"
OPT_DIV_SCA="${OUT_SCA}/sca/swarm_opt.var_div.sca"

# check if the output directories exist and are writable
[ -d "${PLOT_OUT_PACKETS}" -a -w "${PLOT_OUT_PACKETS}" ] || {
    error "$10 ${PLOT_OUT_PACKETS} does not exist or is not writable"
}
[ -d "${PLOT_OUT_DIVERGENCE}" -a -w "${PLOT_OUT_DIVERGENCE}" ] || {
    error "${PLOT_OUT_DIVERGENCE} does not exist or is not writable"
}
[ -d "${OUT_SCA}" -a -w "${OUT_SCA}" ] || {
    error "${OUT_SCA} does not exist or is not writable"
}

# set the executable measurement- and plotscript
DISTRIB_COMPARE="${SWARM_OPT_BASE}/scripts/distrib_compare.sh"
PLOT2D="${PLOT_BASE}/plot2d.pl"
FAILED_FILE="${CONDEL_ROOT}/experiments/common_files/measurement_failed.ps"
GNUPLOT_DIV="${SWARM_OPT_BASE}/scripts/.packets_D.stub"
GNUPLOT_DISCO="${SWARM_OPT_BASE}/scripts/.packets_G.stub"

# set some variables we use swarm_opt.average the default
# player, which represents the average of whole hosts:
MIN_COMPARE=$1
MAX_COMPARE=$2
STEPSIZE=$3
LOSS_RATE_OPT=$4
DELAY_OPT=$5
RECEPTION_OPT=$6
LOCAL_OPT=$7
TIMEOUT_OPT=$8
NR_BEES_OPT=$9
LOSS_RATE_LOOSE=${10}
DELAY_LOOSE=${11}
QUEEN_DIST_LOOSE=${12}
BEE_DIST_LOOSE=${13}
NR_BEES_LOOSE=${14}
opt_host="swarm_opt.average"
opt_host_eye="swarm_opt.eye"
loose_host="swarm_loose.average"
loose_host_eye="swarm_loose.eye"
packets="avg. packets sent"
update="regular update rate"
disco="avg. G"
batching="batching"
divergence="D"
linewidth="3"

# scafile for ideal
IDEAL_FILE="${SWARM_IDEAL_RESULTS}/swarm_ideal.nrBees_${NR_BEES_OPT}.sca"

# set the filename from results and the ps_file for output
tmp_net_opt="${PLOT_OUT_PACKETS}/net_opt.ps"
tmp_div_opt="${PLOT_OUT_PACKETS}/div_opt.ps"
tmp_net_loose="${PLOT_OUT_PACKETS}/net_loose.ps"
tmp_div_loose="${PLOT_OUT_PACKETS}/net_loose.ps"
GRAPHIC_PACKETS_DIV="${PLOT_OUT_PACKETS}/compare_packets_div_${MIN_COMPARE}_${MAX_COMPARE}.optparms_${LOSS_OPT_RATE}_${DELAY_OPT}_${RECEPTION_OPT}_${LOCAL_OPT}_${TIMEOUT_OPT}_nrBees_${NR_BEES_OPT}.looseparms_${LOSS_RATE_LOOSE}_${DELAY_LOOSE}_${QUEEN_DIST_LOOSE}_${BEE_DIST_LOOSE}_nrBess_${NR_BEES_LOOSE}.ps"
GRAPHIC_PACKETS_DISCO="${PLOT_OUT_PACKETS}/compare_packets_disco_${MIN_COMPARE}_${MAX_COMPARE}.optparms_${LOSS_OPT_RATE}_${DELAY_OPT}_${RECEPTION_OPT}_${LOCAL_OPT}_${TIMEOUT_OPT}_nrBees_${NR_BEES_OPT}.looseparms_${LOSS_RATE_LOOSE}_${DELAY_LOOSE}_${QUEEN_DIST_LOOSE}_${BEE_DIST_LOOSE}_nrBess_${NR_BEES_LOOSE}.ps"
#GRAPHIC_DIVERGENCE="${PLOT_OUT_DIVERGENCE}/compare_divergence_${MIN_COMPARE}_${MAX_COMPARE}_${LOSS_RATE_OPT}_${DELAY_OPT}_${RECEPTION_OPT}_${LOCAL_OPT}_${TIMEOUT_OPT}_nrBees_${NR_BEES_OPT}.looseparms_${LOSS_RATE_LOOSE}_${DELAY_LOOSE}_${QUEEN_DIST_LOOSE}_${BEE_DIST_LOOSE}_nrBess_${NR_BEES_LOOSE}.ps"


# execute
log "starting the measurement script for netload in loose and opt..."
${DISTRIB_COMPARE} "-net" "${MIN_COMPARE}" "${MAX_COMPARE}" "${STEPSIZE}" "${LOSS_RATE_OPT}" "${DELAY_OPT}" "${RECEPTION_OPT}" "${LOCAL_OPT}" "${TIMEOUT_OPT}" "${NR_BEES_OPT}" "${LOSS_RATE_LOOSE}" "${DELAY_LOOSE}" "${QUEEN_DIST_LOOSE}" "${BEE_DIST_LOOSE}" "${NR_BEES_LOOSE}"|| {
    # copy an error message postscript file
    for dest in "${GRAPHIC_PACKETS_DIV}" "${GRAPHIC_PACKETS_DISCO}"; do
        cp "${FAILED_FILE}" "${dest}"
    done

    log "measurement failed, aborting"
    exit 0
}

# copy the file into netload directory
# find the range for "regular update rate" and "batching"
# -------------------------------------------------------------------------------------------------------------------------------------------
cp "${SWARM_LOOSE_RESULTS}/swarm_loose.var_compare.sca" "${LOOSE_NET_SCA}"
packets_data="${PLOT_OUT_PACKETS}/loose_net.packets.data"
divergence_data="${PLOT_OUT_PACKETS}/loose_net.divergence.data"
disco_data="${PLOT_OUT_PACKETS}/loose_net.disco.data"
loose_net_div="${PLOT_OUT_PACKETS}/loose_net.packets_div.data"
loose_net_disco="${PLOT_OUT_PACKETS}/loose_net.packets_disco.data"
# scratch netload divergence and discontinuity for plotting
fgrep "${loose_host}" ${LOOSE_NET_SCA} | grep "\"${packets}\"" | cut -sf 3 > ${packets_data}
fgrep "${loose_host_eye}" ${LOOSE_NET_SCA} | grep "\"${divergence}\"" | cut -sf 3 > ${divergence_data}
fgrep "${loose_host}" ${LOOSE_NET_SCA} | grep "\"${disco}\"" | cut -sf 3  > ${disco_data}

# create plotdata for plots x-achse: netload & y-achse: DIV or DISCO
paste "${packets_data}" "${divergence_data}" | sort -n > ${loose_net_div} 
paste "${packets_data}" "${disco_data}" | sort -n > ${loose_net_disco}

MIN_COMPARE=`fgrep "\"parm\"" ${LOOSE_NET_SCA} | grep "\"${update}\"" | cut -sf 3 | sort -n | head -1`
MAX_COMPARE=`fgrep "\"parm\"" ${LOOSE_NET_SCA} | grep "\"${update}\"" | cut -sf 3 | sort -n | tail -1`

# packets for loose              
log "starting the plot for loose \"${packets}\" at \"${loose_host}\""
${PLOT2D} "${LOOSE_NET_SCA}" "${MIN_COMPARE}" "${MAX_COMPARE}" "$update" "$loose_host" "$packets" > "${tmp_net_loose}" || {
    cp "${FAILED_FILE}" "${tmp_net_loose}"
    log "WARNING: plot2d failed with the following parameters:${PLOT2D} ${LOOSE_NET_SCA} ${MIN_COMPARE} ${MAX_COMPARE} $update $loose_host $packets > ${tmp_net_loose}"
}
# divergence for loose
log "starting the plot for loose \"${divergence}\" at \"${loose_host_eye}\""
${PLOT2D} "${LOOSE_NET_SCA}" "${MIN_COMPARE}" "${MAX_COMPARE}" "$update" "${loose_host_eye}" "$divergence" > "${tmp_div_loose}" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DIVERGENCE}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_COMPARE} ${MAX_COMPARE} $update ${loose_host_eye} $divergence > ${tmp_div_loose}"
}

# now we take care of the opt case
cp "${SWARM_OPT_RESULTS}/swarm_opt.var_compare.sca" "${OPT_NET_SCA}"
packets_data="${PLOT_OUT_PACKETS}/opt_net.packets.data"
divergence_data="${PLOT_OUT_PACKETS}/opt_net.divergence.data"
disco_data="${PLOT_OUT_PACKETS}/opt_net.disco.data"
opt_net_div="${PLOT_OUT_PACKETS}/opt_net.packets_div.data"
opt_net_disco="${PLOT_OUT_PACKETS}/opt_net.packets_disco.data"
# scratch netload divergence and discontinuity for plotting
fgrep "${opt_host}" ${OPT_NET_SCA} | grep "\"${packets}\"" | cut -sf 3 > ${packets_data}
fgrep "${opt_host_eye}" ${OPT_NET_SCA} | grep "\"${divergence}\"" | cut -sf 3 > ${divergence_data}
fgrep "${opt_host}" ${OPT_NET_SCA} | grep "\"${disco}\"" | cut -sf 3  > ${disco_data}

# create plotdata for plots x-achse: netload & y-achse: DIV or DISCO
paste "${packets_data}" "${divergence_data}" | sort -n > ${opt_net_div} 
paste "${packets_data}" "${disco_data}" | sort -n > ${opt_net_disco}
MIN_COMPARE=`fgrep "\"${batching}\"" "${OPT_NET_SCA}" | cut -sf 3 | sort -n | head -1`
MAX_COMPARE=`fgrep "\"${batching}\"" "${OPT_NET_SCA}" | cut -sf 3 | sort -n | tail -1`
# packets for opt              
log "starting the plot for opt \"${packets}\" at \"${opt_host}\""
${PLOT2D} "${OPT_NET_SCA}" "${MIN_COMPARE}" "${MAX_COMPARE}" "$batching" "${opt_host}" "$packets" > "${tmp_net_opt}" || {
    cp "${FAILED_FILE}""${tmp_net_opt}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${OPT_NET_SCA} ${MIN_COMPARE} ${MAX_COMPARE} $batching $opt_host $packets > ${tmp_net_opt}"
}
# divergence for opt
log "starting the plot for opt \"${divergence}\" at \"${opt_host_eye}\""
${PLOT2D} "${OPT_NET_SCA}" "${MIN_COMPARE}" "${MAX_COMPARE}" "$batching" "${opt_host_eye}" "$divergence" > "${tmp_div_opt}" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DIVERGENCE}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${OPT_NET_SCA} ${MIN_COMPARE} ${MAX_COMPARE} $batching ${opt_host_eye} $divergence > ${tmp_div_opt}"
}

# combine data
plotscript_div="${PLOT_OUT_PACKETS}/packets_div.gnuplot"
cp "${GNUPLOT_DIV}" "${plotscript_div}"
echo "set title \"${loose_net_div}\"" | sed -e s/_/\\\\\\\\_/g >> ${plotscript_div}
echo "plot \"${loose_net_div}\" title \"loose\" with linespoints lw ${linewidth},  \"${opt_net_div}\" title \"opt\" with linespoints lw ${linewidth}" >> ${plotscript_div}
cat ${plotscript_div}
plotscript_disco="${PLOT_OUT_PACKETS}/packets_disco.gnuplot"
cp "${GNUPLOT_DISCO}" "${plotscript_disco}"
echo "set title \"${loose_net_disco}\"" | sed -e s/_/\\\\\\\\_/g>> ${plotscript_disco}
echo "plot \"${loose_net_disco}\" title \"loose\" with linespoints lw ${linewidth}, \"${opt_net_disco}\" title \"opt\" with linespoints lw ${linewidth}" >> ${plotscript_disco}
cat ${plotscript_disco}

gnuplot ${plotscript_disco} > ${GRAPHIC_PACKETS_DISCO}
gnuplot ${plotscript_div} > ${GRAPHIC_PACKETS_DIV}

# # -------------------------------------------------------------------------------------------------------------------------------------------
# # We observe this in DIVERGENCE case
# log "starting the measurement script for standard diviation..."
# ${DISTRIB_COMPARE} "-div" "$1" "$2" "${STEPSIZE}" "${LOSS_RATE_OPT}" "${DELAY_OPT}" "${RECEPTION_OPT}" "${LOCAL_OPT}" "${TIMEOUT_OPT}" "${NR_BEES_OPT}" "${LOSS_RATE_LOOSE}" "${DELAY_LOOSE}" "${QUEEN_DIST_LOOSE}" "${BEE_DIST_LOOSE}" "${NR_BEES_LOOSE}"|| {
#     # copy an error message postscript file
#     for dest in "${GRAPHIC_PACKETS}" "${GRAPHIC_DIVERGENCE}" ; do
#         cp "${FAILED_FILE}" "${dest}"
#     done

#     log "measurement failed, aborting"
#     exit 0
# }
# # copy the file into netload directory
# # find the range for "regular update rate" and "batching" 
# cp "${SWARM_LOOSE_REULTS}/swarm_loose.var_compare.sca" "${LOOSE_DIV_SCA}"
# MIN_COMPARE=`fgrep "\"parm\"" ${LOOSE_NET_SCA} | grep "\"${update}\"" | cut -c 37- | sort -n | head -1`
# MAX_COMPARE=`fgrep "\"parm\"" ${LOOSE_NET_SCA} | grep "\"${update}\"" | cut -c 37- | sort -n | tail -1`
# # packets for loose
# log "starting the plot for loose \"${packets}\" at \"${loose_host}\""
# ${PLOT2D} "${LOOSE_NET_SCA}" "${MIN_COMPARE}" "${MAX_COMPARE}" "$update" "$loose_host" "$packets" > "${tmp_net_loose}" || {
#     cp "${FAILED_FILE}" "${tmp_net_loose}"
#     log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${LOOSE_NET_SCA} ${MIN_COMPARE} ${MAX_COMPARE} $update $loose_host $packets > ${tmp_net_loose}"
# }
# mv "${packets_data}" "${PLOT_OUT_PACKETS/update_packets.data"
# # divergence for loose
# log "starting the plot for loose \"${divergence}\" at \"${loose_host_eye}\""
# ${PLOT2D} "${LOOSE_NET_SCA}" "${MIN_COMPARE}" "${MAX_COMPARE}" "$update" "$loose_host_eye" "$divergence" > "${tmp_div_loose}" || {
#     cp "${FAILED_FILE}" "${GRAPHIC_DIVERGENCE}"
#     log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_COMPARE} ${MAX_COMPARE} $update ${loose_host_eye} $divergence > ${tmp_div_loose}"
# }
# mv "${divergence_data}" "${PLOT_OUT_PACKETS/update_divergence.data"

# cp "${SWARM_OPT_REULTS}/swarm_opt.var_compare.sca" "${OPT_DIV_SCA}/."

# log "starting the plot for loose \"${packets}\" at \"${loose_host}\""
# ${PLOT2D} "${LOOSE_NET_SCA}" "${MIN_COMPARE}" "${MAX_COMPARE}" "$update" "$loose_host" "$packets" > "${tmp_net_loose}" || {
#     cp "${FAILED_FILE}""${GRAPHIC_PACKETS}"
#     log "WARNING: plot2d failed with the following parameters:${PLOT2D} ${LOOSE_NET_SCA} ${MIN_COMPARE} ${MAX_COMPARE} $update $loose_host $packets > ${tmp_net_loose}"
# }
# mv "${packets_data}" "${PLOT_OUT_PACKETS/update_packets.data"

# cp "${SWARM_OPT_REULTS}/swarm_opt.var_compare.sca" "${OPT_DIV_SCA}"
# MIN_COMPARE=`fgrep "\"${batching}\"" | cut -c 26- | sort -n | head -1`
# MAX_COMPARE=`fgrep "\"${batching}\"" | cut -c 26- | sort -n | tail -1`
# # packets for opt              
# log "starting the plot for opt \"${packets}\" at \"${opt_host}\""
# ${PLOT2D} "${OPT_NET_SCA}" "${MIN_COMPARE}" "${MAX_COMPARE}" "$batching" "$opt_host" "$packets" > "${tmp_net_opt}" || {
#     cp "${FAILED_FILE}""${GRAPHIC_PACKETS}"
#     log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${OPT_NET_SCA} ${MIN_COMPARE} ${MAX_COMPARE} $batching $opt_host $packets > ${tmp_net_loose}"
# }
# mv "${packets_data}" "${PLOT_OUT_PACKETS/batching_packets.data"
# # divergence for opt
# log "starting the plot for opt \"${divergence}\" at \"${opt_host_eye}\""
# ${PLOT2D} "${OPT_NET_SCA}" "${MIN_COMPARE}" "${MAX_COMPARE}" "$batching" "$opt_host_eye" "$divergence" > "${tmp_div_opt}" || {
#     cp "${FAILED_FILE}" "${GRAPHIC_DIVERGENCE}"
#     log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${OPT_NET_SCA} ${MIN_COMPARE} ${MAX_COMPARE} $batching ${opt_host_eye} $divergence > ${tmp_div_opt}"
# }
# mv "${divergence_data}" "${PLOT_OUT_PACKETS/batching_divergence.data"


# # copy the file into divergence files
# cp "${SWARM_LOOSE_REULTS}/swarm_loose.var_compare.sca" "${LOOSE_DIV_SCA}"
# cp "${SWARM_OPT_REULTS}/swarm_opt.var_compare.sca" "${OPT_DIV_SCA}"

