#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jan Sablatnig, Jiehua Chen, Sven Grottke.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# masterscript for the local_lag measurement script(distrib_local.sh) and the analysis script(plot2d.pl)
# usage: local_master.sh min_local max_local stepsize loss_rate delay reception_delay timeout batching nr_bees out_dir

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}
# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 10 ] || {
    error "usage: local_master.sh min_local max_local stepsize loss_rate delay reception_delay timeout batching nr_bees output_directory"
}

# set some paths
SWARM_OPT_BASE="${CONDEL_ROOT}/experiments/swarm_opt"
SWARM_IDEAL_BASE="${CONDEL_ROOT}/experiments/swarm_ideal"
SWARM_OPT_RESULTS="${SWARM_OPT_BASE}/results"
SWARM_IDEAL_RESULTS="${SWARM_IDEAL_BASE}/results"
PLOT_BASE="${CONDEL_ROOT}/util/plot"
PLOT_OUT_BASE="${10}"
PLOT_OUT_CPU="${PLOT_OUT_BASE}/cpu"
PLOT_OUT_PACKETS="${PLOT_OUT_BASE}/packets"
PLOT_OUT_DIVERGENCE="${PLOT_OUT_BASE}/div"
PLOT_OUT_SCORE="${PLOT_OUT_BASE}/score"
PLOT_OUT_DISCO="${PLOT_OUT_BASE}/disco"
OUT_SCA="${PLOT_OUT_BASE}/sca"

# check if the output directories exist and are writable
[ -d "${PLOT_OUT_PACKETS}" -a -w "${PLOT_OUT_PACKETS}" ] || {
    error "${PLOT_OUT_PACKETS} does not exist or is not writable"
}
[ -d "${PLOT_OUT_CPU}" -a -w "${PLOT_OUT_CPU}" ] || {
    error "${PLOT_OUT_CPU} does not exist or is not writable"
}
[ -d "${PLOT_OUT_DIVERGENCE}" -a -w "${PLOT_OUT_DIVERGENCE}" ] || {
    error "${PLOT_OUT_DIVERGENCE} does not exist or is not writable"
}
[ -d "${PLOT_OUT_SCORE}" -a -w "${PLOT_OUT_SCORE}" ] || {
    error "${PLOT_OUT_SCORE} does not exist or is not writable"
}
[ -d "${PLOT_OUT_DISCO}" -a -w "${PLOT_OUT_DISCO}" ] || {
    error "${PLOT_OUT_DISCO} does not exist or is not writable"
}

# set the executable measurement- and plotscript
DISTRIB_LOCAL="${SWARM_OPT_BASE}/scripts/distrib_local.sh"
SCA_FILE="${SWARM_OPT_RESULTS}/swarm_opt.var_local.sca"
PLOT2D="${PLOT_BASE}/plot2d.pl"
FAILED_FILE="${CONDEL_ROOT}/experiments/common_files/measurement_failed.ps"


# set some variables we use swarm_opt.average the default
# player, which represents the average of whole hosts:
MIN_LOCAL=$1
MAX_LOCAL=$2
STEPSIZE=$3
LOSS=$4
DELAY=$5
RECEPTION=$6
TIMEOUT=$7
BATCHING=$8
NR_BEES=$9
local="local lag" 
host="swarm_opt.average"
host_eye="swarm_opt.eye"
packets="avg. packets sent"
cpu="avg. cpu ops /tick"
divergence="D"
score="score achieved /s"
disco="avg. G"

# scafile for ideal
IDEAL_FILE="${SWARM_IDEAL_RESULTS}/swarm_ideal.nrBees_${NR_BEES}.sca"

# set the filename from results and the ps_file for output
GRAPHIC_PACKETS="${PLOT_OUT_PACKETS}/local_packets_${MIN_LOCAL}_${MAX_LOCAL}_${LOSS}_${DELAY}_${RECEPTION}_${TIMEOUT}_${BATCHING}_nrBees_${NR_BEES}.ps"
GRAPHIC_CPU="${PLOT_OUT_CPU}/local_cpu_${MIN_LOCAL}_${MAX_LOCAL}_${LOSS}_${DELAY}_${RECEPTION}_${TIMEOUT}_${BATCHING}_nrBees_${NR_BEES}.ps"
GRAPHIC_DIVERGENCE="${PLOT_OUT_DIVERGENCE}/local_divergence_${MIN_LOCAL}_${MAX_LOCAL}_${LOSS}_${DELAY}_${RECEPTION}_${TIMEOUT}_${BATCHING}_nrBees_${NR_BEES}.ps"
GRAPHIC_SCORE="${PLOT_OUT_SCORE}/local_score_${MIN_LOCAL}_${MAX_LOCAL}_${LOSS}_${DELAY}_${RECEPTION}_${TIMEOUT}_${BATCHING}_nrBees_${NR_BEES}.ps"
GRAPHIC_DISCO="${PLOT_OUT_DISCO}/local_disco_${MIN_LOCAL}_${MAX_LOCAL}_${LOSS}_${DELAY}_${RECEPTION}_${TIMEOUT}_${BATCHING}_nrBees_${NR_BEES}.ps"


# execute
log "starting the measurement script..."
${DISTRIB_LOCAL} "${MIN_LOCAL}" "${MAX_LOCAL}" "${STEPSIZE}" "${LOSS}" "${DELAY}" "${RECEPTION}" "${TIMEOUT}" "${BATCHING}" "${NR_BEES}" || {
    # copy an error message postscript file
    for dest in "${GRAPHIC_PACKETS}" "${GRAPHIC_CPU}" "${GRAPHIC_DIVERGENCE}" "${GRAPHIC_SCORE}" "${GRAPHIC_DISCO}"; do
        cp "${FAILED_FILE}" "${dest}"
    done

    log "measurement failed, aborting"
    exit 0
}

# copy the sca file under /local/sca/.
SCA_FILE="${OUT_SCA}/swarm_opt.var_local_${MIN_LOCAL}_${MAX_LOCAL}_$stepsize_${STEPSIZE}_${LOSS}_${DELAY}_${RECEPTION}_${TIMEOUT}_${BATCHING}_nrBees_${NR_BEES}.sca"

cp "${SWARM_OPT_RESULTS}/swarm_opt.var_local.sca" "${SCA_FILE}"
 
# plots
# packets
log "starting the plot for \"${packets}\" at \"${host}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOCAL}" "${MAX_LOCAL}" "$local" "$host" "$packets" > "$GRAPHIC_PACKETS" || {
    cp "${FAILED_FILE}" "${GRAPHIC_PACKETS}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOCAL} ${MAX_LOCAL} $local $host $packets > ${GRAPHIC_PACKETS}"
}

#cpu
log "starting the plot for \"${cpu}\" at \"${host}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOCAL}" "${MAX_LOCAL}" "$local" "$host" "$cpu" > "$GRAPHIC_CPU" || {
    cp "${FAILED_FILE}" "${GRAPHIC_PACKETS}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOCAL} ${MAX_LOCAL} $local $host $cpu > $GRAPHIC_CPU"
}

# divergence
log "starting the plot for \"${divergence}\" at \"${host_eye}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOCAL}" "${MAX_LOCAL}" "$local" "$host_eye" "$divergence" > "$GRAPHIC_DIVERGENCE" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DIVERGENCE}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOCAL} ${MAX_LOCAL} $local $host_eye $divergence > ${GRAPHIC_DIVERGENCE}"
}

# score
log "starting the plot for \"${score}\" at \"${host}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOCAL}" "${MAX_LOCAL}" "$local" "-i" "${IDEAL_FILE}" "$host" "$score" > "$GRAPHIC_SCORE" || {
    cp "${FAILED_FILE}" "${GRAPHIC_SCORE}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOCAL} ${MAX_LOCAL} $local -i ${IDEAL_FILE} $host $score > $GRAPHIC_SCORE"
}

# disco
log "starting the plot for \"${disco}\" at \"${host}\""
${PLOT2D} "${SCA_FILE}" "${MIN_LOCAL}" "${MAX_LOCAL}" "$local" "$host" "$disco" > "$GRAPHIC_DISCO" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DISCO}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_LOCAL} ${MAX_LOCAL} $local $host $disco > $GRAPHIC_DISCO"
}
