#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jan Sablatnig, Jiehua Chen, Sven Grottke.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# masterscript for nr. of bees measurement script(distrib_nr_bees.sh) and the analysis script(plot2d.pl)
# usage: nr_bees_master.sh min_nr_bees max_nr_bees loss_rate delay reception_delay local_lag timeout batching out_dir

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}
# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 9 ] || {
    error "usage: nr_bees_master.sh min_nr_bees max_nr_bees loss_rate delay reception_delay local_lag timeout batching out_dir"
}

# set some paths
SWARM_OPT_BASE="${CONDEL_ROOT}/experiments/swarm_opt"
SWARM_IDEAL_BASE="${CONDEL_ROOT}/experiments/swarm_ideal"
SWARM_OPT_RESULTS="${SWARM_OPT_BASE}/results"
SWARM_IDEAL_RESULTS="${SWARM_IDEAL_BASE}/results"
PLOT_BASE="${CONDEL_ROOT}/util/plot"
PLOT_OUT_BASE="$9"
PLOT_OUT_CPU="${PLOT_OUT_BASE}/cpu"
PLOT_OUT_PACKETS="${PLOT_OUT_BASE}/packets"
PLOT_OUT_DIVERGENCE="${PLOT_OUT_BASE}/div"
PLOT_OUT_SCORE="${PLOT_OUT_BASE}/score"
PLOT_OUT_DISCO="${PLOT_OUT_BASE}/disco"
OUT_SCA="${PLOT_OUT_BASE}/sca"

# check if the output directories exist and are writable
[ -d "${PLOT_OUT_CPU}" -a -w "${PLOT_OUT_CPU}" ] || {
    error "${PLOT_OUT_CPU} does not exist or is not writable"
}
[ -d "${PLOT_OUT_PACKETS}" -a -w "${PLOT_OUT_PACKETS}" ] || {
    error "${PLOT_OUT_PACKETS} does not exist or is not writable"
}
[ -d "${PLOT_OUT_DIVERGENCE}" -a -w "${PLOT_OUT_DIVERGENCE}" ] || {
    error "${PLOT_OUT_DIVERGENCE} does not exist or is not writable"
}
[ -d "${PLOT_OUT_SCORE}" -a -w "${PLOT_OUT_SCORE}" ] || {
    error "${PLOT_OUT_SCORE} does not exist or is not writable"
}
[ -d "${PLOT_OUT_DISCO}" -a -w "${PLOT_OUT_DISCO}" ] || {
    error "${PLOT_OUT_DISCO} does not exist or is not writable"
}
[ -d "${OUT_SCA}" -a -w "${OUT_SCA}" ] || {
    error "${OUT_SCA} does not exist or is not writable"
}

# set the executable measurement- and plotscript
DISTRIB_NR_BEES="${SWARM_OPT_BASE}/scripts/distrib_nr_bees.sh"
SCA_FILE="${SWARM_OPT_RESULTS}/swarm_opt.var_nrBees.sca"
PLOT2D="${PLOT_BASE}/plot2d.pl"
FAILED_FILE="${CONDEL_ROOT}/experiments/common_files/measurement_failed.ps"


# set some variables we use swarm_opt.average the default
# player, which represents the average of whole hosts:
MIN_NR_BEES=$1
MAX_NR_BEES=$2
LOSS=$3
DELAY=$4
RECEPTION=$5
LOCAL=$6
TIMEOUT=$7
BATCHING=$8
nr_bees="nr. of bees"
host="swarm_opt.average"
cpu="avg. cpu ops /tick"
host_eye="swarm_opt.eye"
packets="avg. packets sent"
#divergence="avg. divergence"
divergence="D"
score="score achieved /s"
disco="avg. G"
#disco="l2 discontinuity /s"

# scafile for ideal
IDEAL_FILE="${SWARM_IDEAL_RESULTS}/swarm_ideal.nrBees_min_${MIN_NR_BEES}_max_${MAX_NR_BEES}.sca" 

# set the filename from results and the ps_file for output
GRAPHIC_PACKETS="${PLOT_OUT_PACKETS}/nrBees_packets_${MIN_NR_BEES}_${MAX_NR_BEES}_${LOSS}_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}_${BATCHING}.ps"
GRAPHIC_CPU="${PLOT_OUT_CPU}/nrBees_cpu_${MIN_NR_BEES}_${MAX_NR_BEES}_$LOSS_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}_${BATCHING}.ps"
GRAPHIC_DIVERGENCE="${PLOT_OUT_DIVERGENCE}/nrBees_divergence_${MIN_NR_BEES}_${MAX_NR_BEES}_${LOSS}_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}_${BATCHING}.ps"
GRAPHIC_SCORE="${PLOT_OUT_SCORE}/nrBees_score_${MIN_NR_BEES}_${MAX_NR_BEES}_${LOSS}_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}_${BATCHING}.ps"
GRAPHIC_DISCO="${PLOT_OUT_DISCO}/nrBees_disco_${MIN_NR_BEES}_${MAX_NR_BEES}_${LOSS}_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}_${BATCHING}.ps"

#  execute
log "starting the measurement script..."
${DISTRIB_NR_BEES} "${MIN_NR_BEES}" "${MAX_NR_BEES}" "${LOSS}" "${DELAY}" "${RECEPTION}" "${LOCAL}" "${TIMEOUT}" "${BATCHING}" || {
    # copy an error message postscript file
    for dest in "${GRAPHIC_PACKETS}" "${GRAPHIC_CPU}" "${GRAPHIC_DIVERGENCE}" "${GRAPHIC_SCORE}" "${GRAPHIC_DISCO}"; do
        cp "${FAILED_FILE}" "${dest}"
    done

    log "measurement failed, aborting"
    exit 0
}

# copy the sca file under /nr_bees/sca/.
SCA_FILE="${OUT_SCA}/swarm_opt.var_nrBees_${MIN_NR_BEES}_${MAX_NR_BEES}_${LOSS}_${DELAY}_${RECEPTION}_${LOCAL}_${TIMEOUT}_${BATCHING}.sca"
cp "${SWARM_OPT_RESULTS}/swarm_opt.var_nrBees.sca" "${SCA_FILE}"

# plots
# packets
log "starting the plot for \"${packets}\" at \"${host}\" ( $GRAPHIC_PACKETS )"
${PLOT2D} "${SCA_FILE}" "${MIN_NR_BEES}" "${MAX_NR_BEES}" "$nr_bees" "$host" "$packets" > "$GRAPHIC_PACKETS" || {
#echo ${PLOT2D} \"${SCA_FILE}\" \"${MIN_NR_BEES}\" \"${MAX_NR_BEES}\" \"$nr_bees\" \"$host\" \"$packets\" > \"$GRAPHIC_PACKETS\"" || {
    cp "${FAILED_FILE}" "${GRAPHIC_PACKETS}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_NR_BEES} ${MAX_NR_BEES} $bee $host $packets > $GRAPHIC_PACKETS"
}

# cpu 
log "starting the plot for \"${cpu}\" at \"${host}\" ( ${GRAPHIC_CPU} )"
${PLOT2D} "${SCA_FILE}" "${MIN_NR_BEES}" "${MAX_NR_BEES}" "${nr_bees}" "$host" "$cpu" > "${GRAPHIC_CPU}" || {
    cp "${FAILED_FILE}" "${GRAPHIC_CPU}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_NR_BEES} ${MAX_NR_BEES} ${nr_bees} $host $cpu > ${GRAPHIC_CPU}"
}

# divergence
log "starting the plot for \"${divergence}\" at \"${host_eye}\" ( $GRAPHIC_DIVERGENCE )"
${PLOT2D} "${SCA_FILE}" "${MIN_NR_BEES}" "${MAX_NR_BEES}" "$nr_bees" "$host_eye" "$divergence" > "$GRAPHIC_DIVERGENCE" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DIVERGENCE}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_NR_BEES} ${MAX_NR_BEES} $bee ${host_eye} $divergence > $GRAPHIC_DIVERGENCE"
}

# score
log "starting the plot for \"${score}\" at \"${host}\" ( $GRAPHIC_SCORE ) "
${PLOT2D} "${SCA_FILE}" "${MIN_NR_BEES}" "${MAX_NR_BEES}" "$nr_bees" "-i" "${IDEAL_FILE}" "$host" "$score" > "$GRAPHIC_SCORE" || {
    cp "${FAILED_FILE}" "${GRAPHIC_SCORE}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_NR_BEES} ${MAX_NR_BEES} $bee -i ${IDEAL_FILE} $host $score > $GRAPHIC_SCORE"
}

# disco
log "starting the plot for \"${disco}\" at \"${host}\" ( $GRAPHIC_DISCO )"
${PLOT2D} "${SCA_FILE}" "${MIN_NR_BEES}" "${MAX_NR_BEES}" "$nr_bees" "$host" "$disco" > "$GRAPHIC_DISCO" || {
    cp "${FAILED_FILE}" "${GRAPHIC_DISCO}"
    log "WARNING: plot2d failed with the following parameters: ${PLOT2D} ${SCA_FILE} ${MIN_NR_BEES} ${MAX_NR_BEES} $bee $host $disco > $GRAPHIC_DISCO"
}
