#!/usr/bin/perl
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Sven Grottke, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.


#usage: calc_P_delay_error.pl filename1 filename2 filename3 filename4 delta_T
#tips: filename1 is scafile made from single_run.sh(experiment);
#      filename2 is datfile made from P_delay (theory, mode: precisely);
#      filename3 is datfile made from P_delay (theory, mode: interpolated approx.)
#      filename4 is datfile made from P_delay (theory, mode: hybrid)

use strict;

sub usage{
    print STDERR "usage: calc_P_delay_error.pl filename1 filename2 filename3 filename4 delta_T\ntips:  filename1 is scafile made from single_run.sh(experiment);\n       filename2 is datafile made from P_delay (theory, mode: precisely);\n       filename3 is datafile made from P_delay (theory, mode: interpolated approx.);\n       filename3 is datafile made from P_delay (theory, mode: hybrid).\n";
}

sub failure{
    my $failedparm = shift;

    if( defined $failedparm ){
        print STDERR "$failedparm format is wrong\n";
    }else{
        print STDERR "parameters failed\n";
    }

    &usage;
    print STDERR "Please see the usage for more details.\n";
    die "aborting.";
}

#sanity check
if( @ARGV != 5 ){ &failure; }

my $scafile = $ARGV[0];
my $datafile_prec = $ARGV[1];
my $datafile_inter = $ARGV[2];
my $datafile_hybrid = $ARGV[3];
my $delta_T = $ARGV[4];

my $P_delay_error_prec = 0.0;
my $P_delay_error_inter = 0.0;
my $P_delay_error_hybrid = 0.0;

open(SCAFILE, $scafile);
open(DATA_PREC, $datafile_prec);
open(DATA_INTER, $datafile_inter);
open(DATA_HYBRID, $datafile_hybrid);

my @sca_lines = <SCAFILE>;
my @data_prec_lines = <DATA_PREC>;
my @data_inter_lines = <DATA_INTER>;
my @data_hybrid_lines = <DATA_HYBRID>;
my $count_t_sca = 0;
my $T_min = 10000000;
my $T_max = -1;

#find the accordance probabilities in sca- and datafiles
my $sca_line;
foreach $sca_line ( @sca_lines ){
    if( $sca_line =~ /\"T\s+=\s+([\.\d\-e]+)\"\s+([\.\d\-e]+)/ ){
        my $p_sca = $2;
        if ($1 < $T_min) {
            $T_min = $1;
        }
        if ($1 > $T_max) {
            $T_max = $1;
        }
#       print "p_sca: ${p_sca} ";
        my @pair_prec = split(/\s+/, $data_prec_lines[$count_t_sca] );
#        print "\n$data_prec_lines[$count_t_sca]";
        $P_delay_error_prec += abs($p_sca-$pair_prec[1]);
#       print "prec: $pair_prec[0]  $pair_prec[1] ";
        my @pair_inter = split(/\s+/, $data_inter_lines[$count_t_sca] );
        $P_delay_error_inter += abs($p_sca-$pair_inter[1]);
#       print "inter: $pair_inter[0]  $pair_inter[1] ";
#       print "\n";
        my @pair_hybrid = split(/\s+/, $data_hybrid_lines[$count_t_sca] );
        $P_delay_error_hybrid += abs($p_sca-$pair_hybrid[1]);
#       print "hybrid: $pair_hybrid[0]  $pair_hybrid[1] ";
#       print "\n";
        $count_t_sca++;
    }
}

# print "${count_t_sca}\n";
# print "P_delay_error_prec: ${P_delay_error_prec}\n";

$P_delay_error_prec *= $delta_T;
$P_delay_error_inter *= $delta_T;
$P_delay_error_hybrid *= $delta_T;
# print "P_delay_error_prec: ${P_delay_error_prec}\n";
# print "P_delay_error_inter: ${P_delay_error_inter}\n";
# print "P_delay_error_hybrid: ${P_delay_error_hybrid}\n";

# scale with interval width
if ($T_min >= $T_max) {
    die "screwed up T_min and T_max: $T_min $T_max\n";
}

$P_delay_error_prec /= ($T_max - $T_min);
$P_delay_error_inter /= ($T_max - $T_min);
$P_delay_error_hybrid /= ($T_max - $T_min);

print "scalar \"timeoutNetwork.R\"\t\"P_delay_error_prec\"\t$P_delay_error_prec\n";
print "scalar \"timeoutNetwork.R\"\t\"P_delay_error_inter\"\t$P_delay_error_inter\n";
print "scalar \"timeoutNetwork.R\"\t\"P_delay_error_hybrid\"\t$P_delay_error_hybrid\n";

close(SCAFILE);
close(DATA_PREC);
close(DATA_INTER);
close(DATA_HYBRID);

# write a new scafile with extra information about P_delay_error_prec, P_delay_error_inter and P_delay_error_inter
# and copy it again into scafile
my $scafile_new =  $scafile . "temp.sca";

open(SCAFILE, $scafile);
open(SCAFILE_NEW, ">$scafile_new");
@sca_lines = <SCAFILE>;
$count_t_sca += 3;

my $i;
for( $i = 0; $i < 3; ++$i ){
    print SCAFILE_NEW "$sca_lines[$i]";
}

print SCAFILE_NEW "scalar \"timeoutNetwork.R\"\t\"P_delay_error_prec\"\t$P_delay_error_prec\n";
print SCAFILE_NEW "scalar \"timeoutNetwork.R\"\t\"P_delay_error_inter\"\t$P_delay_error_inter\n";
print SCAFILE_NEW "scalar \"timeoutNetwork.R\"\t\"P_delay_error_hybrid\"\t$P_delay_error_hybrid\n";

for( $i = $count_t_sca; $i < @sca_lines; ++$i ){
    print SCAFILE_NEW "$sca_lines[$i]";
}

close(SCAFILE);
close(SCAFILE_NEW);
