#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Sven Grottke, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# varies the ARQ timeout, leaving all other parameters constant
# usage: single_timeout.sh min_timeout max_timeout stepsize delay_fun rtt_expect [arg] data_expect [arg] loss_data loss_ack
# delay_fun can be "step"(no arg), "exp"(arg: x_min), "pareto"(arg: k)

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}

# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"

# sanity check
[ $# = 8 -o $# = 10 ] || {
    error "usage: single_timeout.sh min_timeout max_timeout stepsize delay_fun rtt_expect [arg] data_expect [arg] loss_data loss_ack"
}
    
# set some paths and variables
TIMEOUT_BASE="${CONDEL_ROOT}/experiments/timeout"
SINGLE_RUN="${CONDEL_ROOT}/experiments/common_files/single_run.sh"
CALC_ERROR="${TIMEOUT_BASE}/scripts/calc_P_delay_error.pl"
CALC_P_delay="${CONDEL_ROOT}/util/timeout-plot/P_delay"
CALC_E_N="${CONDEL_ROOT}/util/timeout-plot/E_N"
RESULTS="${TIMEOUT_BASE}/results"
VEC_FILE="${RESULTS}/omnetpp.vec"
SCA_FILE="${RESULTS}/timeout.sca"
DELAY_STEPSIZE="0.001"

# remove old results
[ -f "${VEC_FILE}" ] && rm "${VEC_FILE}"
[ -f "${SCA_FILE}" ] && rm "${SCA_FILE}"

# step through it
timeout=$1
shift
max_timeout=$1
shift
stepsize=$1
shift
DELAY_FUN=$1
shift
EXPECT_RTT=$1
shift
case ${DELAY_FUN} in
"step")
        EXPECT_DATA=$1;
        delay_parm_set_rtt="${DELAY_FUN} ${EXPECT_RTT} ${DELAY_DATA}";
        delay_parm_set_data="${DELAY_FUN} $EXPECT_DATA";
        shift;;
"exp")
        ARG_RTT=$1;
        shift;
        EXPECT_DATA=$1;
        shift;
        ARG_DATA=$1;
        delay_parm_set_rtt="${DELAY_FUN} $EXPECT_RTT ${ARG_RTT}"; 
        delay_parm_set_data="${DELAY_FUN} $EXPECT_DATA ${ARG_DATA}";
        shift;;
"pareto")
        delay_parm_set_rtt="${DELAY_FUN} $EXPECT_RTT $1"; 
        delay_parm_set_data="${DELAY_FUN} $EXPECT_DATA $1";
        shift;;
esac   
delay_parm_set_rtt_fn=`echo "${delay_parm_set_rtt}" | sed -e "s/ /_/g"`

while [ `echo "[1]sa $max_timeout $timeout !>a z p" | dc` = 1 ]; do
    log "measuring with a timeout of ${timeout} ..."
    RESULT_SCAFILE="${RESULTS}/timeout_delay_func_rtt_${delay_parm_set_rtt_fn}_timeout_${timeout}_lossD_$1_lossA_$2.sca"
    RESULT_PRECFILE="${RESULTS}/timeout_delay_func_rtt_${delay_parm_set_rtt_fn}_timeout_${timeout}_loss_$1_prec.dat"
    RESULT_INTERFILE="${RESULTS}/timeout_delay_func_rtt_${delay_parm_set_rtt_fn}_timeout_${timeout}_loss_$1_inter.dat"
    RESULT_HYBRIDFILE="${RESULTS}/timeout_delay_func_rtt_${delay_parm_set_rtt_fn}_timeout_${timeout}_loss_$1_hybrid.dat"

    # execute experimentaly
    ${SINGLE_RUN} ${TIMEOUT_BASE} 1000000000s "${delay_parm_set_rtt} $1 $2 $timeout" ${RESULT_SCAFILE} || {
        error "single_run.sh failed with the following parameters: ${delay_parm_set_rtt_fn} $1 $2 ${timeout} ${RESULT_SCAFILE}"
    }

    # execute theory P_delay ( precisely )
    ${CALC_P_delay} --dist ${delay_parm_set_data} --timeout $timeout --stepsize ${DELAY_STEPSIZE} --loss $1 --mode prec > ${RESULT_PRECFILE} || {
        error "${CALC_P_delay} failed with the following parameters: --dist ${DELAY_FUN} ${EXPECT_DATA} --timeout $timeout --stepsize ${DELAY_STEPSIZE} --loss $1 --mode prec > ${RESULT_PRECFILE} "
    }

    # execute theory P_delay ( interpolated approx. )
    ${CALC_P_delay} --dist ${delay_parm_set_data} --timeout $timeout --stepsize ${DELAY_STEPSIZE} --loss $1 --mode inter > ${RESULT_INTERFILE} || {
        error "${CALC_P_delay} failed with the following parameters: --dist ${DELAY_FUN} ${EXPECT_DATA} --timeout $timeout --stepsize ${DELAY_STEPSIZE} --loss $1 --mode inter > ${RESULT_INTERFILE} "
    }

    # execute theory P_delay ( hybrid )
    ${CALC_P_delay} --dist ${delay_parm_set_data} --timeout $timeout --stepsize ${DELAY_STEPSIZE} --loss $1 --mode hybrid > ${RESULT_HYBRIDFILE} || {
        error "${CALC_P_delay} failed with the following parameters: --dist ${DELAY_FUN} ${EXPECT_DATA} --timeout $timeout --stepsize ${DELAY_STEPSIZE} --loss $1 --mode hybrid > ${RESULT_HYBRIDFILE} "
    }

    # calculate the errors between experiment and theories
    ${CALC_ERROR} ${RESULT_SCAFILE} ${RESULT_PRECFILE} ${RESULT_INTERFILE} ${RESULT_HYBRIDFILE} ${DELAY_STEPSIZE} || {
        error "${CALC_ERROR} ${RESULT_SCAFILE} ${RESULT_PRECFILE} ${RESULT_INTERFILE} ${DELAY_STEPSIZE} "
    }
    cat "${RESULT_SCAFILE}temp.sca" >> "${SCA_FILE}"

    tmp_timeout_max=`echo "${timeout} 0.000001 + p" | dc`

    # execute theory E_N( precisely )
    E_N_PREC=`${CALC_E_N} --dist ${delay_parm_set_rtt} --min ${timeout} --max ${tmp_timeout_max} --mode prec --loss-data $1 --loss-ack $2 | cut -f2 -d' ' `  || {
        error "${CALC_E_N} failed with the following parameters: --dist ${delay_parm_set_rtt} --min ${timeout} --max ${tmp_timeout_max} --mode prec --loss-data $1 --loss-ack $2 "
    }
    echo -e "scalar \"timeoutNetwork.S\"\t\"E_N_PREC\"\t${E_N_PREC}"
    echo -e "scalar \"timeoutNetwork.S\"\t\"E_N_PREC\"\t${E_N_PREC}" >> ${SCA_FILE}

    # execute theory E_N( approx. )
    E_N_APPROX=`${CALC_E_N} --dist ${delay_parm_set_rtt} --min ${timeout} --max ${tmp_timeout_max} --mode approx --loss-data $1 --loss-ack $2 | cut -f2 -d' '` || {
        error "${CALC_E_N} failed with the following parameters: --dist ${delay_parm_set_rtt} --min ${timeout} --max ${tmp_timeout_max} --mode approx --loss-data $1 --loss-ack $2 "
    }
    echo -e "scalar \"timeoutNetwork.S\"\t\"E_N_APPROX\"\t${E_N_APPROX}"
    echo -e "scalar \"timeoutNetwork.S\"\t\"E_N_APPROX\"\t${E_N_APPROX}" >> ${SCA_FILE}

    E_A_PREC=`echo "8k 1 $1 - ${E_N_PREC} * p" | dc`
    echo -e "scalar \"timeoutNetwork.R\"\t\"E_A_PREC\"\t${E_A_PREC}"
    echo -e "scalar \"timeoutNetwork.R\"\t\"E_A_PREC\"\t${E_A_PREC}" >> ${SCA_FILE}
       
    E_A_APPROX=`echo "8k 1 $1 - ${E_N_APPROX} * p" | dc`
    echo -e "scalar \"timeoutNetwork.R\"\t\"E_A_APPROX\"\t${E_A_APPROX}"
    echo -e "scalar \"timeoutNetwork.R\"\t\"E_A_APPROX\"\t${E_A_APPROX}" >> ${SCA_FILE}

    # remove the tmpfiles
    rm "${RESULT_SCAFILE}"
    rm "${RESULT_SCAFILE}temp.sca"
    rm "${RESULT_PRECFILE}"
    rm "${RESULT_INTERFILE}"
    timeout=`echo "8 k $timeout $stepsize + p q" | dc`
done

# done
log "single_timeout.sh finished with exit status ok."
