/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include "attribute.hh"

cAttribute::cAttribute(const ATTRIBUTE_ID& id)
  : m_id(id), m_value(0) {
}

cAttribute::cAttribute(const string& name)
  : m_id(cAttributeFactory::nameToID(name)), m_value(0) {
}

cAttribute::cAttribute(const ATTRIBUTE_ID& id, double value)
  : m_id(id), m_value(value) {
}

cAttribute::cAttribute(const string& name, double value)
  : m_id(cAttributeFactory::nameToID(name)), m_value(value) {
}

cAttribute::cAttribute(const cAttribute& src)
  : m_id(src.m_id), m_value(src.m_value) {
}





cAttributeSet::cAttributeSet() : map<const ATTRIBUTE_ID, cAttribute>() {
}

bool cAttributeSet::hasAttribute(const ATTRIBUTE_ID& id) const {
  return (find(id) != end());
}

const cAttribute& cAttributeSet::get(const ATTRIBUTE_ID& id) const {
  const_iterator iter = find(id);

  // sanity check
  assert(iter != end());

  return (*iter).second;
}

cAttribute& cAttributeSet::get(const ATTRIBUTE_ID& id) {
  iterator iter = find(id);

  // sanity check
  assert(iter != end());

  return (*iter).second;
}

void cAttributeSet::add(const cAttribute& attr) {
  // insert it
#ifndef NDEBUG
  const bool ret =
#endif
    insert(value_type(attr.getID(), attr)).second;
#ifndef NDEBUG
  assert(ret);
#endif
}

cAttributeSet::const_iterator cAttributeSet::begin() const {
  return map<const ATTRIBUTE_ID, cAttribute>::begin();
}

cAttributeSet::iterator cAttributeSet::begin() {
  return map<const ATTRIBUTE_ID, cAttribute>::begin();
}

cAttributeSet::const_iterator cAttributeSet::end() const {
  return map<const ATTRIBUTE_ID, cAttribute>::end();
}

cAttributeSet::iterator cAttributeSet::end() {
  return map<const ATTRIBUTE_ID, cAttribute>::end();
}







// Messages (which are Omnet objects) contain attributes, therefore they should
// contain Descriptors.  This is a copy from the one autogenerated for cMsg,
// with a few edits of course.

#include <omnetpp.h>

class cAttributeDescriptor : public cStructDescriptor
{
  public:
    cAttributeDescriptor();
    virtual ~cAttributeDescriptor();
    cAttributeDescriptor& operator=(const cAttributeDescriptor& other);
    virtual cPolymorphic *dup() const {return new cAttributeDescriptor(*this);}

    virtual int getFieldCount();
    virtual const char *getFieldName(int field);
    virtual int getFieldType(int field);
    virtual const char *getFieldTypeString(int field);
    virtual const char *getFieldEnumName(int field);
    virtual int getArraySize(int field);

    virtual bool getFieldAsString(int field, int i, char *resultbuf, int bufsize);
    virtual bool setFieldAsString(int field, int i, const char *value);

    virtual const char *getFieldStructName(int field);
    virtual void *getFieldStructPointer(int field, int i);
    virtual sFieldWrapper *getFieldWrapper(int field, int i);
};

Register_Class(cAttributeDescriptor);

cAttributeDescriptor::cAttributeDescriptor() : cStructDescriptor("cAttribute")
{
}

cAttributeDescriptor::~cAttributeDescriptor()
{
}

int cAttributeDescriptor::getFieldCount()
{
    return baseclassdesc ? 2+baseclassdesc->getFieldCount() : 2;
}

int cAttributeDescriptor::getFieldType(int field)
{
    if (baseclassdesc) {
        if (field < baseclassdesc->getFieldCount())
            return baseclassdesc->getFieldType(field);
        field -= baseclassdesc->getFieldCount();
    }
    switch (field) {
        case 0: return FT_BASIC;
        case 1: return FT_BASIC;
        default: return FT_INVALID;
    }
}

const char *cAttributeDescriptor::getFieldName(int field)
{
    if (baseclassdesc) {
        if (field < baseclassdesc->getFieldCount())
            return baseclassdesc->getFieldName(field);
        field -= baseclassdesc->getFieldCount();
    }
    switch (field) {
        case 0: return "id";
        case 1: return "value";
        default: return NULL;
    }
}

const char *cAttributeDescriptor::getFieldTypeString(int field)
{
    if (baseclassdesc) {
        if (field < baseclassdesc->getFieldCount())
            return baseclassdesc->getFieldTypeString(field);
        field -= baseclassdesc->getFieldCount();
    }
    switch (field) {
        case 0: return "unsigned int";
        case 1: return "double";
        default: return NULL;
    }
}

const char *cAttributeDescriptor::getFieldEnumName(int field)
{
    if (baseclassdesc) {
        if (field < baseclassdesc->getFieldCount())
            return baseclassdesc->getFieldEnumName(field);
        field -= baseclassdesc->getFieldCount();
    }
    switch (field) {
        default: return NULL;
    }
}

int cAttributeDescriptor::getArraySize(int field)
{
    if (baseclassdesc) {
        if (field < baseclassdesc->getFieldCount())
            return baseclassdesc->getArraySize(field);
        field -= baseclassdesc->getFieldCount();
    }
    //cAttribute *pp = (cAttribute *)p;
    switch (field) {
        default: return 0;
    }
}

bool cAttributeDescriptor::getFieldAsString(int field, int i, char *resultbuf, int bufsize)
{
    if (baseclassdesc) {
        if (field < baseclassdesc->getFieldCount())
            return baseclassdesc->getFieldAsString(field,i,resultbuf,bufsize);
        field -= baseclassdesc->getFieldCount();
    }
    cAttribute *pp = (cAttribute *)p;
    switch (field) {
        case 0: long2string(pp->getID(),resultbuf,bufsize); return true;
        case 1: double2string(pp->getValue(),resultbuf,bufsize); return true;
        default: return false;
    }
}

bool cAttributeDescriptor::setFieldAsString(int field, int i, const char *value)
{
    if (baseclassdesc) {
        if (field < baseclassdesc->getFieldCount())
            return baseclassdesc->setFieldAsString(field,i,value);
        field -= baseclassdesc->getFieldCount();
    }
    cAttribute *pp = (cAttribute *)p;
    switch (field) {
        case 0: return false;   // this one can't be set.
        case 1: pp->setValue(string2double(value)); return true;
        default: return false;
    }
}

const char *cAttributeDescriptor::getFieldStructName(int field)
{
    if (baseclassdesc) {
        if (field < baseclassdesc->getFieldCount())
            return baseclassdesc->getFieldStructName(field);
        field -= baseclassdesc->getFieldCount();
    }
    switch (field) {
        default: return NULL;
    }
}

void *cAttributeDescriptor::getFieldStructPointer(int field, int i)
{
    if (baseclassdesc) {
        if (field < baseclassdesc->getFieldCount())
            return baseclassdesc->getFieldStructPointer(field, i);
        field -= baseclassdesc->getFieldCount();
    }
    //cAttribute *pp = (cAttribute *)p;
    switch (field) {
        default: return NULL;
    }
}

sFieldWrapper *cAttributeDescriptor::getFieldWrapper(int field, int i)
{
    return NULL;
}

