/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/

#ifndef __ATTRIBUTE_FACTORY_H
#define __ATTRIBUTE_FACTORY_H

#include <map>

#include "gen.hh"

namespace Condel {
  
/// Type for attribute IDs.
/// This typedef may change in the future, so use this defined type.
typedef unsigned ATTRIBUTE_ID;

/// Attribute ID manager.
/// This is a purely static class used to globablly assign the IDs for
/// attributes.  Also handles the translation of attribute name->attribute
/// ID.
///
/// Note that this scheme is a cludge, it would not be possible to do this
/// on a truly distributed system.  There are schemes to assign such numbers
/// in a distributed fashion though, and we decided to ignore this sub-
/// problem for greater clarity of the rest of the code.
/// @sa cObjectFactory

class cAttributeFactory {
private:
  // get the next free id and count it up.
  static ATTRIBUTE_ID nextFreeID(void);
  
  // map names to attribute IDs
  static map<const string, ATTRIBUTE_ID> &factoryList(void);  

public:
  /// Create a unique ID for a new attribute.
  /// @param name The name of the new attribute.
  /// @return The newly assigned attribute ID.
  static ATTRIBUTE_ID registerAttribute(const string &name);

  /// Returns the ID of a formerly registered attribute.
  /// If the attribute was NOT registered before, the application will
  /// exit because of a builtin assert here.
  /// @param name The name of the existing attribute.
  /// @return The corresponding attribute ID.
  /// @sa nameExists
  static ATTRIBUTE_ID nameToID(const string &name);

  /// Check if an attribute was already registered.
  /// @param name The name of the attribute in question.
  /// @return True if this name exists already in the DB.
  static bool nameExists(const string &name);
};

};

#endif // __ATTRIBUTE_FACTORY_H
