/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/

#include "eye.hh"
#include "host.hh"


// For the first INITIAL_DROP seconds, there will be no measurements in the
// vec file.
#define INITIAL_DROP 0

Define_Module(cEye);


cEye::cEye()
  : cSimpleModule(), m_varMeter(), m_pTick(NULL), m_attNames()
{ }

void cEye::initialize(void)
{
  cHost *any_host= NULL;
  {
  for (cSubModIterator iter(*parentModule()); !iter.end(); iter++)
    if (iter() != this)
      {
      any_host = check_and_cast<cHost*>(iter());
      break;
      }
  }

  m_varMeter.init(any_host->m_attributeNorm);

  m_attNames.clear();

  for (cSubModIterator iter(*parentModule()); !iter.end(); iter++)
    if (iter() != this)
      m_varMeter.addHost(check_and_cast<cHost*>(iter()));


  cStringTokenizer vm(par("variance_weights")," \t\n");
  while (vm.hasMoreTokens())
    {
    string at= vm.nextToken();
    assert(vm.hasMoreTokens());

#if VERBOSITY >= 3
    cout << "Now parsing " << at << endl; 
#endif
    if (!cAttributeFactory::nameExists(at))
      {
      cerr << "Error in .ned file:  Attribute '" << at << "' not found.\n";
      exit(-1);
      }

    m_attNames.push_back(at);

    m_varMeter.addAttribute(cAttributeFactory::nameToID(at), atof(vm.nextToken()));
    }

  m_varMeter.initWeights();

  // set up self events for ticking away
  m_pTick= new cMessage("tick");
  scheduleAt(cHost::TICK_TIME + 0.001, m_pTick);

  m_consistencyVec.setName("current variance");
}

void cEye::handleMessage(cMessage* pMsg)
{
  assert(pMsg == m_pTick);
  scheduleAt(cHost::roundTime(simTime() + cHost::TICK_TIME), m_pTick);

  // this is needed, otherwise the average consistency will not be computed
  // correctly
  m_varMeter.computeConsistency();

#if VERBOSITY >= 2
  ev << "Time: " << simTime() << "Consistency: " << con << endl;
#endif

  if (simTime() > INITIAL_DROP) {
    //    m_consistencyVec.record(con);
    m_consistencyVec.record(0);
  }
}

void cEye::finish(void)
{
  cancelAndDelete(m_pTick);

  recordScalar("end time", cHost::roundTime(simTime()));
  recordScalar("avg. variance", m_varMeter.averageConsistency2());
  recordScalar("D", m_varMeter.averageConsistency1());

  list<string>::const_iterator at;
  for (at= m_attNames.begin(); at != m_attNames.end(); at++)
    {
    char name[200];
    snprintf(name, sizeof(name), "avg. variance (%s)", (*at).data());
    recordScalar (name, m_varMeter.averageConsistency2(cAttributeFactory::nameToID(*at)));
    snprintf(name, sizeof(name), "D(%s)", (*at).data());
    recordScalar (name, m_varMeter.averageConsistency1(cAttributeFactory::nameToID(*at)));
    }
}
