/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/

#ifndef __EYE_H
#define __EYE_H

#include <omnetpp.h>

#include "gen.hh"
#include "variance_meter.hh"
#include <list>

namespace Condel {

// forward references

/// The eye-of-god for global measurements.
/// This class is added to the system just like a host.  This allows it
/// to use OMNeT's methods to find all other hosts and survey their data.
/// Thus, this object does something that is never possible in a real
/// network:  It looks at the states of all data on all hosts simultaneously.
/// This ability is only used for measurements.  The divergence measure,
/// namely, is measured from here.
  
class cEye : public cSimpleModule {
private:
  cVarianceMeter m_varMeter;
  cMessage *m_pTick;
  cOutVector m_consistencyVec;
  list<string> m_attNames;        // list of attributes seen in .ned file

public:
  cEye(void);
  virtual ~cEye(void) {}

protected:

  /// Reset everything.
  /// This member is called by OMNeT when the system is about to start.
  virtual void initialize(void);

  /// Handles messages to eye.
  /// This member is called by OMNeT when the internal tick-event-timer
  /// is up.  Outside of that, you should never send a message to eye.
  virtual void handleMessage(cMessage* pMsg);

  /// System Done.
  /// This member is called by OMNeT when the system has ended.
  virtual void finish(void);
};
  
};

#endif // __EYE_H
