/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/

/// @file gen.hh
/// Common includes and definitions for Adam.

#ifndef __GEN_H
#define __GEN_H


// for assert()
#include <assert.h>

// scoped_ptr etc.
#include <boost/smart_ptr.hpp>
using namespace boost;

// std::string
#include <string>

// datatypes
#include <datatypes.h>


/// The namespace for our project.
/// Everything related to Adam is in the Condel namespace, except for
/// a few minor things that cannot because their namespace is set by OMNeT.
/// Also, local objects (visible only within some file) are usually not
/// namespaced.

namespace Condel {

  /// Print error and exit.
  /// @param pWhat The message shown to the user just before aborting.
  void condel_error(const char* pWhat);
};

// namespace definitions
using namespace Condel;
using namespace std;


/// Set the debug level.
/// Debugging depends on level:
///     <ul><li>0: none at all, this also means sanity checks are skipped
///	<li>1: some (unexpected things occurring once in a while)
///	<li>2: lots (i.e., checks and printouts per tick)
///	<li>3: extreme (probably unnecessary checks)
///     </ul>
/// Set DEBUG level either in gen.hh or in the Makefile via -DDEBUG=2.
///
/// Defaults to 1.
#ifndef DEBUG
#define DEBUG 1
#endif

/// Set No debug mode.
/// If defined, overrides the DEBUG setting (i.e. DEBUG=0).
#ifdef DOXYGEN_ONLY
#define NDEBUG
#endif // DOXYGEN_ONLY

#ifdef NDEBUG
#ifdef DEBUG
#undef DEBUG
#endif
#ifndef DOXYGEN_SKIP
#define DEBUG 0
#endif // DOXYGEN_SKIP
#endif


/// Verbosity depends on level:
///  <ul><li>0: Nothing is printed during a normal run
///  <li>1: some progress info
///  <li>2: lots (i.e., printouts per tick)
///  <li>3: extreme (additionally shows massive amounts of internal information)
///     </ul>
///
/// Defaults to 0.
#ifndef VERBOSITY
#define VERBOSITY 0
#endif

#endif
