/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include "instance.hh"
#include "instance_factory.hh"
#include "world.hh"



map<const string, instanceCreator> &cInstanceFactory::factoryList(void)
{
    // This trick avoids the static constructor order calling problem.
    static map<const string, instanceCreator> slist;
    return slist;
}


void cInstanceFactory::registerInstance(const string &name,
                                        instanceCreator cic)
{
    string claname(name);
    long hashpos= claname.find("#");
    if (hashpos >0)		// contains a #
	{ claname.resize(hashpos); }
    
    factoryList()[claname]= cic;
}



cInstance *cInstanceFactory::createInstance(const OBJECT_ID& id,
                                            const string &name)
{
    string claname(name);
    long hashpos= claname.find("#");
    if (hashpos >0)		// contains a #
	{ claname.resize(hashpos); }

    map<const string, instanceCreator>::const_iterator x(factoryList().find(claname));

    if (x == factoryList().end())
	{ condel_error("class name not found"); return NULL; }

    // retrieve the correct creator function and use it to create a new
    // instance, which is passed to the caller
    return (*x).second(id, name);
}
