/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include "object_factory.hh"

#if DEBUG >=1
#include <iostream>
#endif


map<const string, OBJECT_ID> &cObjectFactory::factoryList(void)
{
  // This trick avoids the static constructor order calling problem.
  static map<const string, OBJECT_ID> slist;
  return slist;
}

OBJECT_ID cObjectFactory::nextFreeID(void)
{
  static OBJECT_ID nextfree= 0;

  nextfree++;
  if (!nextfree)
    condel_error("ran out of object ID's");

  return nextfree;
}

OBJECT_ID cObjectFactory::registerObject(const string &name)
{
  // make sure that an object of the same name has not already been registered
  assert(factoryList().find(name) == factoryList().end());

  OBJECT_ID newID= nextFreeID();

#if VERBOSE >=1
  cout << "OBJECT: " << name << " = " << newID << endl;
#endif
  
  factoryList()[name]= newID;

  return newID;
}

OBJECT_ID cObjectFactory::nameToID(const string& name)
{
  map<const string, OBJECT_ID>::const_iterator x(factoryList().find(name));

  // make sure it's there
  assert(x != factoryList().end());

  return (*x).second;
}

bool cObjectFactory::nameExists(const string& name)
{
  return factoryList().find(name) != factoryList().end();
}


void cObjectFactory::clear(void)
{
  static map<const string, OBJECT_ID> &m= factoryList();

  m.erase(m.begin(), m.end());
}
