/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include <math.h>

#include "polar.hh"


void cPolar::update(void)
{
  m_abs= sqrt(x()*x() + y()*y());
  if (x() == 0.0)
    {
    if (y() == 0.0)
      m_phi= 0.0;
    else
      m_phi= y() < 0.0 ? M_PI_2 : 3* M_PI_2;
    }
  else
    {
    m_phi= -atan(y()/x());      // - because y is positive DOWNward
    if (x()<0.0)
      {
      m_phi += M_PI;
      if (m_phi > M_PI)
        m_phi -= 2 * M_PI;
      }
    }
}

double cPolar::normalize_phi(double phi)
{
  while (phi < -M_PI)
    phi += 2* M_PI;

  while (phi > M_PI)
    phi -= 2* M_PI;

  return phi;
}

void cPolar::setPhiAbs(double phi, double nabs)
{
  setXY(nabs * cos(phi), -nabs * sin(phi));     // - because y is positive DOWN
}


cPolar operator*(double v, const cPolar &p) { return p * v; }

std::ostream& operator<<(std::ostream& os, const cPolar &value)
{
  os << (*(cVector<2,double> *) &value);
  os << "[" << value.phi() << ", " << value.abs() << "]";
  return os;
}
