/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#ifndef __ROUTER_H
#define __ROUTER_H

#include <cstat.h>

#include "gen.hh"
#include "sliding_window.hh"
#include "host.hh"


namespace Condel {

// forward references
class cHost;
class cConsistency;
class cMsg;

/// The basic netlayer interface.
/// This is the common base class of all actual routing/network libraries.
/// 
/// Routers basically send and receive information to/from other hosts on
/// the networks.  They contain the algorithms needed to find lists of
/// other valid hosts, send packets to a subset of them, even participate
/// in a peer2peer network with other hosts.

class cRouter : public cHostComponent
{
protected:
  list<unsigned long> m_hid;    // A list of host-ids this is connected to.

public:
  cRouter(cHost &host);
  virtual ~cRouter(void);

  /// Send a message to the network.
  /// Note the message is sent to ALL hosts.
  /// @param pMsg A pointer to the message.  This message will be duped during
  /// this call, the original caller retains ownership of the supplied
  /// msg, so you may delete it after the call.
  virtual void send(cMsg* pMsg) =0;

  /// Receive a message from the network.
  /// This is called by the host (i.e., a subclass of cHost).
  /// @param pMsg A pointer to the message.  The host (i.e. the caller of this
  /// function) retains ownership of the message, do NOT delete it.
  /// If you need your own copy, you'll have to dup() it.
  virtual void handleMessage(cMsg* pMsg) =0;

  /// Called each tick.
  /// Called regularly (each cHost::TICK_TIME) by the host.
  virtual void handleTick(void) { }

  /// Called each check-interval.
  /// Called regularly (each cHost::CHECK_TIME) by the host.
  virtual void handleCheck(void) { }

  /// An iterator.
  list<unsigned long>::const_iterator beginHID(void) const
        { return m_hid.begin(); }
  /// An iterator.
  list<unsigned long>::const_iterator endHID(void) const
        { return m_hid.end(); }
  
  /// Reset everything.
  /// This member is called by the host when the simulation is about to start.
  virtual void initialize(void);

  /// System Done.
  /// This member is called by the host when the simulation has ended.
  virtual void finish(void) { }
};

  
};

#endif // __ROUTER_H
