/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include <math.h>

#include "sliding_window.hh"

cSlidingWindow::cSlidingWindow(double windowSizeTime, double start,
    bool exclusive)
: m_windowSize(windowSizeTime), m_buffer(), m_lastTime(start), m_lastVal(0.0),
m_start(start), m_sum(0.0), m_total(0.0), m_max(-INFINITY),
m_exclusive(exclusive)
{ }

void cSlidingWindow::initialize(double newstart)
{
  m_buffer.clear();
  m_lastTime= newstart;
  m_lastVal= 0.0;
  
  m_start= newstart;
  m_sum= 0.0;
  m_total= 0.0;
  m_max= -INFINITY;
//  m_exclusive= exclusive
}

void cSlidingWindow::collect(double time, double value)
{
  update_to(time);

  // append the new data
  R_2 tmp;

  tmp[0] = time; tmp[1] = value;
  m_buffer.push_back(tmp);

  // update mean sum
  m_total += value;
  m_sum += m_lastVal;
  m_lastVal= value;
  m_lastTime = time;

  getAverage(time);        // to update Max
}
  

double cSlidingWindow::getAverage(double time)
{
  update_to(time);

  if (m_buffer.empty())
    return 0.0;
  list<R_2>::iterator b= m_buffer.begin();
  double t= m_lastTime -(*b)[0];
  if (t <=0.0)
    return 0.0;
 
  double ret= m_sum / t;
  if (!m_exclusive)
    ret= (m_sum -(*b)[1] +m_lastVal) /t;

  if (time-m_start >= m_windowSize && m_max < ret)
    m_max= ret;
  return ret;
}

double cSlidingWindow::getMax(double time)
{
  if (time >=0.0)
    getAverage(time);

  return m_max;
}

void cSlidingWindow::update_to(double time)
{
  if (time <0.0)
    time= m_lastTime;

  list<R_2>::iterator iter= m_buffer.begin();

  while (iter != m_buffer.end() && (*iter)[0] < time -m_windowSize)
    {
    m_sum -= (*iter)[1];
    iter++;
    m_buffer.pop_front();
    }
  
}

double cSlidingWindow::getTotalAverage(double time)
{
  if (time <0.0)
    time= m_lastTime;

  // This isn't quite correct, I SHOULD be substracting m_lastVal, but
  // then I'd have to substract firstVal for !m_exclusive, and I don't have
  // that...  Anyhow, the difference is very minor.
  return m_total/ (time-m_start);
}

