/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include "vector.hh"

template<unsigned N, class T>
cVector<N, T>::cVector() {
}

template<unsigned N, class T>
cVector<N, T>::cVector(const cVector<N, T>& src) {
  for (unsigned i = 0; i < N; i++)
    m_data[i] = src.m_data[i];
}

template<unsigned N, class T>
const cVector<N, T>& cVector<N, T>::assign(const cVector<N, T>& src) {
  for (unsigned i = 0; i < N; i++)
    m_data[i] = src.m_data[i];

  return *this;
}

template<unsigned N, class T>
const cVector<N, T>& cVector<N, T>::assign(const T& value) {
  for (unsigned i = 0; i < N; i++)
    m_data[i] = value;

  return *this;
}

template<unsigned N, class T>
const cVector<N, T>& cVector<N, T>::operator=(const cVector<N, T>& src) {
  return assign(src);
}

template<unsigned N, class T>
const cVector<N, T>& cVector<N, T>::operator=(const T& value) {
  return assign(value);
}

template<unsigned N, class T>
const T& cVector<N, T>::at(unsigned pos) const {
  assert(pos < N);
  return m_data[pos];
}

template<unsigned N, class T>
T& cVector<N, T>::at(unsigned pos) {
  assert(pos < N);
  return m_data[pos];
}

template<unsigned N, class T>
const T& cVector<N, T>::operator[](unsigned pos) const {
  return at(pos);
}

template <unsigned N, class T>
T& cVector<N, T>::operator[](unsigned pos) {
  return at(pos);
}

template<unsigned N, class T>
const cVector<N, T>& cVector<N, T>::add(const cVector<N, T>& src) {
  for (unsigned i = 0; i < N; i++)
    m_data[i] += src.m_data[i];

  return *this;
}

template<unsigned N, class T>
const cVector<N, T>& cVector<N, T>::subtract(const cVector<N, T>& src) {
  for (unsigned i = 0; i < N; i++)
    m_data[i] -= src.m_data[i];

  return *this;
}

template<unsigned N, class T>
const cVector<N, T>& cVector<N, T>::scale(const T& value) {
  for (unsigned i = 0; i < N; i++)
    m_data[i] *= value;

  return *this;
}

template<unsigned N, class T>
const cVector<N, T>& cVector<N, T>::operator+=(const cVector<N, T>& src) {
  return add(src);
}

template<unsigned N, class T>
const cVector<N, T>& cVector<N, T>::operator-=(const cVector<N, T>& src) {
  return subtract(src);
}

template<unsigned N, class T>
const cVector<N, T>& cVector<N, T>::operator*=(const T& value) {
  return scale(value);
}

template<unsigned N, class T>
T cVector<N, T>::scalarProduct(const cVector<N, T>& src) const {
  T sum = 0;

  for (unsigned i = 0; i < N; i++)
    sum += m_data[i] * src.m_data[i];

  return sum;
}

template<unsigned N, class T>
cVector<N, T> cVector<N, T>::operator+(const cVector<N, T>& rhs) const {
  cVector<N, T> result(*this);
  result += rhs;
  return result;
}

template<unsigned N, class T>
cVector<N, T> cVector<N, T>::operator-(const cVector<N, T>& rhs) const {
  cVector<N, T> result(*this);
  result -= rhs;
  return result;
}

template<unsigned N, class T>
T cVector<N, T>::operator*(const cVector<N, T>& rhs) const {
  return scalarProduct(rhs);
}

template<unsigned N, class T>
cVector<N, T> cVector<N, T>::operator*(const T& value) const {
  cVector<N, T> result(*this);
  result *= value;
  return result;
}

template<unsigned N, class T>
std::ostream& operator<<(std::ostream& os,
                         const cVector<N, T>& value) {
  os << "(" << value[0];
  for (unsigned i = 1; i < N; i++)
    os << ", " << value[i];
  os << ")";
  
  return os;
}

// implementations
template class cVector<2, double>;
template class cVector<3, double>;
template class cVector<2, INT32>;
template class cVector<3, INT32>;

template std::ostream& operator<<(std::ostream&, const cVector<2, double>&);
