/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/

#ifndef __VECTOR_H
#define __VECTOR_H

#include <iostream>

#include "gen.hh"

namespace Condel {

/// An n-dimensional numeric vector.
/// Type T can be anything numerical, but I really recommend (double)
/// for the purposes of Adam.

template <unsigned N, class T>
class cVector {
private:
  T m_data[N];

public:
  cVector(void);
  cVector(const cVector<N, T>& src);
  ~cVector(void) {};

  // assignments
  const cVector<N, T>& assign(const cVector<N, T>& src);
  /// Set all Elements to the same value.
  /// @param value A numerical value that all components of the vector
  /// are set to.
  const cVector<N, T>& assign(const T& value);
  const cVector<N, T>& operator=(const cVector<N, T>& src);
  const cVector<N, T>& operator=(const T& value);

  // accessors
  const T& at(unsigned pos) const;
  T& at(unsigned pos);
  const T& operator[](unsigned pos) const;
  T& operator[](unsigned pos);
  
  // basic algebraic operations (addition, subtraction, scalar
  // multiplication...)
  const cVector<N, T>& add(const cVector<N, T>& src);
  const cVector<N, T>& subtract(const cVector<N, T>& src);
  const cVector<N, T>& scale(const T& value);
  const cVector<N, T>& operator+=(const cVector<N, T>& src);
  const cVector<N, T>& operator-=(const cVector<N, T>& src);
  const cVector<N, T>& operator*=(const T& value);
  cVector<N, T> operator+(const cVector<N, T>& src) const;
  cVector<N, T> operator-(const cVector<N, T>& src) const;
  T operator*(const cVector<N, T>& src) const;
  cVector<N, T> operator*(const T& value) const;
  
  // scalar product
  T scalarProduct(const cVector<N, T>& src) const;

};

/// A typical vector.
/// Most vectors in Adam use this type.
typedef cVector<2, double> R_2;
typedef cVector<3, double> R_3;
typedef cVector<2, INT32> Z_2;
typedef cVector<3, INT32> Z_3;

};

template<unsigned N, class T>
std::ostream& operator<<(std::ostream& os, const cVector<N, T>& value);
                         
#endif // __VECTOR_H
