/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include "world.hh"
#include "instance.hh"
#include "scenario.hh"
#include "load.hh"

cWorld::cWorld(cHost &host)
: map<const OBJECT_ID, shared_ptr<cInstance> >(), cHostComponent(host),
  m_name(""), m_simtime(0.0)
{
}

cWorld::cWorld(cHost &host, const string& name)
: map<const OBJECT_ID, shared_ptr<cInstance> >(), cHostComponent(host),
  m_name(name), m_simtime(0.0)
{
}

auto_ptr<cWorld> cWorld::clone(void)
{
  auto_ptr<cWorld> ret(new cWorld(host(), getName()));
  *ret = *this;

  return ret;
}

const cWorld &cWorld::operator= (const cWorld& src)
{
  // Get rid of all the old stuff
  clear();

  const_iterator iter;
  // clone all instances
  for (iter = src.begin(); iter != src.end(); iter++) {
    (*this)[(*iter).first].reset((*iter).second->clone());
  }

  setSimTime(src.simTime());

  return *this;
}

bool cWorld::isReplicated(const OBJECT_ID& objectID) const
{
  return (find(objectID) != end());
}

cInstance& cWorld::getInstance(const OBJECT_ID& objectID)
{
  const_iterator iter = find(objectID);
  assert(iter != end());
  assert((*iter).second.get());
  
  return *((*iter).second.get());
}

const cInstance& cWorld::getInstance(const OBJECT_ID& objectID) const
{
  const_iterator iter = find(objectID);
  assert(iter != end());
  assert((*iter).second.get());
  
  return *((*iter).second.get());
}

void cWorld::addInstance(const cInstance* pInstance) {
  assert(pInstance);
  assert(!isReplicated(pInstance->getObjectID()));

  // clone the original and link it to this world
  cInstance* pClone = pInstance->clone();
  (*this)[pInstance->getObjectID()].reset(pClone);
}

unsigned long long cWorld::getTotalLength(void)
{
  unsigned long long ret=0;

  const_iterator iter;
  for (iter= begin(); iter != end(); iter++)
    ret += (*iter).second->getCodedLength();

  return ret;
}

void cWorld::simulate(double newtime, cLoadMeter &load)
{
  host().getScenario().simulate(*this, newtime, load);
}

void cWorld::initialize(void)
{
  clear();
  setSimTime(host().now());
}

