#!/bin/sh
# usage: make_release.sh <path in svn repo> <target name>
# example: make_release.sh branches/RELEASE_1.0.0/adam adam-1.0.0

error() {
    echo "ERROR: $@"
    return 0
}

failure() {
    error "$@"
    usage
    echo "aborting"
    exit 1
}

usage() {
    echo "usage: make_release.sh <path in repo> <target tarball>"
    return 0
}

# sanity check
[ $# -eq 2 ] || failure "missing argument"

# setup variables
SVN_BASE="https://svn.mulf.tu-berlin.de/mulf_svn/condel"
SVN_PONG="${SVN_BASE}/$1"

TARGET=`basename $2`

REL_BASE=`pwd`/"rel_base"
REL_PONG="${REL_BASE}/$TARGET"


# export base directory
echo "exporting repo"
svn export "${SVN_PONG}" "${REL_PONG}" 1>/dev/null || failure "could not export ${SVN_PONG}"


# create akaroa patch; apply with 'patch -p1'
echo "creating Akaroa2 patch"
pushd "${REL_PONG}"
AKAROA_VENDOR="akaroa.original"
AKAROA_PONG="akaroa.patched"

svn export "${SVN_BASE}/vendor/akaroa/current" "${AKAROA_VENDOR}" 1>/dev/null || failure "could not export akaroa vendor branch"
mv akaroa "${AKAROA_PONG}"
diff -r -c -N "${AKAROA_VENDOR}" "${AKAROA_PONG}" > akaroa.patch #|| failure "could not diff akaroa patch"

# remove akaroa (we are not allowed to distribute that)
rm -rf "${AKAROA_VENDOR}" "${AKAROA_PONG}" || failure "could not remove akaroa subdir"
popd



# create tarball
echo "creating target ${TARGET}"
pushd "${REL_BASE}"
tar czvf "${TARGET}.tar.gz" "${TARGET}" || failure "could not create tarball ${TARGET}.tar.gz"
popd
mv "${REL_BASE}/${TARGET}.tar.gz" . || failure "could not move tarball to final destination"

# cleanup
echo "cleaning up"
rm -Rf "${REL_BASE}"
echo "done"
