/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/

#ifndef __BALL_H
#define __BALL_H

#include "gen.hh"
#include "instance.hh"
#include "vector.hh"
#include "attribute.hh"

namespace Condel {

// Pong's ball object.

class cBall : public cInstance {
public:
  // the ball attribute IDs
  static ATTRIBUTE_ID s_positionXID;  ///< ball's X position.
  static ATTRIBUTE_ID s_positionYID;  ///< ball's Y position.
  static ATTRIBUTE_ID s_velocityXID;  ///< ball's X velocity.
  static ATTRIBUTE_ID s_velocityYID;  ///< ball's Y velocity.
  static ATTRIBUTE_ID s_seedID;       ///< ball's random seed, used for
                                      /// throw-in
  
private:
  R_2 m_position; 
  R_2 m_velocity;
  unsigned int m_seed;

protected:
  cBall(const OBJECT_ID& id, const string& name);
public:
  virtual ~cBall(void) {}

  // accessors
  void setPosition(const R_2& pos) { m_position = pos; }
  void setVelocity(const R_2& v);
  const R_2& getPosition(void) const { return m_position; }
  const R_2& getVelocity(void) const { return m_velocity; }

  /// Set a new random Y speed.
  /// This is based on the seed in m_seed.
  /// It's used when the ball is thrown in.
  void randomYSpeed(void);
  
  static cInstance* create(const OBJECT_ID& id, const string &name);
  
  virtual cAttributeSet getAttributes(void) const;

  virtual void setAttributes(const cAttributeSet& na);
  
  virtual cBall* clone(void) const;

  /// Compares two instances.
  /// Normally, this is based on the cartesian distance.
  /// For the ideal case, velocity is also used.
  /// @param other Another instance of the same object.
  /// @return A metric of the similarity of the two instances.
  virtual double getDistance(const cInstance& other) const;

  /// Simulate this instance.
  /// Move the ball according to its speed.  Reflect of upper and lower
  /// boundaries.
  /// @param newtime A simulation time in the future of simTime().  Only
  /// the difference between the two times is advanced.
  /// @param load A tool to measure CPU and memory usage during the simulation.
  virtual void simulate(double newtime, cLoadMeter &load);

};

};

#endif // __BALL_H
