/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/

#ifndef __RACKET_H
#define __RACKET_H

#include "gen.hh"
#include "instance.hh"
#include "vector.hh"
#include "attribute.hh"

namespace Condel {

/// Pong's racket object.
/// This represents the players.
///
/// Note there are @em two racket objects, one for the left player and one for
/// the right player.

class cRacket : public cInstance {
public:
  // the racket attribute IDs
  static ATTRIBUTE_ID s_positionXID;    ///< racket's X position.
  static ATTRIBUTE_ID s_positionYID;    ///< racket's Y position.  Note this is
                                        /// constant.
  static ATTRIBUTE_ID s_velocityXID;    ///< racket's X velocity.
  static ATTRIBUTE_ID s_velocityYID;    ///< racket's Y velocity.  Note this is
                                        /// 0.
  
private:
  R_2 m_position;    // this is the position of the centre of the racket
  R_2 m_velocity;    // horizontal speed is always zero
public:
  cRacket(const OBJECT_ID& id, const string& name);
  virtual ~cRacket(void) {}

  // accessors
  void setPosition(const R_2& pos) { m_position = pos; }
  void setVelocity(const R_2& pos) { m_velocity = pos; }
  const R_2& getPosition(void) const { return m_position; }
  R_2& getPosition(void) { return m_position; }
  const R_2& getVelocity(void) const { return m_velocity; }
  R_2& getVelocity(void) { return m_velocity; }

  // some convenience functions
  double upperEnd(void) const;
  double lowerEnd(void) const;
  
  static cInstance* create(const OBJECT_ID& id, const string& name);
  
  virtual cAttributeSet getAttributes(void) const;
  virtual void setAttributes(const cAttributeSet& na);
  
  virtual cRacket* clone(void) const;

  /// Compares two instances.
  /// Normally, this is based on the cartesian distance.
  /// For the ideal case, velocity is also used.
  /// @param other Another instance of the same object.
  /// @return A metric of the similarity of the two instances.
  virtual double getDistance(const cInstance& other) const;

  /// Simulate this instance.
  /// Move the racket according to its speed.  Stop at upper and lower
  /// boundaries.
  /// @param newtime A simulation time in the future of simTime().  Only
  /// the difference between the two times is advanced.
  /// @param load A tool to measure CPU and memory usage during the simulation.
  virtual void simulate(double newtime, cLoadMeter &load);
};
  
};

#endif // __RACKET_H
