/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include <math.h>

#include "score.hh"
#include "instance_factory.hh"

ATTRIBUTE_ID cScore::s_scoreID = cAttributeFactory::registerAttribute("cScore::score");

cScore::cScore(const OBJECT_ID& id, const string& name)
: cInstance(id, name, 1)
{ }

cAttributeSet cScore::getAttributes() const {
  cAttributeSet ret;

  ret.add(cAttribute(s_scoreID, getScore()));

  return ret;
}

void cScore::setAttributes(const cAttributeSet& na) {
  cAttributeSet::const_iterator iter;

  double score = getScore();

  for (iter = na.begin(); iter != na.end(); iter++) {
    const cAttribute& attr = (*iter).second;
    
    if (attr.getID() == s_scoreID) score = attr.getValue();
    else assert(false);                // unkown attribute found
  }

  setScore(score);
}

cScore* cScore::clone() const {
  cScore* pNew = new cScore(getObjectID(), getName());

  pNew->setSimTime(simTime());
  pNew->setUpdateTime(updateTime());

  pNew->setScore(getScore());

  return pNew;
}

double cScore::getDistance(const cInstance& other) const {
  assert(other.getObjectID() == getObjectID());
  return fabs((double) (getScore() - ((const cScore&) other).getScore()));
}

void cScore::simulate(double newtime, cLoadMeter &) {
  // nothing to be done here
  setSimTime(newtime);
}

cInstance* cScore::create(const OBJECT_ID& id, const string& name) {
  return new cScore(id, name);
}

static const class cScoreSlave { public: cScoreSlave() {
    cInstanceFactory::registerInstance(string("cScore"), cScore::create);
} } scoreSlave;
