/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jan Sablatnig, Sven Grottke, Jiehua Chen.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/

#ifndef __SCORE_H
#define __SCORE_H

#include "gen.hh"
#include "instance.hh"
#include "attribute.hh"

namespace Condel {

/// Swarm's score object.
/// This is used for scorekeeping; it's actually just an number.
///
/// Note there are @em many score objects, one for each player.

class cScore : public cInstance {
private:
  // the score attribute IDs
  static ATTRIBUTE_ID s_scoreID;        ///< The score.  In honey-units.
  
private:
  double m_score;
public:
  cScore(const OBJECT_ID& id, const string& name);
  virtual ~cScore(void) {}

  // accessors
  void setScore(double score) { m_score = score; }
  double getScore(void) const { return m_score; }
  double& getScore(void) { return m_score; }
  
  static cInstance* create(const OBJECT_ID& id, const string& name);

  virtual cAttributeSet getAttributes(void) const;
  virtual void setAttributes(const cAttributeSet& na);
  
  virtual cScore* clone(void) const;

  virtual double getDistance(const cInstance& other) const;

  /// Simulate this instance.
  /// Does nothing.
  /// @param newtime A simulation time in the future of simTime().  Only
  /// the difference between the two times is advanced.
  /// @param load A tool to measure CPU and memory usage during the simulation.
  virtual void simulate(double newtime, cLoadMeter &load);
};
  
};

#endif // __SCORE_H
