#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# efficiency analysis for distributed simulations
# usage: ak_eff.sh simulation [parameters ...]

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}

# include common functions
source "${CONDEL_ROOT}/experiments/common_files/gen.sh"
source "${CONDEL_ROOT}/experiments/common_files/akaroa.sh"

# sanity check
[ $# -ge 1 ] || {
    error "usage: ak_eff.sh simulation [parameters ...]"
}
    
# measure slave load rating as the percentage of slaves that are actually busy
slave_load() {
    # set path to binary
    ak_setup_vars

    # compute load once a second
    while [ 1 ]; do
        sl_active_list=`get_active_slaves`
        sl_busy_list=`get_busy_slaves`
        sl_active=`get_size ${sl_active_list}`
        sl_busy=`get_size ${sl_busy_list}`
        [ "${sl_active}" = 0 ] && {
            sl_load=0
        } || {
            sl_load=`echo "4 k ${sl_busy} ${sl_active} / p q" | dc`
        }
        echo "${sl_load}" >> "${LOAD_LOG}"
        sleep 1
    done
}

# compute average load rating from log file
avg_load() {
    al_load_sum=0
    al_load_count=0
    while read al_load; do
        al_load_sum=`echo "6 k ${al_load_sum} ${al_load} + p q" | dc`
        al_load_count=`expr "${al_load_count}" + 1`
    done
    echo "4 k ${al_load_sum} ${al_load_count} / p q" | dc
}

# exit handler
kill_load_logger() {
    kill "${SL_PID}"
}


# create log file
LOAD_LOG="slave_load.log"
echo "0" > "slave_load.log"
log "writing loads to ${LOAD_LOG}"

# start measurement thread
slave_load &
SL_PID=$!
add_exit_handler kill_load_logger

# start application
time -p "$@"

# stop measurement thread
kill "${SL_PID}" 1>/dev/null 2>&1
wait "${SL_PID}"
remove_exit_handler kill_load_logger

# compute average
LOAD=`avg_load < "${LOAD_LOG}"`
log "Load average: ${LOAD}"
