#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Sven Grottke, Jan Sablatnig, Jiehua Chen.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# measures the network load
#
# usage: measure_netload HOST_LIST vector-file


# config
AVG_MAX_SCRIPT="measure_windowed_max_avg.awk"

# error function
failure() {
    echo "ERROR: $@"
    exit 1;
}

# get n-th argument
get_nth() {
    shift $1
    echo $1
}

# sanity checks
[ -r $2 ] || failure "$2 is not a readable file"

# set some constants
WINDOW_SIZE=10
GRANULARITY=0.05
START_TIME=${WINDOW_SIZE}

# calculate END_TIME
END_TIME=`tail -1 $2 | cut -f 2`

echo "WINDOW_SIZE: ${WINDOW_SIZE}"
echo "START_TIME: ${START_TIME}"
echo "END_TIME: ${END_TIME}"
echo "GRANULARITY: ${GRANULARITY}"
echo "Host list: $1"

# split the vector file
splitvec $2 || failure "splitvec failed"

VEC_BASE_NAME=`basename $2 .vec`
VEC_DIR_NAME=`dirname $2`

# process hosts

for host in $1; do
    for key in messagesSent messagesReceived; do
        echo "checking for ${key} on ${host}"
        # find the file that contains data for that host and key
        for file in "${VEC_DIR_NAME}/${VEC_BASE_NAME}-"*.vec; do
            fh=`head -1 ${file}`
            fhost=`get_nth 4 ${fh}`
            [ "${fhost}" = "\"${host}\"" ] || continue;
            fkey=`get_nth 5 ${fh}`
            [ "${fkey}" = "\"${key}\"" ] || continue;
            # found it!
            awk -v WINDOW_SIZE="${WINDOW_SIZE}" -v START_TIME="${START_TIME}" -v END_TIME="${END_TIME}" -v GRANULARITY="${GRANULARITY}" -f ${AVG_MAX_SCRIPT} < $file
        done
    done
done
