# measure the windowed average and maximum of a netload plot, for both number
# and size of messages per second
#
# the caller will set the WINDOW_SIZE variable (via the -v option);
# WINDOW_SIZE will be measured in seconds;
# the caller will also set the GRANULARITY variable, which defines the
# sampling distance; it will be measured in seconds;
# the caller will also set START_TIME and END_TIME in seconds
# the result will be written to stdout as a single line of the form
# 'avg_msg max_msg avg_size max_size'

BEGIN {
  size = 0;   # starting index
}

/^\# vector*/ {
}

/[[:digit:]]+/ {
  # gather raw data sorted by time
  t[size] = $1;
  p[size] = $2;
  size++;
}

END {
  # find starting position
  avg_size = 0; max_size = -1;
  avg_num = 0; max_num = -1;
  
  current_time = START_TIME;
  head_index = 0;
  tail_index = 0;
  sum_size = 0;
  sum_num = 0;

  while (current_time < END_TIME) {
    # advance head index
    while (t[head_index] < current_time && head_index < size) {
      sum_size += p[head_index++];
      sum_num++;
    }

    # advance tail index
    while (t[tail_index] < current_time - WINDOW_SIZE) {
      sum_size -= p[tail_index++];
      sum_num--;
    }

    # adjust avg and maximum
    avg_size += sum_size / WINDOW_SIZE;
    if (sum_size / WINDOW_SIZE > max_size)
      max_size = sum_size / WINDOW_SIZE;
    avg_num += sum_num / WINDOW_SIZE;
    if (sum_num / WINDOW_SIZE > max_num)
      max_num = sum_num / WINDOW_SIZE;
    
    # advance current_time
    current_time += GRANULARITY;
  }

  avg_size /= (END_TIME - START_TIME) / GRANULARITY;
  avg_num /= (END_TIME - START_TIME) / GRANULARITY;
  
  print avg_num " " max_num " " avg_size " " max_size;
}
