#!/usr/bin/perl
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Jiehua Chen, Jan Sablatnig, Sven Grottke.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.


# usage: plot2d.pl filename startRange endRange parameter-to-regulate [-i ideal_filename] [-lx] [-ly] HOST1 measurement1 HOST2 measurement2 ...

# for pong we have following tips
#############################################################
#                       HOSTs are:
#
# "pong_loose.player@3"                  "pong_loose.eye"
# "pong_loose.player@2"
#
#############################################################
#             parameters are for examples:
#
#   "loss rate"      "avg. delay"      "regular update rate"
#   "ball update distance"           "racket update distance"
#
#############################################################
#                measurements are:
#
#"avg. packets sent"                    "max. packets sent"
#"avg. bytes sent"                      "max. bytes sent"
#"avg. packets received"                "max. packets received"        
#"avg. bytes received"                  "max. bytes received"
#"overall CPU time"                     "stddev CPU time"
#"avg. CPU time"
# "score achieved /s"            
#"G"                  "avg. G"
#"G (cBall::positionX)"
#"avg. G (cBall::positionX)"
#"G (cBall::positionY)"
#"avg. G (cBall::positionY)"
#"G (cBall::velocityX)"
#"avg. G (cBall::velocityX)"
#"G (cBall::velocityY)"
#"avg. G (cBall::velocityY)" 
#"G (cRacket::positionX)"
#"avg. G (cRacket::positionX)"
#"G (cRacket::positionY)"
#"avg. G (cRacket::positionY)"
#"G (cRacket::velocityX)"
#"avg. G (cRacket::velocityX)"
#"G (cRacket::velocityY)"
#"avg. G (cRacket::velocityY)"
#"G (cScore::score)"
#"avg. G (cScore::score)"
#"end time"
#"D"
#"D (cBall::positionX)"
#"D (cBall::positionY)"
#"D (cBall::velocityX)"
#"D (cBall::velocityY)"
#"D (cRacket::positionX)"
#"D (cRacket::positionY)"
#"D (cRacket::velocityX)"
#"D (cRacket::velocityY)"
#"D (cScore::score)"


# for swarm we have following tips
# consistency can also be "opt" instead "loose"
#############################################################
#                       HOSTs are:
#
# "pong_loose.player@n[m]"                  "pong_loose.eye"
# "pong_loose.average"
# n and m are numbers created from omnetpp
#
#############################################################
#             parameters are for examples:
#
#   "loss rate"      "avg. delay"      "regular update rate"
#   "bee update distance"
#
#############################################################

use strict;

# hash table for plot display(y-achs)
my %trans = {
    "avg. G" => "discontinuity @^{^{\\_\\_}}G ",
    "D" => "divergence D",
    "score achieved /s" => "utility",
};

sub usage{
     print STDERR "usage: plot2d.pl filename start_Range end_Range parameter-to-regulate [-i ideal_filename] [-lx] [-ly] HOST1 measurement1 HOST2 measurement2 ... .\n";
}

sub failure{
    my $failedparm = shift;
    
    
    if ( defined $failedparm ){
        print STDERR "$failedparm format is wrong\n";
    }
    else{
        print STDERR "parameters failed!\n";
      }
    &usage;
    print STDERR "Please see the usage for more details.\n";
    die "aborting.";
}

my @QUOTED_ARGV = @ARGV;
map { s/(\W)/\\$1/g; } @QUOTED_ARGV;

# parse parameters
my $argc = 0;

# start with mandatory paramaters: filename start stop parm
if ( @ARGV - $argc < 4 ) { &failure; }
my $scafile = <$QUOTED_ARGV[$argc++]>; #.sca file with various runs data.
if ( $ARGV[$argc] !~ /^[\.\d]+/ ) { &failure("start_range"); }
my $start = $ARGV[$argc++];   # start range
if ( $ARGV[$argc] !~ /^[\.\d]+/ ) { &failure("end_range"); }
my $end = $ARGV[$argc++];     # end range

# parameter-to-regulate
my $quoted_parm = $QUOTED_ARGV[$argc]; 
my $parm = $ARGV[$argc];
my $parm_with_no_space = $parm;
$parm_with_no_space =~ s/(\b)([\W]*)(\b)/\1\3/g;

$argc++;

#variables for ideal_consistency
my ($scafile_ideal,$y_ideal_value,$y_ideal_count);
#note if plot with ideal_consistency
my $with_ideal = 0;
if ( $ARGV[$argc] eq "-i" ){
    $with_ideal = 1;
    $scafile_ideal = <$QUOTED_ARGV[$argc + 1]>;
    $argc += 2;
}

#note if plot with x logscale
my $with_xlogscale = 0;
if ( $ARGV[$argc] eq "-lx" ){
    $with_xlogscale = 1;
    $argc++;
}

#note if plot with y logscale
my $with_ylogscale = 0;
if ( $ARGV[$argc] eq "-ly" ){
    $with_ylogscale = 1;
    $argc++;
}

# the number of hosts and measurements should be even 
if ( (@ARGV - $argc) % 2 != 0 ){ &failure; }
my $numberOfHosts = ( @ARGV - $argc ) / 2;
my (@measurements,@hosts);
my ($network,$host,$scenario);

my (@plotfiles,@sca_lines,@plotfiles_ideal,@sca_lines_ideal);
my $run_count=0;
my @parms;
my ($x_value,$y_value,$y_count);

#store the weights of variance and discontinuity
my $varianceWeights;
my $discoWeights;
my $countWeights=0;

# print "$QUOTED_ARGV[$argc]\n";
for( my $i = 0 ; $i < $numberOfHosts ; $i++ ){
    open(SCAFILE,$scafile);
    push(@plotfiles,"./plot$i.data"); #file for the output
    open(PLOTFILE,">$plotfiles[$i]");

    #split the name into network and host
    $QUOTED_ARGV[$argc+2*$i] =~ /^(\w+)\\\.(.+)$/ || die "wrong format of hostname\n";
    $network = $1;
#    print "$network\.hosts[$i]\n";
    $hosts[$i] = $2;
    
    $network =~ /^([^_]+)_(.+)$/ || die "wrong format of network name\n";
    $scenario = $1;
    #$consist = $2;
    $measurements[$i] = $QUOTED_ARGV[$argc+2*$i+1];
    $y_value= 0.0; $y_count= 0;

    @sca_lines = <SCAFILE>;
    #find the accordance x- & y-value in scafile 
    #then copy them into the plotfiles 
    foreach( @sca_lines ){
	if ( /^run/ ){
	    $run_count++;
	    next;
	}
	
	#find the y-value for a consistency case
	if ( /\"${network}\.$hosts[$i]\"\s+\"${measurements[$i]}\"\s+([\.\d\-e]+)/ ){
	   # $y_value += $1; $y_count++;
            ${y_value} = $1;
            next;
        }
	
	#find the x-value for a consistency case
	if ( /\"$parm\"\s+([\.|\d-]+)/ ){
	    $x_value = $1;
            # $y_value /= $y_count;
            print PLOTFILE "$x_value  $y_value\n";
            if( ${x_value} < ${start} ){
                $start = ${x_value};
            }
            #$y_value= 0.0; $y_count=0;
	    next;
	}

        #find the variance weights in sca_file once
        if ( /(\"variance weights\".+\".+\")\s+0$/ && $countWeights == 0 ){
            $countWeights++;
            $varianceWeights = $1;
            $varianceWeights =~ s/::/_/g; 
            $varianceWeights =~ s/([_@^])/\\\\$1/g;
            $varianceWeights =~ s/(\")/\\$1/g;
            if( length($varianceWeights) > 100 ){
                $varianceWeights =~ s/(.{100,}?\d)\\\s/$1\\n/g;
            }
            next;
        }

        #find the discontinuity weights in sca_file once
        if ( /(\"disco weights\".+\".+\")\s+0$/ && $countWeights == 1 ){
            $countWeights++;
            $discoWeights = $1;
            $discoWeights =~ s/::/_/g;
            $discoWeights =~ s/([_@^\"])/\\\\$1/g;
            $discoWeights =~ s/(\")/\\$1/g;
            if ( length($discoWeights) > 100 ){
                $discoWeights =~ s/(.{100,}?\d)\\\s/$1\\n/g;
            }
            next;
        }
        
	#filter the parameter values 
	if ( $run_count == 1 ){
	    if (/\"parm\"\s+\"(.+)\"\s+([\.\d\-e]+)/){
		if ( "$1" ne $parm){
		    push(@parms,($1, $2, "\\n"));
		}
	    }
	}
    }
    close(PLOTFILE);
    close(SCAFILE);
}

# option -i to compare with ideal_file
# notice the non-zero y-value
my $ifParmExist_ideal;
my @ifNotZero_ideal;
if ( $with_ideal == 1 ){
    # search for the parm in ideal, if
    # it exist, then notice the variable 
    open(SCAFILE_IDEAL,$scafile_ideal) || die "can't open ideal file $scafile_ideal";
    @sca_lines_ideal = <SCAFILE_IDEAL>;
    foreach( @sca_lines_ideal ){
        if ( /\"$parm\"\s+([\.|\d]+)/ ){
            $ifParmExist_ideal = 1;
            last;
        }    
    }
    close(SCAFILE_IDEAL);
    
    my $data;
    #.sca file for ideal_consistency 
    $scafile_ideal = <$QUOTED_ARGV[5]>;
    
    for( my $i = 0 ; $i < $numberOfHosts ; $i++ ){
	#file for the output in pong_ideal
	push(@plotfiles_ideal,"./plot$i.ideal.data");
	open(SCAFILE_IDEAL,$scafile_ideal) || die "can't open ideal file $scafile_ideal";
	open(PLOTFILE_IDEAL,">$plotfiles_ideal[$i]");
        $y_ideal_value= 0.0; $y_ideal_count= 0;

	#find the accordant y-values for ideal_consistency
        @sca_lines_ideal = <SCAFILE_IDEAL>;
        foreach( @sca_lines_ideal ){
	    if ( /\"${scenario}_ideal\.$hosts[$i]\"\s+\"${measurements[$i]}\"\s+([\.\d\-e]+)/ ){
                ${ifNotZero_ideal[$i]} = ( $1!=0 );
               # $y_ideal_value += $1; $y_ideal_count++;
                ${y_ideal_value} = $1;
            }
            # find the x-value when parm exists
            if ( /\"$parm\"\s+([\.|\d]+)/ ){
                $x_value = $1;
                # $y_ideal_value /= $y_ideal_count;
                print PLOTFILE_IDEAL "$x_value  $y_ideal_value\n";
                $ifParmExist_ideal= 1;
                # $y_ideal_value= 0.0; $y_ideal_count= 0;
            }
            
        }

        # build an x-axis for ideal the same with other consistency 
        if( !$ifParmExist_ideal )
          {
          open(PLOTFILE,$plotfiles[0]);
          my @lines = <PLOTFILE>;

          # $y_ideal_value /= $y_ideal_count;
          # print the ideal value into plotfile_ideal with the
          # same x value as in plotfile
          foreach $data (@lines)
            {
            if ( $data =~ /^([\.\d\-e]+)\s+/ )
              {
              $x_value = $1;
              print PLOTFILE_IDEAL "$x_value  ${y_ideal_value}\n";
              }
            }
          }
        
        close(SCAFILE_IDEAL);
        close(PLOTFILE_IDEAL);
        close(PLOTFILE); 
    }
}

#write the script for the plot
my $plotscript = <./plotscript.gnuplot>;
open(PLOTSCRIPT,">$plotscript");
my $time = localtime;
## Two variations:                         ## 
## 1. suitable for 6 pictures in A4 format ##
# print PLOTSCRIPT "set size 0.6,0.6\n";
# print PLOTSCRIPT "set term postscript eps 14 colour enhanced \"Arial, 18\"\n";
## 2. exactly one picture in A4 format
print PLOTSCRIPT "set term postscript colour enhanced\n";
print PLOTSCRIPT "set xlabel \"$parm\"\n";
print PLOTSCRIPT "set title \" $scafile \\n $time @parms $varianceWeights \\n $discoWeights\"\n";
#print PLOTSCRIPT "set xtics rotate\n";
if ( $with_xlogscale ){
    print PLOTSCRIPT "set logscale x 2\n";
    print PLOTSCRIPT "set xrange [$start:$end]\n";
} elsif( $start < 0 ){ # start-value is negativ
    print PLOTSCRIPT "set xrange [$start:$end]\n";
} else {print PLOTSCRIPT "set xrange [0:$end]\n";
    }
if ( $with_ylogscale ){ 
    print PLOTSCRIPT "set logscale y 2\n";
    print PLOTSCRIPT "set yrange [1:]\n";
} else {
    print PLOTSCRIPT "set yrange [0:]\n";
}
print PLOTSCRIPT "set key below\n";
print PLOTSCRIPT "set grid\n";
#sort the plotdatas
system("sort -n $plotfiles[0] > $plotfiles[0].sorted");
my $transdata;
if( exists $trans{$measurements[0]} ){
    $transdata = $trans{$measurements[0]};
}else{
    $transdata = $measurements[0]; 
}

print PLOTSCRIPT "set ylabel \"$transdata\"\n";
print PLOTSCRIPT "plot \"$plotfiles[0].sorted\" title \"$transdata\" with linespoints";

for( my $i = 1 ; $i < $numberOfHosts ; $i++ ){
    system("sort -n $plotfiles[$i] > $plotfiles[$i].sorted");
    if (exists $trans{$measurements[$i]}) {
        $transdata = $trans{$measurements[$i]};
    }else {
        $transdata = $measurements[$i];
    }
    print PLOTSCRIPT ", \"$plotfiles[$i].sorted\" title \"$transdata\" with linespoints";
}
 
if( $with_ideal ) {
    for( my $i = 0 ; $i < $numberOfHosts ; $i++){
        if ( $ifNotZero_ideal[$i] ){
            system("sort -n $plotfiles_ideal[$i] > $plotfiles_ideal[$i].sorted");
            my $linestyle = "lines";
            # print style for ideal case points or line?
            if( $ifParmExist_ideal ){
                $linestyle = "linespoints";
            }
            # measurementsname will be changed according to the theoretical paper
            if(exists $trans{$measurements[$i]}) {
                $transdata = $trans{$measurements[$i]};
            }else {
                $transdata = $measurements[$i];
            }
            print PLOTSCRIPT ", \"$plotfiles_ideal[$i].sorted\" title \"${transdata}_ideal\" with $linestyle";
        }
    }
}

print PLOTSCRIPT "\n";
close(PLOTSCRIPT);

#execute
system("gnuplot < $plotscript ");

# copy for debug
# my $measurement_with_no_space = $measurements[0];
# $measurement_with_no_space =~ s/(\b)([\W]*)(\b)/\1\3/g;
# system("cp $plotscript ${plotscript}.${parm_with_no_space}.${measurement_with_no_space}");
# for( my $i = 0 ; $i < $numberOfHosts ; $i++ ){
#     system("cp $plotfiles[$i].sorted $plotfiles[$i].sorted.${parm_with_no_space}.${measurement_with_no_space}");
#     if( $with_ideal ){
#         system("cp $plotfiles_ideal[$i].sorted $plotfiles_ideal[$i].sorted.${parm_with_no_space}.${measurement_with_no_space}");
#     }
# }

#remove
system("rm $plotscript");
for( my $i = 0 ; $i < $numberOfHosts ; $i++ ){
   system("rm $plotfiles[$i]");
   system("rm $plotfiles[$i].sorted");
    if ( $with_ideal ){
        system("rm $plotfiles_ideal[$i]");
        system("rm $plotfiles_ideal[$i].sorted");
    }
}
