// distribution function definitions

#ifndef __DISTRIB_H
#define __DISTRIB_H

// base class
class DistribFunction {
public:
  virtual ~DistribFunction() {}
  
  virtual double P(double x) const = 0;
};

// step function
// the step function is defined as 0 for x < m, and 1 for x > m
class Step : public DistribFunction {
private:
  const double m_m;   // mimimum; this is also the expection
public:
  Step(double m) : DistribFunction(), m_m(m) {}
  virtual ~Step() {}

  virtual double P(double x) const;
};

// exponential distribution with minimum m and expectation a
class Exponential : public DistribFunction {
private:
  const double m_m;  // minimum
  const double m_a;  // expectation - minimum
public:
  Exponential(double m, double e) : DistribFunction(), m_m(m), m_a(e - m) {}
  virtual ~Exponential() {}
  
  virtual double P(double x) const;
};

// pareto distribution with expectation a and fitting parameter k
class Pareto : public DistribFunction {
private:
  const double m_cutoff; // cutoff parameter
  const double m_m;      // minimum
  const double m_k;      // fitting parameter
public:
  Pareto(double e, double k, double cutoff = 5);
  virtual ~Pareto() {}
  
  virtual double P(double x) const;
};

#endif // __DISTRIB_H
