/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jiehua Chen, Sven Grottke, Jan Sablatnig.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>

#include <SDL.h>
#include <SDL_gfxPrimitives.h>
#include <math.h>
//#define _XOPEN_SOURCE 500

#include <iostream>
#include <fstream>
#include <string>
#include <list>

using namespace std;

struct systemState{
  double timestamp;
  double states[16];

  systemState(){
    for( int i = 0 ;i < 16 ; i++ ){
      states[i] = INFINITY;
    }
  }
};

void visulisation( list<systemState> history ){
  // init SDL lib
  if (SDL_Init(SDL_INIT_VIDEO) < 0) {
    fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
    exit(1);
  }
  atexit(SDL_Quit);
  
  // init screen
  SDL_Surface* pScreen;
    
  if ((pScreen = SDL_SetVideoMode(1000, 600 , 16 , SDL_SWSURFACE)) == NULL) {
    fprintf(stderr, "oops, could not create surface: %s\n", SDL_GetError());
    exit(1);
  }

  //store the bool values, if it is to be rendered.
  bool toRender[8];  
 
  
  //system time consume for an iterate.
  timeval timeStart,timeStop;

  //variables for modify render-type
  SDL_Event event;
  SDLKey keyName;
  int fold = 1 ;
  bool singleStep = false;
  
  //controls the mode for one player visulisation
  bool leftOff = false; 
  bool rightOff = false;
  
  for( int i = 0 ; i < 8 ; i++ )
    toRender[i] = false;

  //store the attribute's vaules.
  int ballPosX_L,ballPosY_L,racketLeftPosX_L,racketLeftPosY_L,racketRightPosX_L,racketRightPosY_L;
  int ballPosX_R,ballPosY_R,racketLeftPosX_R,racketLeftPosY_R,racketRightPosX_R,racketRightPosY_R;

  // timeintervall between two states.
  long long supposedToWait;

  list<systemState>::iterator iter1,iter2;
  char leftScore_L[100],rightScore_L[100],leftScore_R[100],rightScore_R[100];
  char time[100];
  long long timeToWait = 1;
  
  iter2 = history.begin();
  
  for( iter1 = history.begin(),iter2++ ; iter2 != history.end() ; iter1++,iter2++ ){
    //Check for event
    while( SDL_PollEvent(&event) ){
      //Quits the application
      switch ( event.type ) {
      case SDL_KEYDOWN:
        keyName = event.key.keysym.sym;
        switch( keyName ){
        case SDLK_1:  // control the left player visulisation mode
          leftOff = !leftOff;
          break;
        case SDLK_2:  // control the right player visulisation mode
          rightOff = !rightOff;
          break;
        case SDLK_LEFT: // slow down
          fold = 2;
          singleStep = false;
          break;
        case SDLK_RETURN:  // reset
          singleStep = false;
          fold = 1;
          break;
        case SDLK_SPACE: // single step
          singleStep = true;
          break;
        case SDLK_q:// quit with "q" button
          return;
        default:
          singleStep = false; 
          break;  
        }
        break;
      case SDL_QUIT: // quit
        return;
      default:
        break;
      }
    }
    
    // start to render
    gettimeofday(&timeStart,NULL);
    
    snprintf(time,100,"%f",(*iter1).timestamp);        
    supposedToWait = (long long)( ((*iter2).timestamp - (*iter1).timestamp)*1000000 );
    
    
    boxColor( pScreen , 0 , 0 , 1000 , 600 , 0x000000ff );
    lineColor( pScreen , 0 , 500 ,1000 , 500 ,0xffffffff );
    lineColor( pScreen , 500 , 0 , 500 , 500 , 0xffffff7f );
    
    // use white when rendering is in time, red when it is too slow
    if( timeToWait > 0 ) stringColor( pScreen , 470 , 520 , time , 0Xffffffff );
    else  stringColor( pScreen , 470 , 520 , time , 0Xff0000ff );
    
    for( int i = 0 ; i < 8 ; i++ ){
      if( (*iter1).states[2*i+1] != INFINITY ) toRender[i] = true;
      else toRender[i] = false;
    }
    
    //Initialisation of attributes on the player@2(player_left),
    //when it is valid.
    //Visulisation on player@2(player_left)
    //use color red
    //when the leftOff is on, it doesn't show the visu from left player
    if( !leftOff ){
      //ball
      if( toRender[0] ){
        ballPosX_L = (int)round((*iter1).states[0]);
        ballPosY_L = (int)round((*iter1).states[1]);
        filledCircleColor(pScreen,ballPosX_L,ballPosY_L,4,0xff00007f);
      }
    
      //leftracket
      if( toRender[1] ){
        racketLeftPosX_L = (int)round((*iter1).states[2]);
        racketLeftPosY_L = (int)round((*iter1).states[3])-25;
        boxColor(pScreen,racketLeftPosX_L,racketLeftPosY_L,racketLeftPosX_L+10,racketLeftPosY_L+50,0xff00007f); 
      }
    
      //rightracket
      if( toRender[2] ){
        racketRightPosX_L = (int)round((*iter1).states[4]);
        racketRightPosY_L = (int)round((*iter1).states[5])-25;
        boxColor(pScreen,racketRightPosX_L-10,racketRightPosY_L,racketRightPosX_L,racketRightPosY_L+50,0xff00007f);
      }
    
      //score
      if( toRender[3] ){
        snprintf(leftScore_L,100,"%d",(int)round((*iter1).states[6]));
        snprintf(rightScore_L,100,"%d",(int)round((*iter1).states[7]));
        stringColor(pScreen, 200 , 550 , leftScore_L ,0xff0000ff);
        stringColor(pScreen, 700 , 550, rightScore_L ,0xff0000ff);
      }

    }
    //Initialisation of attributes on the player@3(player_right),
    //when it is valid.
    //Visulisation on player@2(player_left)
    //use color blue
    if( !rightOff ){
      if( toRender[4] ){
        ballPosX_R = (int)round((*iter1).states[8]);
        ballPosY_R = (int)round((*iter1).states[9]);
        filledCircleColor(pScreen,ballPosX_R,ballPosY_R,4,0x0000ff7f);
      }

      if( toRender[5] ){
        racketLeftPosX_R = (int)round((*iter1).states[10]);
        racketLeftPosY_R = (int)round((*iter1).states[11])-25;
        boxColor(pScreen,racketLeftPosX_R,racketLeftPosY_R,racketLeftPosX_R+10,racketLeftPosY_R+50,0x0000ff7f);
      }
    
      if( toRender[6] ){
        racketRightPosX_R = (int)round((*iter1).states[12]);
        racketRightPosY_R = (int)round((*iter1).states[13])-25;
        boxColor(pScreen,racketRightPosX_R-10,racketRightPosY_R,racketRightPosX_R,racketRightPosY_R+50,0x0000ff7f);
      }
    
      if( toRender[7] ){
        snprintf(leftScore_R,100,"%d",(int)round((*iter1).states[14]));
        snprintf(rightScore_R,100,"%d",(int)round((*iter1).states[15]));
        stringColor(pScreen, 240 , 550 , leftScore_R ,0x0000ffff);
        stringColor(pScreen, 740 , 550 , rightScore_R ,0x0000ffff);
      }
    }
    SDL_UpdateRect(pScreen, 0, 0, 1000, 600);
    
    gettimeofday(&timeStop,NULL);

    if( !singleStep ){
      long long timeSpent = (long long)( timeStop.tv_sec*1000000 + timeStop.tv_usec - timeStart.tv_sec*1000000 - timeStart.tv_usec );
      timeToWait = supposedToWait*fold - timeSpent;
      
      if( timeToWait > 0 ) usleep(timeToWait);
    } else {
      bool spacePressed = false;
      do {
        SDL_WaitEvent(&event);
        
        switch(event.type){
        case SDL_KEYDOWN:
          keyName = event.key.keysym.sym;
          switch( keyName ){
          case SDLK_1:  // control the left player visulisation mode
            leftOff = !leftOff;
            break;
          case SDLK_2:  // control the right player visulisation mode
            rightOff = !rightOff;
            break;
          case SDLK_RETURN: // reset
            singleStep = false;
            fold = 1;
            break;
          case SDLK_SPACE:// single step
            spacePressed = true;
            break;
          case SDLK_p:
            spacePressed = true;
            if( iter1 != history.begin() )iter1--;
            if( iter1 != history.begin() )iter1--;
            if( iter2 != history.begin() )iter2--;
            if( iter2 != history.begin() )iter2--;
            break;
          case SDLK_q:// quit with "q" button
            return;
          default:break; 
          }
          break;
        case SDL_QUIT:  //quit
          return;
        }
      } while (!spacePressed && singleStep);
    }
  }

  //Initialisation of attributes on the player@3(player_right).
  snprintf(time,100,"%f",(*iter1).timestamp);        
  boxColor( pScreen , 0 , 0 , 1000 , 600 , 0x000000ff );
  lineColor( pScreen , 0 , 513 ,1000 , 513 ,0xffffffff );
  stringColor( pScreen , 480 , 520 , time , 0Xffffffff );
  
  for( int i = 0 ; i < 8 ; i++ ){
    if( (*iter1).states[2*i-1] != INFINITY ) toRender[i] = true;
    else toRender[i] = false;
  }
  
  //last initialisation of attributes on the player@2(player_left),
  //when it is valid.
  //last visualisation on player@3(player_left)
  //use color red
  if( toRender[0] ){
  ballPosX_L = (int)round((*iter1).states[0]);
  ballPosY_L = (int)round((*iter1).states[1]);
  filledCircleColor(pScreen,ballPosX_L,ballPosY_L,4,0xff00007f);
  }

  if( toRender[1] ){
    racketLeftPosX_L = (int)round((*iter1).states[2]);
    racketLeftPosY_L = (int)round((*iter1).states[3])-25;
    boxColor(pScreen,racketLeftPosX_L,racketLeftPosY_L,racketLeftPosX_L+10,racketLeftPosY_L+50,0xff00007f); 
  }
  
  if( toRender[2] ){
    racketRightPosX_L = (int)round((*iter1).states[4]);
    racketRightPosY_L = (int)round((*iter1).states[5])-25;
    boxColor(pScreen,racketRightPosX_L-10,racketRightPosY_L,racketRightPosX_L,racketRightPosY_L+50,0xff00007f);
  }
  
  if( toRender[3] ){
    snprintf(leftScore_L,100,"%d",(int)round((*iter1).states[6]));
    snprintf(rightScore_L,100,"%d",(int)round((*iter1).states[7]));
    stringColor(pScreen, 200 , 550 , leftScore_L ,0xff0000ff);
    stringColor(pScreen, 700 , 550, rightScore_L ,0xff0000ff);
  }
  
  //last initialisation of attributes on the player@3(player_right),
  //when it is valid.
  //last visulisation on player@2(player_left)
  //use color blue
  if( toRender[4] ){
    ballPosX_R = (int)round((*iter1).states[8]);
    ballPosY_R = (int)round((*iter1).states[9]);
  filledCircleColor(pScreen,ballPosX_R,ballPosY_R,4,0x0000ff7f);
  }
  
  if( toRender[5] ){
    racketLeftPosX_R = (int)round((*iter1).states[10]);
    racketLeftPosY_R = (int)round((*iter1).states[11])-25;
    boxColor(pScreen,racketLeftPosX_R,racketLeftPosY_R,racketLeftPosX_R+10,racketLeftPosY_R+50,0x0000ff7f);
  }
  
  if( toRender[6] ){
    racketRightPosX_R = (int)round((*iter1).states[12]);
    racketRightPosY_R = (int)round((*iter1).states[13])-25;
    boxColor(pScreen,racketRightPosX_R-10,racketRightPosY_R,racketRightPosX_R,racketRightPosY_R+50,0x0000ff7f);
  }
  
  if( toRender[7] ){
    snprintf(leftScore_R,100,"%d",(int)round((*iter1).states[14]));
    snprintf(rightScore_R,100,"%d",(int)round((*iter1).states[15]));
    stringColor(pScreen, 240 , 550 , leftScore_R ,0x0000ffff);
    stringColor(pScreen, 740 , 550 , rightScore_R ,0x0000ffff);
  }
  
  SDL_UpdateRect(pScreen, 0, 0, 1000, 600);
}



int main( int argc , char* argv[] ){
  
  // arguments format;
  // visu_pong start_time end_time files
  
  char* endptr;

  //argumentsnumber control
  if( argc != 19 ){
    cerr << "The arguments' format is wrong:" << endl;
    cerr << "visu_pong start_time end_time 16-omnetpp-files"<<endl;
    return -1;
  }
  
  // the starttime should be a number.
  double start_time = strtod(argv[1],&endptr);
  if( (*argv[1]) == '\0' || (*endptr) != '\0' ){
    cerr << "The starttime is invalid." << endl;
    return -1;
  }

  // the endtime should be a number.
  double end_time = strtod(argv[2],&endptr);
  if( (*argv[2]) == '\0' || (*endptr) != '\0' ){
    cerr << "The endtime is invalid." << endl;
    return -1;
  }

  
  // cout << " start time: " << start_time << endl;
  //cout << "end_time: " << end_time << endl;
  //cout << argv[3] << endl;
  
  FILE* infile[argc-3];
  char data[1000];
  char* pointerToData;

  list<systemState> history;
  list<systemState>::iterator iterToHistory;
  
  //use to filter the first line of every file
  for( int i = 3 ; i < argc ; i++ ){
      
    infile[i-3] = fopen(argv[i],"r");
      
    if( infile[i-3] == NULL ){
      // open failed for some reason...
      cerr <<"unable to open input file: "
           << argv[i] << endl;
      return -1;
    }
    else{
      // ok: read each line of the input file
      // format in first line
      // # vector instance_id host instance_name 1
      // format each line from second one on
      // timestamp value
      fgets(data,1000,infile[i-3]);
    }
  }
 
  // cout << "---------------" << endl;
  //cout << "start_stamp: " << start_time << "\t" << "end_time: "<< end_time << endl;

  bool positionFound=false;
  double timestamp = start_time;
  double value=0;
  
  for( int i = 0 ; i < argc - 3 ; i++ ){
    
    timestamp = start_time;
    iterToHistory = history.begin();

    while( fgets(data,1000,infile[i])!= NULL ){
      pointerToData = NULL;

      // read and store the timestamp of a line per line.
      timestamp = strtod(data,&pointerToData);
      if( timestamp >= start_time ){
        // read and storage the value of the file perline.
        value = strtod(pointerToData,&pointerToData);
        break;
      }
    }
    
    while( fgets(data,1000,infile[i]) != NULL && timestamp <= end_time){
      positionFound = false;

      while( iterToHistory != history.end() && !positionFound ){
        if( (*iterToHistory).timestamp == timestamp ){
          // the struct systemState from the same timestamp exists, storage the
          // value in the according to the index.
          positionFound = true; 
          (*iterToHistory).states[i]=value;
        }
        else {
          if( (*iterToHistory).timestamp > timestamp  ){
            // the struct systemState from the same timestamp doesn't exists,
            // and the next timestamp is later.
            // create a new struct systemState and insert it inbetween.
            struct systemState state;
            state.timestamp = timestamp;
            state.states[i] = value;
            iterToHistory = history.insert(iterToHistory,state);
            positionFound = true;
          }
        }
        iterToHistory++;
      }
      
      if( timestamp <= end_time && !positionFound ){
        // the struct systemState from the same timestamp doesn't exists,
        // and we are in the end of the history.
        // create a new struct systemState and paste it to the end.
        struct systemState state;
        state.timestamp = timestamp;
        state.states[i] = value;
        history.push_back(state);
      }
      
      pointerToData = NULL;
      
      // read and storage the timestamp of the file perline.
      timestamp = strtod(data,&pointerToData);
      // read and storage the value of the file perline.
      value = strtod(pointerToData,&pointerToData);
    }
    
  }
  
  /* for( iterToHistory = history.begin() ; iterToHistory != history.end() ; iterToHistory++ ){
    cout << "timestamp: " << (*iterToHistory).timestamp << endl;
    for( int i = 3 ; i < argc ; i++ ){
      cout << i-2 << "th "<<" value (from "<<argv[i] << "): "
           << (*iterToHistory).states[i-3] << endl;
    }
    cout << endl;
    
    }*/
  
  
  //close all files,
  for( int i = 0 ; i < argc - 3 ; i++  )
    fclose(infile[i]);
  
  visulisation(history);

  // leave
  return 0;
} 
 

