#!/bin/sh
# Copyright (C) 2007, 2008  Condel project, TU-Berlin.
# Written by Sven Grottke, Jiehua Chen, Jan Sablatnig.
# 
# This file is part of Adam, the DVE testbed.
# 
# Adam is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3, or (at your option) any later
# version.
# 
# Adam is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Adam; see the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.
# 
# Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.

# measures the network load
#
# usage: visu_pong [-c] START_TIME END_TIME vector-file
# if -c is given, splitvec will not be called

# try to set CONDEL_ROOT
[ -z "${CONDEL_ROOT}" ] && {
    pushd ../.. 2>&1 1>/dev/null
    CONDEL_ROOT=`pwd`
    export CONDEL_ROOT
    popd 2>&1 1>/dev/null
}

# error function
failure() {
    echo "ERROR: $@"
    echo "aborting"
    exit 1;
}

# get n-th argument
get_nth() {
    shift $1
    echo $1
}

# wrong paramaters
usage() {
    echo "usage: visu_pong.sh [-c] start_time end_time vec-file"
    failure "user stupidity"
}

# at least three argument is required
[ $# -ge 3 ] || usage

# check options
[ $1 = "-c" ] && {
    shift
    CACHE=yes
} || {
    CACHE=no
}

# still, we need at least three arguments
[ $# -ge 3 ] || usage

# set path to splitvec utility
SPLITVEC="${CONDEL_ROOT}/omnetpp/bin/splitvec"

# set some constants from parameters
START_TIME=$1
END_TIME=$2
VEC_FILE=$3
VISU_TOOL="visu_pong"

# sanity checks
[ -r "${VEC_FILE}" ] || failure "${VEC_FILE} is not a readable file"

echo "START_TIME: ${START_TIME}"
echo "END_TIME: ${END_TIME}"

# split the vector file unless $CACHE is set to yes
if [ ${CACHE} != "yes" ]; then
    "${SPLITVEC}" ${VEC_FILE} || failure "splitvec failed"
fi

VEC_BASE_NAME=`basename "${VEC_FILE}" .vec`
VEC_DIR_NAME=`dirname "${VEC_FILE}"`

# process hosts

FILES=""
FOUND_ONE=""
for host in 'pong_[a-z]*\.player@2' 'pong_[a-z]*\.player@3'; do
    for key in ballPosX ballPosY racketLeftPosX racketLeftPosY racketRightPosX racketRightPosY score_left score_right; do
        FOUND_ONE=""
        # find the file that contains data for that host and key
        for file in "${VEC_DIR_NAME}/${VEC_BASE_NAME}-"*.vec; do
            fh=`head -1 ${file}`
            fhost=`get_nth 4 ${fh}`
            echo "${fhost}" | grep -q "\"${host}\"" || continue
            fkey=`get_nth 5 ${fh}`
            echo "${fkey}" | grep -q "\"${key}\"" || continue
            # found it!
            echo "Found ${fkey} on ${fhost} in file ${file}"
            FILES="${FILES} ${file}"
            FOUND_ONE="yes"
        done
        [ -n "$FOUND_ONE" ] || echo "Could not find ${key} on ${host}!"
    done
done

# call it
${VISU_TOOL} "${START_TIME}" "${END_TIME}" ${FILES}
