/*
Copyright (C) 2007, 2008  Condel project, TU-Berlin.
Written by Jiehua Chen, Jan Sablatnig, Sven Grottke.

This file is part of Adam, the DVE testbed.

Adam is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3, or (at your option) any later
version.

Adam is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Adam; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

Contact the Condel project at <http://www.math.tu-berlin.de/condel/>.
*/
//
// omnetpp-vec-files:(view1) queen-pos-x, queen-pos-y, bees...\n"
//                   (view2) ...
//                   ...";
// bees: bee-pos-x, bee-pos-y, bee-score, bee-dead
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <unistd.h>
#include <sys/time.h>

#include <SDL.h>
#include <SDL_gfxPrimitives.h>
#include <math.h>
//#define _XOPEN_SOURCE 500

#include <iostream>
#include <fstream>
#include <string>
#include <list>



#include "visu_swarm.hh"

#define MAX_NR_VIEWS 3
#define LENGTH 1000
#define HEIGHT 500
#define COMMENT_HEIGHT 200

const unsigned long viewColors[MAX_NR_VIEWS] = {0x00ff00, 0xff0000, 0x0000ff};

// If defined, will dump all visualized data to files of this name.
// %d is replaced by the time of that frame
//#define DUMP_FILE "frame_%08.3lf.ppm"

#ifdef DUMP_FILE
#include <SDL_syswm.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#endif


// constructor for the class View
View::View():
  m_nrBees(0),
  m_bees(NULL){
}

// copy constructor for View
View::View(const View& orig):
  m_nrBees(orig.m_nrBees),
  m_queen(orig.m_queen), m_bees(new beeState[m_nrBees]){
  // copy data
  for (int bee = 0; bee < m_nrBees; bee++)
    m_bees[bee] = orig.m_bees[bee];
}

// set number of bees
void View::setNumberOfBees(const int nrBees){
  assert(nrBees > 0);
  m_nrBees = nrBees;
  m_bees.reset(new beeState[nrBees]);
}

// constructor for the class CurrentState
CurrentState::CurrentState(const int nrViews, const int nrBees):
  m_timestamp(0.0),
  m_nrViews(nrViews),
  m_views(new View[nrViews]){

  // init views
  for( int v = 0; v < m_nrViews; v++ ){
    m_views[v].setNumberOfBees(nrBees);
  }
}

// copy constructor for the class CurrentState
CurrentState::CurrentState(const CurrentState& orig):
  m_timestamp(orig.m_timestamp),m_nrViews(orig.m_nrViews),
  m_views(m_nrViews ? new View[m_nrViews] : 0){
//   if( m_nrViews > 0  ){
//     int nrBees = orig.m_views[0].m_nrBees;
//     m_views.reset(new View[m_nrViews](nrBees));
    
    for( int v = 0; v < m_nrViews; v++ ){
      int nrBees = orig.m_views[v].m_nrBees;
      m_views[v].setNumberOfBees(nrBees);
      m_views[v].m_queen = orig.m_views[v].m_queen;
      for( int b = 0; b < nrBees; b++ )
        m_views[v].m_bees[b] = orig.m_views[v].m_bees[b];
    }
//   }
}

std::ostream& operator<<(std::ostream& os, const beeState& b){
  os << "x:" << b.m_x
     << ", y:" << b.m_y
     << ", score:" << b.m_score
     <<", dead? " << b.m_dead << std::endl;
  return os;
}

std::ostream& operator<<(std::ostream& os, const queenState& q){
  os << "x:" << q.m_x
     << ", y:" << q.m_y
     << std::endl;
  return os;
}

std::ostream& operator<<(std::ostream& os, const View& v){
  os << "nrBees: " << v.m_nrBees << "\n";
  for(int b = 0; b < v.m_nrBees; b++)
    os << "b: " << b << ", " << v.m_bees[b] << std::endl;
    os << "queen:" << v.m_queen << std::endl;
  return os;
}

std::ostream& operator<<(std::ostream& os,
                         const CurrentState& cState){
  os << "timestamp:" << cState.m_timestamp << "\n";
  for(int v = 0; v < cState.m_nrViews; v++)
    os << cState.m_views[v] << std::endl;
  return os;
}

// helper-function
void usage() {
  std::cerr << "visu_pong start_time end_time"
            << " nrViews omnetpp-vec-files\n";
  std::cerr << "omnetpp-vec-files:(view1) queen-pos-x,"
            << " queen-pos-y, bees...\n"
            << "                  (view2)...\n                   ...\n"
            << "bees: bee-pos-x, bee-pos-y, bee-score, bee-dead\n";
}

// helper-function
void errorFile(const char* filename){
  // open failed for some reason...
  std::cerr <<"unable to open input file: "
            << filename << std::endl;
}

// helper-function
void error(const char* line){
  std::cerr << line << std::endl;
  exit(1);
}

// helper-function
void writeInstanceValue(const int host_i,
                        const int state_i,
                        const int view_i,
                        const double instance_value,
                        CurrentState& cState){
  if( host_i < 0 ) error("wrong host index!");
  // for host queen
  if( host_i == 0 ){
    switch( state_i ){
    case 0: ((cState.m_views[view_i]).m_queen).m_x
              = instance_value; break;
    case 1: ((cState.m_views[view_i]).m_queen).m_y
              = instance_value; break;
    default: error("wrong queen state!");
    }
  }else{
    // for host bee
    switch( state_i ){
    case 0:
      ((cState.m_views[view_i]).m_bees[host_i-1]).m_x
        = instance_value; break;
    case 1:
      ((cState.m_views[view_i]).m_bees[host_i-1]).m_y
        = instance_value; break;
    case 2:
      ((cState.m_views[view_i]).m_bees[host_i-1]).m_score
        = instance_value; break;
    case 3:
      ((cState.m_views[view_i]).m_bees[host_i-1]).m_dead
        = instance_value; break;
    default: error("wrong bee state"); 
    }
  }// end if
}

static void dump_frame(double ts)
{

#ifdef DUMP_FILE
        const int w = LENGTH, h = HEIGHT+COMMENT_HEIGHT;
        XImage *tim=NULL;

        try {

        // Part A:  Get the image
        // from xv and
        // from http://blog.gustavobarbieri.com.br/2006/11/13/playing-videos-with-mplayer-and-gstreamer-inside-sdl-applications/

        unsigned char *rows[h];

        SDL_SysWMinfo sdl_info;
        SDL_VERSION (&sdl_info.version);
        SDL_GetWMInfo(&sdl_info);
                
//        if (sdl_info.info.x11.window->class != TrueColor)
//            return;
        // Perhaps we need sdl_info.info.x11.lock_func (); ?
        tim= XGetImage(sdl_info.info.x11.display,
                        sdl_info.info.x11.window,
                        0, 0, w, h, AllPlanes, ZPixmap);
        
        if (tim->bitmap_unit != 32)
            return;
        for (int i = 0; i < h; i++)
            rows[i]= (unsigned char *) (tim->data + (i * tim->bytes_per_line));


        

        // Part B:  Write it to file
        // from http://www.cse.ohio-state.edu/~shareef/cse681/labs/writePPM.html
        char file_name[1024];
        snprintf(file_name, 1023, DUMP_FILE, ts);
        file_name[1023]= '\0';
        FILE *fp;
        if (!(fp = fopen(file_name, "wb")))
           return;

        // Write the file header information
        fprintf(fp, "P6\n%d %d\n255\n", w, h );

        // Write the contents of the buffer to file
        for( int iy = 0; iy < h; iy++)
            {
            for( int ix = 0; ix < w; ix++ )
                {
                // Access pixel (ix, iy) by indexing the appropriate row and then column in the
                // image buffer and taking into account that each pixel has
                // three unsigned char values. This command will write a single pixel value (rgb)
                // to the file in binary format.
                fwrite( rows[iy] + ix * 4, 1, 3, fp);
                }
            }
        fclose(fp);

        } catch (...) {
        if (tim) XDestroyImage(tim);
        return; }
        if (tim) XDestroyImage(tim);
        
#else
        ts=0;
#endif
}

// visualise
void visualiseView( SDL_Surface* pScreen,
                    const View& v,
                    const int yPosForScore,
                    unsigned long color ){
  const int nrBees = v.m_nrBees;

  const double x_interval = ((double) LENGTH) / ( nrBees + 1 );
  
  // for queen
  const int queenPosX = (int)v.m_queen.m_x;
  const int queenPosY = (int)v.m_queen.m_y;
  if( queenPosX != INFINITY && queenPosY != INFINITY ){
    filledCircleColor(pScreen, (int)queenPosX, (int)queenPosY,
                      10, color*0x100+0x7f);
    //circleColor(pScreen, (int)queenPosX, (int)queenPosY,
    //                  10, 0xffffffff);
    stringColor(pScreen,(int) queenPosX-2, (int)queenPosY-3,
                "q", 0xffffffff);
  }

  for( int bee_i = 0; bee_i < nrBees; bee_i++ ){
    const int beePosX = (int)v.m_bees[bee_i].m_x;
    const int beePosY = (int)v.m_bees[bee_i].m_y;

    // for bee position
    if( beePosX != INFINITY  && beePosY != INFINITY ){
      filledCircleColor(pScreen, beePosX, beePosY,
                        10, color*0x100+0x7f);
      char beeName[100];
      snprintf(beeName, 100, "%d", bee_i);
      if( bee_i < 10 ){
        stringColor(pScreen, beePosX-2, beePosY-3,
                    beeName, 0xffffffff);
      }else if( bee_i < 100 ){
        stringColor(pScreen, beePosX-6, beePosY-3,
                    beeName, 0xffffffff);
      }else{
        stringColor(pScreen, beePosX-9, beePosY-3,
                    beeName, 0xffffffff);
      }
    }

    // for bee's state, whether it is dead or live
    if( v.m_bees[bee_i].m_dead != 0 )
      circleColor(pScreen, beePosX, beePosY,
                  13, color*0x100+0x7f);
      
    // for bee score
    const double beeScore = v.m_bees[bee_i].m_score;
    if( beeScore != INFINITY ){
      char score[100];
      snprintf(score,100,"%f",beeScore);
      stringColor(pScreen, (int) x_interval*(bee_i+1),
                  yPosForScore, score, color*0x100+0xff);
    }
  }
}

void visualiseHistory( const std::list<CurrentState>& history,
                       const int nrViews ){
  // init SDL lib
  if (SDL_Init(SDL_INIT_VIDEO) < 0) {
    fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
    exit(1);
  }
  atexit(SDL_Quit);
  
  // check list size
  const int historySize = history.size();
  if( historySize == 0 ){
    std::cerr << "no rendering neccessary!\n";
    exit(1);
  }

  const int nrBees = history.front().m_views[0].m_nrBees;
  const double x_interval = LENGTH / ( 1.0 + nrBees );
  const double y_interval = COMMENT_HEIGHT / ( 2.0 + nrViews); 
  const int timePosY = HEIGHT + (int) y_interval - 20;
  const int timePosX = LENGTH / 2 - 30;
  
  // init screen
  SDL_Surface* pScreen;
    
  if( (pScreen = SDL_SetVideoMode(LENGTH, HEIGHT+COMMENT_HEIGHT,
                                  16, SDL_SWSURFACE)) == NULL ){
    fprintf(stderr, "oops, could not create surface: %s\n",
            SDL_GetError());
    exit(1);
  } 
 
  //system time consume for an iterate.
  timeval timeStart,timeStop;

  //variables for modify render-type
  SDL_Event event;
  SDLKey keyName;
  int fold = 1 ;
  bool singleStep = false;
  
  //controls the mode for one player visulisation
  bool leftOff = false; 
  bool rightOff = false;

  // timeintervall between two states.
  long long supposedToWait;

  std::list<CurrentState>::const_iterator iter1,iter2;

  char time[100];
  long long timeToWait = 1;
  
  iter2 = history.begin();
  
  for( iter1 = history.begin(), iter2++;
       iter2 != history.end(); iter1++,iter2++ ){
    // Check for event
    while( SDL_PollEvent(&event) ){
      // Quits the application
      switch ( event.type ){
      case SDL_KEYDOWN:
        keyName = event.key.keysym.sym;
        switch( keyName ){
          //         case SDLK_1:  // control the left player visulisation mode
          //           leftOff = !leftOff;
          //           break;
          //         case SDLK_2:// control the rig;ht player visulisation mode
          //           rightOff = !rightOff;
          //           break;
        case SDLK_LEFT: // slow down
          fold = 2;
          singleStep = false;
          break;
        case SDLK_RETURN:  // reset
          singleStep = false;
          fold = 1;
          break;
        case SDLK_SPACE: // single step
          singleStep = true;
          break;
        case SDLK_q:// quit with "q" button
          return;
        default:
          singleStep = false; 
          break;  
        }
        break;
      case SDL_QUIT: // quit
        return;
      default:
        break;
      }
    }
    
    // start to render
    gettimeofday(&timeStart,NULL);
    
    snprintf(time,100,"%f",(*iter1).m_timestamp);        
    supposedToWait = (long long)
      ( ((*iter2).m_timestamp - (*iter1).m_timestamp)*1000000 );
    
    
    boxColor(pScreen, 0, 0, LENGTH, HEIGHT+COMMENT_HEIGHT, 0x000000ff);
    lineColor(pScreen, 0, HEIGHT, LENGTH, HEIGHT, 0xffffffff);
    
    // use white when rendering is in time, red when it is too slow
    if( timeToWait > 0 )
      stringColor(pScreen, timePosX, timePosY, time, 0Xffffffff);
    else  stringColor(pScreen, timePosX, timePosY , time , 0Xff0000ff);
    
    //Initialisation of attributes on the player_left,
    //when it is valid.
    //Visulisation on player_left
    //use color red
    //when the leftOff is on, it doesn't show the visu from left player

    for( int bee_i = 0; bee_i < nrBees; bee_i++ ){
      char beeName[100];
      snprintf(beeName, 100, "bee%d", bee_i);
      stringColor(pScreen, (int)((bee_i+1)*x_interval),
                  (int)(HEIGHT+2*y_interval-20),
                  beeName, 0Xffffffff);      
    }
    
    for( int view_i = 0; view_i < nrViews; view_i++ ){
      const int yPosForScore = (int) (HEIGHT+(3+view_i)*y_interval-20);
      char viewName[100];
      snprintf(viewName, 100, "view%d", view_i); 
      stringColor(pScreen, 2, yPosForScore,
                  viewName, 0Xffffffff);
      visualiseView(pScreen, iter1->m_views[view_i],
                    yPosForScore, viewColors[view_i]);
    }
     
    
    SDL_UpdateRect(pScreen, 0, 0, LENGTH, HEIGHT+COMMENT_HEIGHT);

    dump_frame(iter1->m_timestamp);
    
    gettimeofday(&timeStop,NULL);

    if( !singleStep ){
      long long timeSpent = (long long)( timeStop.tv_sec*1000000 +
                                         timeStop.tv_usec -
                                         timeStart.tv_sec*1000000 -
                                         timeStart.tv_usec );
      timeToWait = supposedToWait*fold - timeSpent;
      
      if( timeToWait > 0 ) usleep(timeToWait);
    } else {
      bool spacePressed = false;
      do {
        SDL_WaitEvent(&event);
        
        switch(event.type){
        case SDL_KEYDOWN:
          keyName = event.key.keysym.sym;
          switch( keyName ){
          case SDLK_1:  // control the left player visulisation mode
            leftOff = !leftOff;
            break;
          case SDLK_2:  // control the right player visulisation mode
            rightOff = !rightOff;
            break;
          case SDLK_RETURN: // reset
            singleStep = false;
            fold = 1;
            break;
          case SDLK_SPACE:// single step
            spacePressed = true;
            break;
          case SDLK_p:
            spacePressed = true;
            if( iter1 != history.begin() )iter1--;
            if( iter1 != history.begin() )iter1--;
            if( iter2 != history.begin() )iter2--;
            if( iter2 != history.begin() )iter2--;
            break;
          case SDLK_q:// quit with "q" button
            return;
          default:break; 
          }
          break;
        case SDL_QUIT:  //quit
          return;
        }
      } while (!spacePressed && singleStep);
    }
  }

  for( int bee_i = 0; bee_i < nrBees; bee_i++ ){
    char beeName[100];
    snprintf(beeName, 100, "bee%d", bee_i);
    stringColor(pScreen, (int)((bee_i+1)*x_interval)-20,
                (int)(HEIGHT+2*y_interval),
                beeName, 0Xffffffff);      
  }
  
  for( int view_i = 0; view_i < nrViews; view_i++ ){
    const int yPosForScore = (int)(HEIGHT+(3+view_i)*y_interval)-20;
    char viewName[100];
    snprintf(viewName, 100, "view%d", view_i); 
    stringColor(pScreen, 2, yPosForScore,
                viewName, 0Xffffffff);
    visualiseView(pScreen, iter1->m_views[view_i],
                  yPosForScore, viewColors[view_i]);
  }
  
  SDL_UpdateRect(pScreen, 0, 0, LENGTH, HEIGHT+COMMENT_HEIGHT);
}

int main( int argc , char* argv[] ){
  
  // arguments format;
  // visu_pong start_time end_time files
  
  char* endptr;

  int arg = 1;
  //argumentsnumber control
  if( argc <= 4 ){
    std::cerr << "The arguments' format is wrong:\n";
    usage();
    return -1;
  }
  // the starttime should be a number.
  const double start_time = strtod(argv[arg],&endptr);
  if( (*argv[arg++]) == '\0' || (*endptr) != '\0' ){
    std::cerr << "The start time is invalid.\n";
    std::cerr << "See the usage for more details.\n";
    return -1;
  }

  // the endtime should be a number.
  const double end_time = strtod(argv[arg],&endptr);
  if( (*argv[arg++]) == '\0' || (*endptr) != '\0' ){
    std::cerr << "The endtime is invalid.\n";
    std::cerr << "See the usage for more details.\n";
    return -1;
  }
  
  // cout << " start time: " << start_time << endl;
  //cout << "end_time: " << end_time << endl;
  //cout << argv[3] << endl;

  // nrViews is integer
  const int nrViews = strtol(argv[arg], &endptr, 0);
  if( (*argv[arg++] == '\0') || (*endptr != '\0') ){
    std::cerr << "The number of views is invalid.\n";
    std::cerr << "See the usage for more details.\n";
    return -1;
  }
  
  const int nrVecFiles = argc - arg;
  if( nrVecFiles % nrViews ){
    std::cerr << "The omnetpp-vec-files arguments are wrong:\n";
    usage();
    return -1;
  }

  // check number of attributes
  // like view1: queen.posX queen.posY bee1.posX bee1.posY
  // bee1.score bee1.dead
  const int nrAttributes = (int)( nrVecFiles / nrViews );
  if( nrAttributes <= 2 || nrAttributes % 4 != 2 ){
    std::cerr << "nrAttributes per host format is wrong!\n";
    usage();
    return -1;
  }
  
  // check the number of the hosts
  const int nrHosts = ( nrAttributes + 2 ) / 4; 
  if( nrHosts <= 1 ){
    std::cerr << "nrVecFiles: " << nrVecFiles << "\n";
    std::cerr << "nrAttributes: " << nrAttributes << "\n";
    std::cerr << "nrHosts: " << nrHosts << "\n";
    std::cerr << "nrHosts should be greater than 1!\n";
    return -1;
  }
  
  FILE* infile[nrVecFiles];
  char data[1000];
  char* pointerToData;

  std::list<CurrentState> history;
  std::list<CurrentState>::iterator iterToHistory;
  
  //use to filter the first line of every file
  for( int i = 0; i < nrVecFiles; i++ ){
    infile[i] = fopen(argv[i+arg],"r");
    if( infile[i] == NULL ){
      errorFile(argv[i+arg]);
      return -1;
    }
    // ok: read each line of the input file
    // format in first line
    // # vector instance_id host instance_name 1
    // format each line from second one on
    // m_timestamp value
    fgets(data, 1000, infile[i]);
  }
 
  // cout << "---------------" << endl;
  // cout << "start_stamp: " << start_time << "\t"
  //      << "end_time: "<< end_time << endl;

  bool positionFound=false;
  double timestamp = start_time;
  double instance_value = 0.0;
  
  for( int f = 0 ; f < nrVecFiles; f++ ){
    // set the the host- and view-indexes in currentState
    // now we have 4 attributes per bee
    const int view_i = f / nrAttributes;
    const int attrInView = f % nrAttributes;
    const int host_i = (int) ((attrInView + 2) / 4); 
    int state_i;
    if( attrInView <= 1 ){
      // queen's attributes
      state_i = attrInView;
    }else{
      // bees' attributes
      state_i = (attrInView + 2) % 4;
    }
    
    timestamp = start_time;
    iterToHistory = history.begin();
    
    while( fgets(data,1000,infile[f])!= NULL ){
      pointerToData = NULL;

      // read and store the timestamp per line.
      timestamp = strtod(data,&pointerToData);
      if( timestamp >= start_time ){
        // read and storage the instance value of the file perline.
        instance_value = strtod(pointerToData,&pointerToData);
        break;
      }
    }
    
    while( fgets(data,1000,infile[f]) != NULL
           && timestamp <= end_time){
      positionFound = false;

      while( iterToHistory != history.end() && !positionFound ){
        if( iterToHistory->m_timestamp == timestamp ){
          // the object CurrentState from the same timestamp exists,
          // storage the value in the according to the index.
          positionFound = true;
          writeInstanceValue(host_i, state_i, view_i,
                             instance_value, *iterToHistory);
          //std::cerr << "found state\n";
        }else {
          if( iterToHistory->m_timestamp > timestamp  ){
            // the cState from the same timestamp
            // doesn't exists,
            // and the next timestamp is later.
            // create a new object CurrentState and
            // insert it inbetween.
            CurrentState cState(nrViews, nrHosts-1);
            cState.m_timestamp = timestamp;
            iterToHistory = history.insert(iterToHistory, cState);
            writeInstanceValue(host_i, state_i, view_i,
                               instance_value, *iterToHistory);
            positionFound = true;
            //std::cerr << "inserting into gap\n";
          }
        }
        iterToHistory++;
      }
      
      if( timestamp <= end_time && !positionFound ){
        // the cState from the same timestamp
        // doesn't exists,
        // and we are in the end of the history.
        // create a new object CurrentState and paste it to the end.
        //std::cerr << "inserting new state\n";
        CurrentState cState(nrViews, nrHosts-1);
        cState.m_timestamp = timestamp;
        writeInstanceValue(host_i, state_i, view_i,
                           instance_value, cState);
        history.push_back(cState);
      }
      
      pointerToData = NULL;
      
      // read and storage the timestamp of the file perline.
      timestamp = strtod(data,&pointerToData);
      // read and storage the value of the file perline.
      instance_value = strtod(pointerToData,&pointerToData);
    }

    //std::cerr << "finished reading vecfile, size of history: " << history.size() << std::endl;
  }
  
  /* for( iterToHistory = history.begin() ; iterToHistory != history.end() ; iterToHistory++ ){
    cout << "timestamp: " << (*iterToHistory).m_timestamp << endl;
    for( int i = 3 ; i < argc ; i++ ){
      cout << i-2 << "th "<<" value (from "<<argv[f] << "): "
           << (*iterToHistory).m_views[i-3] << endl;
    }
    cout << endl;
    
    }*/
  
  
  //close all files,
  for( int f = 0 ; f < nrVecFiles ; f++  )
    fclose(infile[f]);

//   for(iterToHistory = history.begin();
//       iterToHistory != history.end();
//       iterToHistory++){
//     std::cout << *iterToHistory << "\n";
//   }

  visualiseHistory(history, nrViews);

  // leave
  return 0;
} 
 

